Package: MetaNet
Type: Package
Title: Network Analysis for Omics Data
Version: 0.3.2
Authors@R: 
    person("Chen", "Peng", email ="bfzede@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9449-7606"))
Description: Comprehensive network analysis package.
    Calculate correlation network fastly, accelerate lots of analysis by parallel computing.
    Support for multi-omics data, search sub-nets fluently.
    Handle bigger data, more than 10,000 nodes in each omics.
    Offer various layout method for multi-omics network and some interfaces to other software ('Gephi', 'Cytoscape', 'ggplot2'), easy to visualize.
    Provide comprehensive topology indexes calculation, including ecological network stability.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0), igraph (>= 1.3.5)
LazyData: true
Imports: graphics, dplyr, ggplot2 (>= 3.2.0), ggnewscale, ggrepel,
        grDevices, magrittr, reshape2, stats, tibble, utils, pcutils
        (>= 0.2.7), rlang
Suggests: pheatmap, vegan, stringr, foreach, doSNOW, snow, knitr,
        rmarkdown, prettydoc, Hmisc, gifski, ggraph, networkD3,
        ggpmisc, ggtree, treeio, circlize, jsonify, ggpubr, corrplot,
        philentropy, spatstat.random, spatstat.geom, sf
VignetteBuilder: knitr
BugReports: https://github.com/Asa12138/MetaNet/issues
URL: https://github.com/Asa12138/MetaNet
ByteCompile: true
biocViews: DataImport, Network analysis, Omics, Software, Visualization
NeedsCompilation: no
Packaged: 2026-05-26 02:59:40 UTC; asa
Author: Chen Peng [aut, cre] (ORCID: <https://orcid.org/0000-0002-9449-7606>)
Maintainer: Chen Peng <bfzede@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-26 03:10:03 UTC
Built: R 4.5.2; ; 2026-05-26 04:37:24 UTC; unix

Package: MetaNet
Type: Package
Title: Network Analysis for Omics Data
Version: 0.3.2
Authors@R: 
    person("Chen", "Peng", email ="bfzede@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9449-7606"))
Description: Comprehensive network analysis package.
    Calculate correlation network fastly, accelerate lots of analysis by parallel computing.
    Support for multi-omics data, search sub-nets fluently.
    Handle bigger data, more than 10,000 nodes in each omics.
    Offer various layout method for multi-omics network and some interfaces to other software ('Gephi', 'Cytoscape', 'ggplot2'), easy to visualize.
    Provide comprehensive topology indexes calculation, including ecological network stability.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0), igraph (>= 1.3.5)
LazyData: true
Imports: graphics, dplyr, ggplot2 (>= 3.2.0), ggnewscale, ggrepel,
        grDevices, magrittr, reshape2, stats, tibble, utils, pcutils
        (>= 0.2.7), rlang
Suggests: pheatmap, vegan, stringr, foreach, doSNOW, snow, knitr,
        rmarkdown, prettydoc, Hmisc, gifski, ggraph, networkD3,
        ggpmisc, ggtree, treeio, circlize, jsonify, ggpubr, corrplot,
        philentropy, spatstat.random, spatstat.geom, sf
VignetteBuilder: knitr
BugReports: https://github.com/Asa12138/MetaNet/issues
URL: https://github.com/Asa12138/MetaNet
ByteCompile: true
biocViews: DataImport, Network analysis, Omics, Software, Visualization
NeedsCompilation: no
Packaged: 2026-05-26 02:59:40 UTC; asa
Author: Chen Peng [aut, cre] (ORCID: <https://orcid.org/0000-0002-9449-7606>)
Maintainer: Chen Peng <bfzede@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-26 03:10:03 UTC
Built: R 4.6.0; ; 2026-05-26 07:38:57 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,cohesion)
S3method(plot,ggig)
S3method(plot,metanet)
S3method(plot,metanet_compare)
S3method(plot,rmt_res)
S3method(plot,robust)
S3method(plot,robustness)
S3method(plot,vulnerability)
S3method(print,cohesion)
S3method(print,coors)
S3method(print,corr)
S3method(print,ggig)
S3method(print,metanet)
S3method(print,metanet_compare)
S3method(print,robust)
S3method(print,robustness)
S3method(print,vulnerability)
S3method(summary,corr)
export("%<>%")
export("%>%")
export(Cohesion)
export(RMT_threshold)
export(adjacency_similarity)
export(anno_edge)
export(anno_vertex)
export(as.ggig)
export(as.metanet)
export(as_arc)
export(as_circle_tree)
export(as_coors)
export(as_line)
export(as_multi_layer)
export(as_poly_sector)
export(as_polyarc)
export(as_polycircle)
export(as_polygon)
export(c_net_annotate)
export(c_net_build)
export(c_net_cal)
export(c_net_calculate)
export(c_net_compare)
export(c_net_difference)
export(c_net_ego)
export(c_net_filter)
export(c_net_from_edgelist)
export(c_net_highlight)
export(c_net_index)
export(c_net_intersect)
export(c_net_lay)
export(c_net_layout)
export(c_net_load)
export(c_net_module)
export(c_net_neighbors)
export(c_net_plot)
export(c_net_save)
export(c_net_set)
export(c_net_skeleton)
export(c_net_stability)
export(c_net_union)
export(c_net_update)
export(cal_sim)
export(check_tabs)
export(clean_igraph)
export(clean_multi_edge_metanet)
export(combine_n_module)
export(compare_rand)
export(df2net_tree)
export(extract_sample_net)
export(fast_cor)
export(filter_n_module)
export(fit_power)
export(g_layout)
export(g_layout_backbone)
export(g_layout_circlepack)
export(g_layout_multi_layer)
export(g_layout_nice)
export(g_layout_poly_sector)
export(g_layout_polyarc)
export(g_layout_polycircle)
export(g_layout_polygon)
export(g_layout_stress)
export(g_layout_treemap)
export(get_big_lay_nice)
export(get_community)
export(get_e)
export(get_group_skeleton)
export(get_module)
export(get_module_eigen)
export(get_n)
export(get_v)
export(input_cytoscape)
export(input_gephi)
export(is_metanet)
export(links_stat)
export(module_detect)
export(module_eigen)
export(module_expression)
export(module_net)
export(multi_net_build)
export(nc)
export(netD3plot)
export(net_par)
export(olympic_rings_net)
export(p.adjust.table)
export(plot_corr_heatmap)
export(plot_e_type_bar)
export(plot_module_tree)
export(plot_multi_nets)
export(plot_net_degree)
export(rand_net)
export(rand_net_par)
export(read_corr)
export(rmt)
export(robust_test)
export(robustness)
export(save_corr)
export(show_MetaNet_logo)
export(skeleton_plot)
export(smallworldness)
export(spatstat_layout)
export(summ_2col)
export(summary_module)
export(to_module_net)
export(transform_coors)
export(twocol_edgelist)
export(venn_net)
export(vulnerability)
export(zp_analyse)
export(zp_plot)
import(ggplot2)
import(igraph)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,symbols)
importFrom(graphics,text)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(pcutils,change_fac_lev)
importFrom(pcutils,get_cols)
importFrom(pcutils,guolv)
importFrom(pcutils,hebing)
importFrom(pcutils,lib_ps)
importFrom(pcutils,list_to_dataframe)
importFrom(pcutils,mmscale)
importFrom(pcutils,tidai)
importFrom(pcutils,trans)
importFrom(pcutils,update_param)
importFrom(reshape2,acast)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,approx)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,kmeans)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pt)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,smooth.spline)
importFrom(stats,time)
importFrom(stats,var)
importFrom(tibble,column_to_rownames)
importFrom(tibble,rownames_to_column)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,packageName)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,write.csv)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# MetaNet <img src="man/figures/MetaNet.png" align="right" width="120" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/Asa12138/MetaNet/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Asa12138/MetaNet/actions/workflows/R-CMD-check.yaml)
[![](https://img.shields.io/badge/doi-10.1101/2025.06.26.661636-yellow.svg)](https://doi.org/10.1101/2025.06.26.661636)
[![](https://img.shields.io/badge/blog-@asa-blue.svg)](https://asa-blog.netlify.app/)
[![](http://cranlogs.r-pkg.org/badges/grand-total/MetaNet)](https://cran.r-project.org/package=MetaNet)
[![](http://cranlogs.r-pkg.org/badges/last-month/MetaNet)](https://cran.r-project.org/package=MetaNet)
[![](https://www.r-pkg.org/badges/version/MetaNet?color=green)](https://cran.r-project.org/package=MetaNet)
[![](https://img.shields.io/badge/devel%20version-0.3.1-green.svg)](https://github.com/Asa12138/MetaNet)

<!-- badges: end -->

MetaNet, a high-performance R package that unifies network construction,
visualization, and analysis across diverse omics layers.

MetaNet enables fast and scalable correlation-based network construction
for datasets with more than 10,000 features, providing over 40 layout
algorithms, rich annotation utilities, and visualization options
compatible with both static and interactive platforms. It further offers
comprehensive topological and stability metrics for in-depth network
characterization. Benchmarking shows that MetaNet delivers up to a
100-fold improvement in computation time and a 50-fold reduction in
memory usage compared to existing R packages.

![](man/figures/MetaNet_GA.jpg)

The HTML documentation of the latest version is available at [Github
page](https://asa12138.github.io/MetaNet/).

- [Installation](#installation)
- [Tutorial](#tutorial)
- [Quick Start](#quick-start)
- [Detailed usage](#detailed-usage)
  - [Overview](#overview)
    - [Network construction](#network-construction)
    - [Network annotation and
      filtering](#network-annotation-and-filtering)
    - [Module detection](#module-detection)
    - [Network comparison](#network-comparison)
  - [Layout and Visualization](#layout-and-visualization)
    - [Basic layouts](#basic-layouts)
    - [spatstat layouts](#spatstat-layouts)
    - [Group and multi-level layouts](#group-and-multi-level-layouts)
  - [Biological network](#biological-networks)
    - [Venn diagram](#venn)
    - [Tree diagram](#tree)
    - [Pie node](#pie-node)
    - [SRTING database](#string-database)
    - [miRNA-Gene networks](#mirna-gene-networks)
    - [Enrichment](#enrichment)
    - [KEGG pathway](#kegg-pathway)

## Citation

Please cite:

1.  Peng, C. et al. MetaNet: a scalable and integrated tool for
    reproducible omics network analysis. 2025.06.26.661636 Preprint at
    <https://doi.org/10.1101/2025.06.26.661636> (2025).

## Installation

You can install the released version of `MetaNet` from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("MetaNet")
```

You can install the development version of `MetaNet` from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("Asa12138/MetaNet")
```

## Tutorial

Please go to **<https://bookdown.org/Asa12138/metanet_book/>** for the
full vignette.

<img src="man/figures/cover1.jpeg" width="250" />

MetaNet is an R-based integrative package designed for comprehensive
network analysis across diverse omics data, including multi-omics
datasets. MetaNet is compatible with operating systems (Windows, macOS,
and Linux) that support R version 4.0 or higher, and its core
functionality is built upon the widely used igraph package. Its
architecture comprises several core functional modules: Calculation,
Manipulation, Layout, Visualization, Topology analysis, Module analysis,
Stability analysis, and I/O (Figure 1A), supporting the end-to-end
analytical process from network construction to visualization. Figure 1B
illustrates the main workflow and essential components within MetaNet.

Pairwise correlation computation is central to most network-based omics
tools, but the growing scale of omics datasets imposes substantial
computational demands. MetaNet addresses this through optimized
vectorized matrix algorithms for calculating correlation coefficients
and corresponding p-values, greatly reducing memory use and runtime
(Figure 1D).

<img src="man/figures/fig1.jpeg"  width="500" />

**Figure 1. Overview of the MetaNet workflow and its high-efficiency
computation.**

## Quick Start

Simply build and draw a co-occurrence network plot, only need to use
`c_net_calculate()`, `c_net_build()`, `c_net_plot()` three functions:

``` r
library(pcutils)
library(MetaNet)
data("otutab", package = "pcutils")

# Inter-species correlation coefficients were calculated after transposition
totu <- t2(otutab[1:70, ])

cor <- c_net_calculate(totu)
net <- c_net_build(cor, r_threshold = 0.65)
#> Have not do p-value adjustment! use the p.value to build network.
c_net_plot(net)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="" width="100%" />

Here is a more complex example of building and visualizing a multi-omics
network with MetaNet, we can use `c_net_set` to flexibly set vertex
attributes such as class and size, and then visualize the network with
`c_net_plot()`:

``` r
data("multi_test")
multi1 <- multi_net_build(list(Microbiome = micro, Metabolome = metab, Transcriptome = transc))
#> All samples matched.
#> All features are OK.
#> Calculating 18 samples and 150 features of 3 groups.
#> Have not do p-value adjustment! use the p.value to build network.
multi1 <- c_net_set(multi1, micro_g, metab_g, transc_g,
  vertex_class = c("Phylum", "kingdom", "type")
)
multi1 <- c_net_set(multi1, data.frame("Abundance1" = colSums(micro)),
  data.frame("Abundance2" = colSums(metab)), data.frame("Abundance3" = colSums(transc)),
  vertex_size = paste0("Abundance", 1:3)
)
c_net_plot(multi1)
```

<img src="man/figures/README-unnamed-chunk-4-1.png" alt="" width="100%" />

For more detailed usage, please refer to the sections below or the full
vignette at **<https://bookdown.org/Asa12138/metanet_book/>**.

## Detailed usage

### Overview

#### Network construction

We use a multi-omics dataset to demonstrate the powerful network
manipulation capabilities of MetaNet. The dataset includes three omics
layers: microbiome, metabolome, and transcriptome, along with their
corresponding metadata.

``` r
data("multi_test")
v_color=setNames(c("#b2df8a","#cab2d6","#a6bce3","#fdbf6f","#fb9a99","#1f78b4"),
                 c("Actinobacteria","Proteobacteria","Benzenoids","Hydrocarbons","Cell motility","Immune system"))

# We create a simulated multi-omics dataset for demonstration purposes:
micro_g=micro_g[micro_g$Phylum%in%c("p__Actinobacteria", "p__Proteobacteria"),]
micro_g$Phylum=gsub(".__","",micro_g$Phylum)
micro=micro[,rownames(micro_g)]
metab_g$kingdom=tidai(metab_g$kingdom,c("Benzenoids","Hydrocarbons"))
transc_g$type=tidai(transc_g$type,c("Cell motility","Immune system"))
```

We first build a multi-omics network using `multi_net_build()` with a
specified correlation threshold, which will automatically calculate the
correlations and construct the network across all omics layers:

``` r
multi1 <- multi_net_build(list(Microbiome = micro,
                               Metabolome = metab,
                               Transcriptome = transc),r_threshold = 0.6)

# Figure 2A
c_net_plot(multi1,legend = F,main="",vertex.color=v_color)
```

#### Network annotation and filtering

In omics and multi-omics studies, networks are often annotated with
external data such as abundance profiles, taxonomy, or clinical
metadata. The `c_net_set` function attaches multiple annotation tables
to a network object and automatically configures visualization
properties (Figure 2B), including color schemes, line types, node
shapes, and legends.

``` r
multi1_with_anno <- c_net_set(multi1, micro_g, metab_g, transc_g,
                    vertex_class = c("Phylum", "kingdom", "type")
)
multi1_with_anno <- c_net_set(multi1_with_anno, 
                    data.frame("Abundance1" = colSums(micro)),
                    data.frame("Abundance2" = colSums(metab)), 
                    data.frame("Abundance3" = colSums(transc)),
                    vertex_size = paste0("Abundance", 1:3)
)
```

``` r
# Figure 2B
c_net_plot(multi1_with_anno,legend_number = T,vertex.color=v_color,
           edge.lty=c(4,1),vertex_size_range = c(4,9),
           lty_legend = T,size_legend = T,legend_cex = 1.2
           )
```

After annotation and customization, researchers may focus on specific
network regions—especially in multi-omics integration. The
`c_net_filter` function extracts sub-networks using flexible filters
(Figure 2C), while `c_net_highlight` visually emphasizes selected nodes
or edges (Figure 2D).

``` r
# filter the network to only show intra-omics correlations within microbiome and metabolome layers:
multi2 <- c_net_filter(multi1_with_anno, v_group %in% c("Microbiome","Metabolome")) %>% 
          c_net_filter(., e_class=="intra", mode="e")

# Figure 2C
c_net_plot(multi2,legend = F,main="",edge.lty=1,vertex.color=v_color)
```

``` r
degree(multi1)%>%sort(decreasing = T)%>%head()
core=c_net_neighbors(multi1_with_anno,"s__Dongia_mobilis")
c_net_highlight(multi1_with_anno,V(core)$name)->multi1_highlight
V(multi1_highlight)$label=ifelse(V(multi1_highlight)$label=="s__Dongia_mobilis",
                                 "s__Dongia_mobilis",NA)
# Figure 2D
c_net_plot(multi1_highlight,tmp_gephi$coors,legend = F,main="",
           labels_num = "all")
```

#### Module detection

Modules or communities—densely connected subgraphs—often represent
biologically meaningful groups. MetaNet supports module detection
through `c_net_module`, which includes multiple community detection
algorithms (Figure 2E).

``` r
c_net_module(multi1_with_anno)->multi1_module

g_layout_treemap(multi1_module)->coors1
c_net_plot(multi1_module,coors1,legend = F,main="",
           plot_module = T,mark_module = T,vertex.color=get_cols())
```

Resulting modules can be visualized with chord or Sankey diagrams to
show proportions and inter-module connections (Figure 2F). For
group-level analysis, the `c_net_skeleton` function summarizes edge
origins and targets across conditions, enhancing interpretability in
multi-condition or longitudinal datasets (Figure 2G).

``` r
links_stat(multi1_module,"module")

c_net_skeleton(multi2)%>%
  skeleton_plot(vertex.color=v_color,split_e_type = F,coors=in_circle(),
                legend=F,main="")
```

<img src="man/figures/fig2.jpeg"  width="500" />

**Figure 2. MetaNet supports flexible and intuitive network
manipulation.**

#### Network comparison

Comparative analysis across multiple networks is also critical.
Researchers may identify differential edges between groups or track
stable subnetworks across transitions. MetaNet enables such comparisons
by computing intersections, unions, and differences between networks
(Figure 2H), offering a flexible framework for comparative and
evolutionary network analysis.

``` r
library(igraph)
set.seed(123)
g1 <- make_graph("Icosahedron")
V(g1)$color <- "#4DAF4A77"
E(g1)$color <- "#4DAF4A77"

g1=as.metanet(g1)

g2 <- make_graph("Octahedron")
V(g2)$name=as.character(9:14)
V(g2)$color <- "#984EA366"
E(g2)$color <- "#984EA366"

g2=as.metanet(g2)

# Perform network operations
g_union <- c_net_union(g1, g2)
E(g_union)$color<-"orange"
g_inter <- c_net_intersect(g1, g2)
g_diff <- c_net_difference(g1, g2)

par_ls=list(main = "",legend = F,vertex_size_range = c(20,20))
c_net_plot(g1, params_list = par_ls)
c_net_plot(g2, params_list = par_ls,coors = in_circle())
c_net_plot(g_union, params_list = par_ls,
           coors = transform_coors(c_net_layout(g_union),rotation = 90))
c_net_plot(g_inter , params_list = par_ls)
c_net_plot(g_diff, params_list = par_ls)
```

### Layout and Visualization

#### Basic layouts

Layout is a critical component of network visualization, as a
well-designed layout can significantly enhance the interpretability of
network structures. MetaNet stores layout coordinates in a flexible
`coors` object, allowing users to control, reuse, and transfer layout
settings. The `c_net_layout` function provides access to over 40 layout
algorithms (Figure 3A), including several new layouts as well as
adaptations from `igraph` and `ggraph` packages.

``` r
net=make_graph("Zachary")
net=as.metanet(net)
V(net)$color=rep(get_cols(6),length=length(net))

layout_methods <- list(
  with_fr(), in_circle(), on_grid(), randomly(), as_tree(),
  with_gem(), with_graphopt(), with_kk(), with_mds(), 
  as_line(angle = 45), as_arc(), as_polygon(), as_polycircle(3), as_circle_tree(),
  as_multi_layer(2)
)
names(layout_methods) <- c(
  "with_fr", "in_circle", "on_grid", "randomly","as_tree", 
  "with_gem", "with_graphopt", "with_kk","with_mds", 
  "as_line", "as_arc","as_polygon", "as_polycircle", "as_circle_tree",
  "as_multi_layer"
)

layout_methods2 <- c(
  'backbone', 'dendrogram', 'eigen', 'focus', 'hive', 
  'stress', 'unrooted', "cactustree", "fabric")

par(mfrow = c(4, 6),mar = c(0.5, 0.5, 0.5, 0.5))
for (i in names(layout_methods)) {
  c_net_plot(net, coors = layout_methods[[i]], 
             edge.color="grey",vertex_size_range = c(10,10),edge_width_range = c(0.8,0.8),
             legend = F, main = i, labels_num = 0,
             rescale = (i=="as_tree"))
}

for (i in layout_methods2) {
  if(i=="focus")coors = c_net_layout(clean_igraph(net),"focus",focus=4)
  else if(i=="hive")coors=c_net_layout(clean_igraph(net),"hive",axis=1:5)
  else if (i=="pmds")coors=c_net_layout(clean_igraph(net),"pmds",pivots=15)
  else coors = c_net_layout(
    clean_igraph(net,direct = i%in%c("dendrogram","partition","treemap","cactustree")),
    i)
  c_net_plot(net, coors = coors, 
             edge.color="grey",vertex_size_range = c(10,10),edge_width_range = c(0.8,0.8),
             legend = F, main = i, labels_num = 0,
             rescale = T)
}
```

<img src="man/figures/fig3.jpeg"  width="500" />

**Figure 3. MetaNet enables diverse and powerful network layout
strategies.**

#### spatstat layouts

In addition to conventional layouts, MetaNet introduces the
`spatstat_layout` method, which constrains layout generation within a
user-defined polygon or along its edges. This layout function supports
uniform or random node distributions inside custom shapes. For example,
arranging a network within a star (Figure 3B) or mapping it to a
geographic region like Australia (Figure 3C).

``` r
library(spatstat.geom)

create_star_window <- function(r_outer = 1, r_inner = 0.4, center = c(0, 0)) {
  # 创建五角星的10个顶点（外、内交替）
  theta <- seq(0, 2 * pi, length.out = 11)[-11]  # 10个点
  theta_outer <- theta[seq(1, 10, 2)]
  theta_inner <- theta[seq(2, 10, 2)]

  x <- c(r_outer * cos(theta_outer),
         r_inner * cos(theta_inner))
  y <- c(r_outer * sin(theta_outer),
         r_inner * sin(theta_inner))

  # 重新排序成首尾相连的路径
  order_index <- c(1,6,2,7,3,8,4,9,5,10)
  x <- x[order_index] + center[1]
  y <- y[order_index] + center[2]

  # 构建 spatstat 的 owin 窗口
  win <- owin(poly = list(x = x, y = y))
  return(win)
}

win_star <- create_star_window()

tmp_net=erdos.renyi.game(400,p.or.m = 0.005)%>%as.metanet()

c_net_plot(co_net, coors = spatstat_layout(co_net, win_star, order_by="v_class"),
     legend=F,edge.color="grey",main="spatstat_layout",edge.width=0.3)
```

#### Group and multi-level layouts

For networks with grouping variables, MetaNet offers an advanced
interface via `g_layout`. Users can define spatial configurations for
each group, including positioning, scaling, and internal layout
strategies, and combine multiple layout types in one visualization.

The resulting `coors` object can be nested or recombined with subsequent
calls to create highly customized multi-level layouts. For example, a
co-abundance network across multiple human body sites can be arranged
with a single `g_layout` call (Figure 3D). This strategy is also useful
for highlighting modular structures. `g_layout_circlepack` visualizes
module distribution using compact circular packing (Figure 3E), while
`g_layout_multi_layer` introduces a pseudo-3D representation emphasizing
inter-module relationships (Figure 3F).

``` r
set.seed(12)
igraph::sample_islands(4,40,0.15,3)%>%as.metanet()%>%c_net_module()->test_net
test_net=to_module_net(test_net)

V(test_net)$v_class=sample(letters[1:4],size = length(test_net),replace = T)
test_net=c_net_set(test_net)
c_net_plot(test_net,plot_module = F)

g_coors=g_layout(test_net,group = "module",
                 layout1 = data.frame(X=c(0,0.3,0.3,0),Y=c(0,2:4)),
                 layout2 = list(with_fr(),on_grid(),as_polycircle(3),as_polygon(3)),
                 zoom2 = c(1.3,1,1,1.3)*3
                 )

img <-png::readPNG("body.png")

# Figure 3D
pdf("3.g_layout.pdf",height = 8,width = 5)
par(xpd = TRUE)
plot(NA, xlim = c(-1, 1), ylim = c(-1, 1), axes = F, asp = 1,xlab = "", ylab = "")
rasterImage(bg_img, -1, -1.2, -0.4, 1.2,interpolate = F)
par(new = TRUE)
c_net_plot(test_net,coors = g_coors,plot_module = F,legend = F,main="",vertex.size=2)

segments(
  x0 = c(-0.52,-0.3,-0.42,-0.6), y0 = c(0.1,0.1,0.5,0.78),
  x1 = c(-0.2,-0.1,-0.12,-0.25), y1 = c(-0.63,0,0.45,0.8),
  col = "black", lwd = 1.5
)
dev.off()
```

``` r
set.seed(12)
E(co_net)$color <- rep("grey", length(E(co_net)))
coors=g_layout_circlepack(multi1_module, group = "module")

# Figure 3E
pdf("3.g_layout_circlepack.pdf",height = 7,width = 10)
c_net_plot(multi1_module, 
           coors = transform_coors(coors,rotation = 130),edge.lty=1,edge.width=0.5,
           legend = F, labels_num = 0, main = "g_layout_circlepack",
           plot_module = T,mark_module = F,vertex_size_range = c(3,8))
dev.off()
```

``` r
set.seed(112)
igraph::sample_islands(3,30,0.15,0)%>%as.metanet()%>%c_net_module()->test_net

test_net=to_module_net(test_net)
test_net=add_edges(test_net,c(25,61,25,63,25,75,25,88,25,79,25,66))
test_net2=add_edges(test_net,c(4,56,4,57,4,60,4,48,4,51,4,55,4,32))

get_e(test_net2)->tmp_e
tmp_e$width=1
tmp_e$lty=1
tmp_e$color=ifelse(is.na(tmp_e$color),"#FA789A",tmp_e$color)

edge.attributes(test_net2)=as.list(tmp_e)
V(test_net2)$size=V(test_net2)$degree=degree(test_net2)

# Figure 3F
pdf("3.multi_layer.pdf",height = 5)
plot(as.metanet(test_net2),
  coors = g_layout_multi_layer(test_net, group = "v_class", layout = on_grid()),
  legend = F, labels_num = 0, main = "g_layout_multi_layer",
  edge.curved=ifelse(is.na(tmp_e$e_type),0.2,0)
)
dev.off()
```

### Biological networks

MetaNet provides native support for a variety of specialized network
types frequently used in bioinformatics workflows, enabling researchers
to visualize and explore biological relationships beyond conventional
correlation or interaction networks.

#### Venn

MetaNet allows the construction of Venn-style networks to illustrate set
relationships across sample groups. These provide a more informative
alternative to traditional Venn diagrams by displaying explicit
connections and network structure (Figure 4A).

``` r
data(otutab, package = "pcutils")
tab <- otutab[420:485, 1:3]
venn_net(tab) -> v_net

# Figure 4A
pdf("3-1.venn_network.pdf",width = 5,height = 5)
plot(v_net, vertex_size_range=list("Group" = c(18, 18),"elements" = c(4, 4)),
     edge.width=.5,vertex.frame.width=0.3)
dev.off()
```

#### Tree

Tree-structured data, such as taxonomies or gene ontology hierarchies,
can be visualized using the built-in “as_circle_tree” layout, offering a
clear and compact representation of hierarchical relationships (Figure
4B).

``` r
data("otutab", package = "pcutils")
cbind(taxonomy, num = rowSums(otutab))[1:20, ] -> test
df2net_tree(test) -> ttt

# Figure 4B
pdf("3-1.circle_tree_network.pdf",width = 5,height = 5)
plot(ttt, coors = as_circle_tree(), legend = F, main = "Circle tree network",
  edge.arrow.size = 0.3,edge.arrow.width = 0.6,rescale=T,vertex.label=ifelse(
    V(ttt)$v_class %in% c("Species"), V(ttt)$name, NA),edge.color="black",edge.width=0.4
  )
dev.off()
```

#### Pie node

MetaNet further supports pie-node visualization, where each node encodes
multivariate annotations, such as group-specific abundances. This
approach allows compositional data to be embedded directly in the
network structure (Figure 4C).

``` r
data("otutab")
data("c_net")
hebing(otutab, metadata$Group) -> otutab_G

V(co_net)$degree=degree(co_net)
co_net_f=c_net_filter(co_net,degree>6,degree<15)

# Figure 4C
pdf("3-1.pie_network.pdf",width = 5,height = 5)
c_net_plot(co_net_f,pie_value=otutab_G,vertex.shape=c("pie"),
    pie_legend=T,color_legend=F,vertex_size_range=c(10,15),labels_num = 3,
    pie_legend_title = "Group")
dev.off()
```

#### STRING database

Beyond generic network types, MetaNet is compatible with biological
networks from external databases. For example, protein–protein
interaction (PPI) networks obtained from the STRING database can be
imported and visualized with customized layout and annotations (Figure
4D).

``` r
read.table("~/Downloads/string_interactions.tsv",comment.char = "",header = TRUE, sep = "\t") -> interactions
colnames(interactions)[1]="node1"

c_net_from_edgelist(interactions)-> net

V(net)$color=get_cols(length(V(net)),"col1")%>%add_alpha(0.5)
coors=c_net_layout(net)%>%transform_coors(mirror_y = T,mirror_x = T)

# Figure 4D
pdf("3-1.string_network.pdf",width = 5,height = 5)
c_net_plot(net,coors,edge.curved=0,vertex.shape="sphere",vertex.size=20,edge.width=1,
           vertex.label.cex=1,vertex.label.dist=2,legend = F,edge.color="green4")
dev.off()
```

#### miRNA-Gene networks

Similarly, miRNA–target gene regulatory networks from miRTarBase, which
are experimentally validated, can be represented to explore
post-transcriptional regulatory mechanisms (Figure 4E).

``` r
readxl::read_excel("~/database/hsa_MTI.xlsx") -> miRNA_target
filter(miRNA_target,`Support Type`=="Functional MTI") -> miRNA_target
distinct(miRNA_target,`miRNA`,`Target Gene`,.keep_all = T) -> miRNA_target

miRNA_target %>%
  select(`miRNA`,`Target Gene`)%>%
  filter(miRNA%in%c("hsa-miR-18a-5p","hsa-miR-199a-5p","hsa-miR-138-5p","hsa-miR-214-3p"))-> miRNA_target_f

c_net_from_edgelist(miRNA_target_f,direct = T)-> miRNA_net
simplify(miRNA_net) -> miRNA_net

# Figure 4E
pdf("3-1.miRNA_network.pdf",width = 6,height = 5)
c_net_plot(miRNA_net,vertex_size_range = list(c(10,10),c(4,4)),vertex.shape=c("triangle1","circle"),
           edge_legend = F,vertex.color=c("miRNA"="#E41A1C","Target Gene"="#A8DEB5"),labels_num = "all",
           edge.color="black",edge.width=.3,vertex.frame.width=0.2)
dev.off()
```

#### Enrichment

MetaNet also integrates with the ReporterScore, an R package we
previously developed for functional enrichment analysis. Using the
results of pathway enrichment, users can directly visualize
relationships between KEGG orthologs (KOs) and their associated pathways
(Figure 4F).

``` r
library(ReporterScore)
data("reporter_score_res")
# View(reporter_score_res$reporter_s)

# Figure 4F
pdf("3-1.enrichment_network.pdf",width = 6,height = 5)
plot_features_network(reporter_score_res, map_id = c("map00780","map00785","map03010","map05230","map04922"),
                      mark_module = T,near_pathway = F)
dev.off()
```

#### KEGG pathway

Furthermore, MetaNet supports direct rendering of any KEGG pathway map
through a specified pathway ID, enabling fully annotated and modifiable
visualizations (Figure 4G).

``` r
library(ReporterScore)

path_net_c <- c_net_from_pathway_xml("~/Documents/R/GRSA/ReporterScore_temp_download/ko01521.xml")
coors=get_v(path_net_c)[,c("name","x","y")]
colnames(coors)=c("name","X","Y")
coors=rescale_coors(as_coors(coors))
coors=transform_coors(coors,aspect_ratio = 0.6)

coors[11,c("X","Y")]=c(-0.75,0.7) #adjust the position of the "map" node

get_v(path_net_c)->tmp_v

# Figure 4G
plot_pathway_net(path_net_c,coors=coors,label_cex=0.6,
                 vertex.frame.width=0.2,arrow_size_cex=2,arrow_width_cex=2,
                 edge.width=0.5)
```

<img src="man/figures/fig3-1.jpeg"  width="500" />

**Figure 4. Diverse specialized network visualizations by MetaNet.**

For more detailed usage, please refer to the full vignette at
**<https://bookdown.org/Asa12138/metanet_book/>**.

MetaNet - free portable library for meta neural network research

INSTALLATION
------------
Make sure you have numpy installed, then
	python3 setup.py install

To remove package
	pip uninstall metanet

<!-- README.md is generated from README.Rmd. Please edit that file -->

# MetaNet <img src="man/figures/MetaNet.png" align="right" width="120" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/Asa12138/MetaNet/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Asa12138/MetaNet/actions/workflows/R-CMD-check.yaml)
[![](https://img.shields.io/badge/doi-10.1093/bioinformatics/btag321-yellow.svg)](https://doi.org/10.1093/bioinformatics/btag321)
[![](https://img.shields.io/badge/blog-@asa-blue.svg)](https://asa-blog.netlify.app/)
[![](http://cranlogs.r-pkg.org/badges/grand-total/MetaNet)](https://cran.r-project.org/package=MetaNet)
[![](http://cranlogs.r-pkg.org/badges/last-month/MetaNet)](https://cran.r-project.org/package=MetaNet)
[![](https://www.r-pkg.org/badges/version/MetaNet?color=green)](https://cran.r-project.org/package=MetaNet)
[![](https://img.shields.io/badge/devel%20version-0.3.2-green.svg)](https://github.com/Asa12138/MetaNet)

<!-- badges: end -->

MetaNet, a high-performance R package that unifies network construction,
visualization, and analysis across diverse omics layers.

MetaNet enables fast and scalable correlation-based network construction
for datasets with more than 10,000 features, providing over 40 layout
algorithms, rich annotation utilities, and visualization options
compatible with both static and interactive platforms. It further offers
comprehensive topological and stability metrics for in-depth network
characterization. Benchmarking shows that MetaNet delivers up to a
100-fold improvement in computation time and a 50-fold reduction in
memory usage compared to existing R packages.

![](man/figures/MetaNet_GA_v2.png)

The HTML documentation of the latest version is available at [Github
page](https://asa12138.github.io/MetaNet/).

- [Installation](#installation)
- [Tutorial](#tutorial)
- [Quick Start](#quick-start)
- [Detailed usage](#detailed-usage)
  - [Overview](#overview)
    - [Network construction](#network-construction)
    - [Network annotation and
      filtering](#network-annotation-and-filtering)
    - [Module detection](#module-detection)
    - [Network comparison](#network-comparison)
  - [Layout and Visualization](#layout-and-visualization)
    - [Basic layouts](#basic-layouts)
    - [spatstat layouts](#spatstat-layouts)
    - [Group and multi-level layouts](#group-and-multi-level-layouts)
  - [Biological network](#biological-networks)
    - [Venn diagram](#venn)
    - [Tree diagram](#tree)
    - [Pie node](#pie-node)
    - [SRTING database](#string-database)
    - [miRNA-Gene networks](#mirna-gene-networks)
    - [Enrichment](#enrichment)
    - [KEGG pathway](#kegg-pathway)

## Citation

If you use the MetaNet package in published research, please cite:

1.  Peng, C., Jiang, L., Huang, Z., Wei, X., Zhu, X., Liu, Z., Chen, Q.,
    Shen, X., Gao, P., and Jiang, C. (2026). MetaNet: a scalable and
    integrated tool for reproducible omics network analysis.
    Bioinformatics, btag321.
    <https://doi.org/10.1093/bioinformatics/btag321>.

## Installation

You can install the released version of `MetaNet` from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("MetaNet")
```

You can install the development version of `MetaNet` from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("Asa12138/MetaNet")
```

## Tutorial

Please go to **<https://bookdown.org/Asa12138/metanet_book/>** for the
full vignette.

<img src="man/figures/cover1.jpeg" width="250" />

MetaNet is an R-based integrative package designed for comprehensive
network analysis across diverse omics data, including multi-omics
datasets. MetaNet is compatible with operating systems (Windows, macOS,
and Linux) that support R version 4.0 or higher, and its core
functionality is built upon the widely used igraph package. Its
architecture comprises several core functional modules: Calculation,
Manipulation, Layout, Visualization, Topology analysis, Module analysis,
Stability analysis, and I/O (Figure 1A), supporting the end-to-end
analytical process from network construction to visualization. Figure 1B
illustrates the main workflow and essential components within MetaNet.

Pairwise correlation computation is central to most network-based omics
tools, but the growing scale of omics datasets imposes substantial
computational demands. MetaNet addresses this through optimized
vectorized matrix algorithms for calculating correlation coefficients
and corresponding p-values, greatly reducing memory use and runtime
(Figure 1D).

<img src="man/figures/fig1.jpeg"  width="700" />

**Figure 1. Overview of the MetaNet workflow and its high-efficiency
computation.**

## Quick Start

Simply build and draw a co-occurrence network plot, only need to use
`c_net_calculate()`, `c_net_build()`, `c_net_plot()` three functions:

``` r
library(pcutils)
library(MetaNet)
data("otutab", package = "pcutils")

# Inter-species correlation coefficients were calculated after transposition
totu <- t2(otutab[1:70, ])

cor <- c_net_calculate(totu)
net <- c_net_build(cor, r_threshold = 0.65)
#> Have not do p-value adjustment! use the p.value to build network.
c_net_plot(net)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="" width="100%" />

Here is a more complex example of building and visualizing a multi-omics
network with MetaNet, we can use `c_net_set` to flexibly set vertex
attributes such as class and size, and then visualize the network with
`c_net_plot()`:

``` r
data("multi_test")
multi1 <- multi_net_build(list(Microbiome = micro, Metabolome = metab, Transcriptome = transc))
#> All samples matched.
#> All features are OK.
#> Calculating 18 samples and 150 features of 3 groups.
#> Have not do p-value adjustment! use the p.value to build network.
multi1 <- c_net_set(multi1, micro_g, metab_g, transc_g,
  vertex_class = c("Phylum", "kingdom", "type")
)
multi1 <- c_net_set(multi1, data.frame("Abundance1" = colSums(micro)),
  data.frame("Abundance2" = colSums(metab)), data.frame("Abundance3" = colSums(transc)),
  vertex_size = paste0("Abundance", 1:3)
)
c_net_plot(multi1)
```

<img src="man/figures/README-unnamed-chunk-4-1.png" alt="" width="100%" />

For more detailed usage, please refer to the sections below or the full
vignette at **<https://bookdown.org/Asa12138/metanet_book/>**.

## Detailed usage

### Overview

#### Network construction

We use a multi-omics dataset to demonstrate the powerful network
manipulation capabilities of MetaNet. The dataset includes three omics
layers: microbiome, metabolome, and transcriptome, along with their
corresponding metadata.

``` r
data("multi_test")
v_color <- setNames(
  c("#b2df8a", "#cab2d6", "#a6bce3", "#fdbf6f", "#fb9a99", "#1f78b4"),
  c("Actinobacteria", "Proteobacteria", "Benzenoids", "Hydrocarbons", "Cell motility", "Immune system")
)

# We create a simulated multi-omics dataset for demonstration purposes:
micro_g <- micro_g[micro_g$Phylum %in% c("p__Actinobacteria", "p__Proteobacteria"), ]
micro_g$Phylum <- gsub(".__", "", micro_g$Phylum)
micro <- micro[, rownames(micro_g)]
metab_g$kingdom <- tidai(metab_g$kingdom, c("Benzenoids", "Hydrocarbons"))
transc_g$type <- tidai(transc_g$type, c("Cell motility", "Immune system"))
```

We first build a multi-omics network using `multi_net_build()` with a
specified correlation threshold, which will automatically calculate the
correlations and construct the network across all omics layers:

``` r
multi1 <- multi_net_build(list(
  Microbiome = micro,
  Metabolome = metab,
  Transcriptome = transc
), r_threshold = 0.6)

# Figure 2A
c_net_plot(multi1, legend = F, main = "", vertex.color = v_color)
```

#### Network annotation and filtering

In omics and multi-omics studies, networks are often annotated with
external data such as abundance profiles, taxonomy, or clinical
metadata. The `c_net_set` function attaches multiple annotation tables
to a network object and automatically configures visualization
properties (Figure 2B), including color schemes, line types, node
shapes, and legends.

``` r
multi1_with_anno <- c_net_set(multi1, micro_g, metab_g, transc_g,
  vertex_class = c("Phylum", "kingdom", "type")
)
multi1_with_anno <- c_net_set(multi1_with_anno,
  data.frame("Abundance1" = colSums(micro)),
  data.frame("Abundance2" = colSums(metab)),
  data.frame("Abundance3" = colSums(transc)),
  vertex_size = paste0("Abundance", 1:3)
)
```

``` r
# Figure 2B
c_net_plot(multi1_with_anno,
  legend_number = T, vertex.color = v_color,
  edge.lty = c(4, 1), vertex_size_range = c(4, 9),
  lty_legend = T, size_legend = T, legend_cex = 1.2
)
```

After annotation and customization, researchers may focus on specific
network regions—especially in multi-omics integration. The
`c_net_filter` function extracts sub-networks using flexible filters
(Figure 2C), while `c_net_highlight` visually emphasizes selected nodes
or edges (Figure 2D).

``` r
# filter the network to only show intra-omics correlations within microbiome and metabolome layers:
multi2 <- c_net_filter(multi1_with_anno, v_group %in% c("Microbiome", "Metabolome")) %>%
  c_net_filter(., e_class == "intra", mode = "e")

# Figure 2C
c_net_plot(multi2, legend = F, main = "", edge.lty = 1, vertex.color = v_color)
```

``` r
degree(multi1) %>%
  sort(decreasing = T) %>%
  head()
core <- c_net_neighbors(multi1_with_anno, "s__Dongia_mobilis")
c_net_highlight(multi1_with_anno, V(core)$name) -> multi1_highlight
V(multi1_highlight)$label <- ifelse(V(multi1_highlight)$label == "s__Dongia_mobilis",
  "s__Dongia_mobilis", NA
)
# Figure 2D
c_net_plot(multi1_highlight, tmp_gephi$coors,
  legend = F, main = "",
  labels_num = "all"
)
```

#### Module detection

Modules or communities—densely connected subgraphs—often represent
biologically meaningful groups. MetaNet supports module detection
through `c_net_module`, which includes multiple community detection
algorithms (Figure 2E).

``` r
c_net_module(multi1_with_anno) -> multi1_module

g_layout_treemap(multi1_module) -> coors1
c_net_plot(multi1_module, coors1,
  legend = F, main = "",
  plot_module = T, mark_module = T, vertex.color = get_cols()
)
```

Resulting modules can be visualized with chord or Sankey diagrams to
show proportions and inter-module connections (Figure 2F). For
group-level analysis, the `c_net_skeleton` function summarizes edge
origins and targets across conditions, enhancing interpretability in
multi-condition or longitudinal datasets (Figure 2G).

``` r
links_stat(multi1_module, "module")

c_net_skeleton(multi2) %>%
  skeleton_plot(
    vertex.color = v_color, split_e_type = F, coors = in_circle(),
    legend = F, main = ""
  )
```

<img src="man/figures/fig2.jpeg"  width="700" />

**Figure 2. MetaNet supports flexible and intuitive network
manipulation.**

#### Network comparison

Comparative analysis across multiple networks is also critical.
Researchers may identify differential edges between groups or track
stable subnetworks across transitions. MetaNet enables such comparisons
by computing intersections, unions, and differences between networks
(Figure 2H), offering a flexible framework for comparative and
evolutionary network analysis.

``` r
library(igraph)
set.seed(123)
g1 <- make_graph("Icosahedron")
V(g1)$color <- "#4DAF4A77"
E(g1)$color <- "#4DAF4A77"

g1 <- as.metanet(g1)

g2 <- make_graph("Octahedron")
V(g2)$name <- as.character(9:14)
V(g2)$color <- "#984EA366"
E(g2)$color <- "#984EA366"

g2 <- as.metanet(g2)

# Perform network operations
g_union <- c_net_union(g1, g2)
E(g_union)$color <- "orange"
g_inter <- c_net_intersect(g1, g2)
g_diff <- c_net_difference(g1, g2)

par_ls <- list(main = "", legend = F, vertex_size_range = c(20, 20))
c_net_plot(g1, params_list = par_ls)
c_net_plot(g2, params_list = par_ls, coors = in_circle())
c_net_plot(g_union,
  params_list = par_ls,
  coors = transform_coors(c_net_layout(g_union), rotation = 90)
)
c_net_plot(g_inter, params_list = par_ls)
c_net_plot(g_diff, params_list = par_ls)
```

### Layout and Visualization

#### Basic layouts

Layout is a critical component of network visualization, as a
well-designed layout can significantly enhance the interpretability of
network structures. MetaNet stores layout coordinates in a flexible
`coors` object, allowing users to control, reuse, and transfer layout
settings. The `c_net_layout` function provides access to over 40 layout
algorithms (Figure 3A), including several new layouts as well as
adaptations from `igraph` and `ggraph` packages.

``` r
net <- make_graph("Zachary")
net <- as.metanet(net)
V(net)$color <- rep(get_cols(6), length = length(net))

layout_methods <- list(
  with_fr(), in_circle(), on_grid(), randomly(), as_tree(),
  with_gem(), with_graphopt(), with_kk(), with_mds(),
  as_line(angle = 45), as_arc(), as_polygon(), as_polycircle(3), as_circle_tree(),
  as_multi_layer(2)
)
names(layout_methods) <- c(
  "with_fr", "in_circle", "on_grid", "randomly", "as_tree",
  "with_gem", "with_graphopt", "with_kk", "with_mds",
  "as_line", "as_arc", "as_polygon", "as_polycircle", "as_circle_tree",
  "as_multi_layer"
)

layout_methods2 <- c(
  "backbone", "dendrogram", "eigen", "focus", "hive",
  "stress", "unrooted", "cactustree", "fabric"
)

par(mfrow = c(4, 6), mar = c(0.5, 0.5, 0.5, 0.5))
for (i in names(layout_methods)) {
  c_net_plot(net,
    coors = layout_methods[[i]],
    edge.color = "grey", vertex_size_range = c(10, 10), edge_width_range = c(0.8, 0.8),
    legend = F, main = i, labels_num = 0,
    rescale = (i == "as_tree")
  )
}

for (i in layout_methods2) {
  if (i == "focus") {
    coors <- c_net_layout(clean_igraph(net), "focus", focus = 4)
  } else if (i == "hive") {
    coors <- c_net_layout(clean_igraph(net), "hive", axis = 1:5)
  } else if (i == "pmds") {
    coors <- c_net_layout(clean_igraph(net), "pmds", pivots = 15)
  } else {
    coors <- c_net_layout(
      clean_igraph(net, direct = i %in% c("dendrogram", "partition", "treemap", "cactustree")),
      i
    )
  }
  c_net_plot(net,
    coors = coors,
    edge.color = "grey", vertex_size_range = c(10, 10), edge_width_range = c(0.8, 0.8),
    legend = F, main = i, labels_num = 0,
    rescale = T
  )
}
```

<img src="man/figures/fig3.jpeg"  width="700" />

**Figure 3. MetaNet enables diverse and powerful network layout
strategies.**

#### spatstat layouts

In addition to conventional layouts, MetaNet introduces the
`spatstat_layout` method, which constrains layout generation within a
user-defined polygon or along its edges. This layout function supports
uniform or random node distributions inside custom shapes. For example,
arranging a network within a star (Figure 3B) or mapping it to a
geographic region like Australia (Figure 3C).

``` r
library(spatstat.geom)

create_star_window <- function(r_outer = 1, r_inner = 0.4, center = c(0, 0)) {
  # 创建五角星的10个顶点（外、内交替）
  theta <- seq(0, 2 * pi, length.out = 11)[-11] # 10个点
  theta_outer <- theta[seq(1, 10, 2)]
  theta_inner <- theta[seq(2, 10, 2)]

  x <- c(
    r_outer * cos(theta_outer),
    r_inner * cos(theta_inner)
  )
  y <- c(
    r_outer * sin(theta_outer),
    r_inner * sin(theta_inner)
  )

  # 重新排序成首尾相连的路径
  order_index <- c(1, 6, 2, 7, 3, 8, 4, 9, 5, 10)
  x <- x[order_index] + center[1]
  y <- y[order_index] + center[2]

  # 构建 spatstat 的 owin 窗口
  win <- owin(poly = list(x = x, y = y))
  return(win)
}

win_star <- create_star_window()

tmp_net <- erdos.renyi.game(400, p.or.m = 0.005) %>% as.metanet()

c_net_plot(co_net,
  coors = spatstat_layout(co_net, win_star, order_by = "v_class"),
  legend = F, edge.color = "grey", main = "spatstat_layout", edge.width = 0.3
)
```

#### Group and multi-level layouts

For networks with grouping variables, MetaNet offers an advanced
interface via `g_layout`. Users can define spatial configurations for
each group, including positioning, scaling, and internal layout
strategies, and combine multiple layout types in one visualization.

The resulting `coors` object can be nested or recombined with subsequent
calls to create highly customized multi-level layouts. For example, a
co-abundance network across multiple human body sites can be arranged
with a single `g_layout` call (Figure 3D). This strategy is also useful
for highlighting modular structures. `g_layout_circlepack` visualizes
module distribution using compact circular packing (Figure 3E), while
`g_layout_multi_layer` introduces a pseudo-3D representation emphasizing
inter-module relationships (Figure 3F).

``` r
set.seed(12)
igraph::sample_islands(4, 40, 0.15, 3) %>%
  as.metanet() %>%
  c_net_module() -> test_net
test_net <- to_module_net(test_net)

V(test_net)$v_class <- sample(letters[1:4], size = length(test_net), replace = T)
test_net <- c_net_set(test_net)
c_net_plot(test_net, plot_module = F)

g_coors <- g_layout(test_net,
  group = "module",
  layout1 = data.frame(X = c(0, 0.3, 0.3, 0), Y = c(0, 2:4)),
  layout2 = list(with_fr(), on_grid(), as_polycircle(3), as_polygon(3)),
  zoom2 = c(1.3, 1, 1, 1.3) * 3
)

img <- png::readPNG("body.png")

# Figure 3D
pdf("3.g_layout.pdf", height = 8, width = 5)
par(xpd = TRUE)
plot(NA, xlim = c(-1, 1), ylim = c(-1, 1), axes = F, asp = 1, xlab = "", ylab = "")
rasterImage(bg_img, -1, -1.2, -0.4, 1.2, interpolate = F)
par(new = TRUE)
c_net_plot(test_net, coors = g_coors, plot_module = F, legend = F, main = "", vertex.size = 2)

segments(
  x0 = c(-0.52, -0.3, -0.42, -0.6), y0 = c(0.1, 0.1, 0.5, 0.78),
  x1 = c(-0.2, -0.1, -0.12, -0.25), y1 = c(-0.63, 0, 0.45, 0.8),
  col = "black", lwd = 1.5
)
dev.off()
```

``` r
set.seed(12)
E(co_net)$color <- rep("grey", length(E(co_net)))
coors <- g_layout_circlepack(multi1_module, group = "module")

# Figure 3E
pdf("3.g_layout_circlepack.pdf", height = 7, width = 10)
c_net_plot(multi1_module,
  coors = transform_coors(coors, rotation = 130), edge.lty = 1, edge.width = 0.5,
  legend = F, labels_num = 0, main = "g_layout_circlepack",
  plot_module = T, mark_module = F, vertex_size_range = c(3, 8)
)
dev.off()
```

``` r
set.seed(112)
igraph::sample_islands(3, 30, 0.15, 0) %>%
  as.metanet() %>%
  c_net_module() -> test_net

test_net <- to_module_net(test_net)
test_net <- add_edges(test_net, c(25, 61, 25, 63, 25, 75, 25, 88, 25, 79, 25, 66))
test_net2 <- add_edges(test_net, c(4, 56, 4, 57, 4, 60, 4, 48, 4, 51, 4, 55, 4, 32))

get_e(test_net2) -> tmp_e
tmp_e$width <- 1
tmp_e$lty <- 1
tmp_e$color <- ifelse(is.na(tmp_e$color), "#FA789A", tmp_e$color)

edge.attributes(test_net2) <- as.list(tmp_e)
V(test_net2)$size <- V(test_net2)$degree <- degree(test_net2)

# Figure 3F
pdf("3.multi_layer.pdf", height = 5)
plot(as.metanet(test_net2),
  coors = g_layout_multi_layer(test_net, group = "v_class", layout = on_grid()),
  legend = F, labels_num = 0, main = "g_layout_multi_layer",
  edge.curved = ifelse(is.na(tmp_e$e_type), 0.2, 0)
)
dev.off()
```

### Biological networks

MetaNet provides native support for a variety of specialized network
types frequently used in bioinformatics workflows, enabling researchers
to visualize and explore biological relationships beyond conventional
correlation or interaction networks.

#### Venn

MetaNet allows the construction of Venn-style networks to illustrate set
relationships across sample groups. These provide a more informative
alternative to traditional Venn diagrams by displaying explicit
connections and network structure (Figure 4A).

``` r
data(otutab, package = "pcutils")
tab <- otutab[420:485, 1:3]
venn_net(tab) -> v_net

# Figure 4A
pdf("3-1.venn_network.pdf", width = 5, height = 5)
plot(v_net,
  vertex_size_range = list("Group" = c(18, 18), "elements" = c(4, 4)),
  edge.width = .5, vertex.frame.width = 0.3
)
dev.off()
```

#### Tree

Tree-structured data, such as taxonomies or gene ontology hierarchies,
can be visualized using the built-in “as_circle_tree” layout, offering a
clear and compact representation of hierarchical relationships (Figure
4B).

``` r
data("otutab", package = "pcutils")
cbind(taxonomy, num = rowSums(otutab))[1:20, ] -> test
df2net_tree(test) -> ttt

# Figure 4B
pdf("3-1.circle_tree_network.pdf", width = 5, height = 5)
plot(ttt,
  coors = as_circle_tree(), legend = F, main = "Circle tree network",
  edge.arrow.size = 0.3, edge.arrow.width = 0.6, rescale = T, vertex.label = ifelse(
    V(ttt)$v_class %in% c("Species"), V(ttt)$name, NA
  ), edge.color = "black", edge.width = 0.4
)
dev.off()
```

#### Pie node

MetaNet further supports pie-node visualization, where each node encodes
multivariate annotations, such as group-specific abundances. This
approach allows compositional data to be embedded directly in the
network structure (Figure 4C).

``` r
data("otutab")
data("c_net")
hebing(otutab, metadata$Group) -> otutab_G

V(co_net)$degree <- degree(co_net)
co_net_f <- c_net_filter(co_net, degree > 6, degree < 15)

# Figure 4C
pdf("3-1.pie_network.pdf", width = 5, height = 5)
c_net_plot(co_net_f,
  pie_value = otutab_G, vertex.shape = c("pie"),
  pie_legend = T, color_legend = F, vertex_size_range = c(10, 15), labels_num = 3,
  pie_legend_title = "Group"
)
dev.off()
```

#### STRING database

Beyond generic network types, MetaNet is compatible with biological
networks from external databases. For example, protein–protein
interaction (PPI) networks obtained from the STRING database can be
imported and visualized with customized layout and annotations (Figure
4D).

``` r
read.table("~/Downloads/string_interactions.tsv", comment.char = "", header = TRUE, sep = "\t") -> interactions
colnames(interactions)[1] <- "node1"

c_net_from_edgelist(interactions) -> net

V(net)$color <- get_cols(length(V(net)), "col1") %>% add_alpha(0.5)
coors <- c_net_layout(net) %>% transform_coors(mirror_y = T, mirror_x = T)

# Figure 4D
pdf("3-1.string_network.pdf", width = 5, height = 5)
c_net_plot(net, coors,
  edge.curved = 0, vertex.shape = "sphere", vertex.size = 20, edge.width = 1,
  vertex.label.cex = 1, vertex.label.dist = 2, legend = F, edge.color = "green4"
)
dev.off()
```

#### miRNA-Gene networks

Similarly, miRNA–target gene regulatory networks from miRTarBase, which
are experimentally validated, can be represented to explore
post-transcriptional regulatory mechanisms (Figure 4E).

``` r
readxl::read_excel("~/database/hsa_MTI.xlsx") -> miRNA_target
filter(miRNA_target, `Support Type` == "Functional MTI") -> miRNA_target
distinct(miRNA_target, `miRNA`, `Target Gene`, .keep_all = T) -> miRNA_target

miRNA_target %>%
  select(`miRNA`, `Target Gene`) %>%
  filter(miRNA %in% c("hsa-miR-18a-5p", "hsa-miR-199a-5p", "hsa-miR-138-5p", "hsa-miR-214-3p")) -> miRNA_target_f

c_net_from_edgelist(miRNA_target_f, direct = T) -> miRNA_net
simplify(miRNA_net) -> miRNA_net

# Figure 4E
pdf("3-1.miRNA_network.pdf", width = 6, height = 5)
c_net_plot(miRNA_net,
  vertex_size_range = list(c(10, 10), c(4, 4)), vertex.shape = c("triangle1", "circle"),
  edge_legend = F, vertex.color = c("miRNA" = "#E41A1C", "Target Gene" = "#A8DEB5"), labels_num = "all",
  edge.color = "black", edge.width = .3, vertex.frame.width = 0.2
)
dev.off()
```

#### Enrichment

MetaNet also integrates with the ReporterScore, an R package we
previously developed for functional enrichment analysis. Using the
results of pathway enrichment, users can directly visualize
relationships between KEGG orthologs (KOs) and their associated pathways
(Figure 4F).

``` r
library(ReporterScore)
data("reporter_score_res")
# View(reporter_score_res$reporter_s)

# Figure 4F
pdf("3-1.enrichment_network.pdf", width = 6, height = 5)
plot_features_network(reporter_score_res,
  map_id = c("map00780", "map00785", "map03010", "map05230", "map04922"),
  mark_module = T, near_pathway = F
)
dev.off()
```

#### KEGG pathway

Furthermore, MetaNet supports direct rendering of any KEGG pathway map
through a specified pathway ID, enabling fully annotated and modifiable
visualizations (Figure 4G).

``` r
library(ReporterScore)

path_net_c <- c_net_from_pathway_xml("~/Documents/R/GRSA/ReporterScore_temp_download/ko01521.xml")
coors <- get_v(path_net_c)[, c("name", "x", "y")]
colnames(coors) <- c("name", "X", "Y")
coors <- rescale_coors(as_coors(coors))
coors <- transform_coors(coors, aspect_ratio = 0.6)

coors[11, c("X", "Y")] <- c(-0.75, 0.7) # adjust the position of the "map" node

get_v(path_net_c) -> tmp_v

# Figure 4G
plot_pathway_net(path_net_c,
  coors = coors, label_cex = 0.6,
  vertex.frame.width = 0.2, arrow_size_cex = 2, arrow_width_cex = 2,
  edge.width = 0.5
)
```

<img src="man/figures/fig3-1.jpeg"  width="700" />

**Figure 4. Diverse specialized network visualizations by MetaNet.**

For more detailed usage, please refer to the full vignette at
**<https://bookdown.org/Asa12138/metanet_book/>**.

# =========2.build======

#' Construct a metanet from a corr object
#'
#' @param corr corr object from `c_net_calculate()` or `read_corr()`.
#' @param r_threshold r_threshold (default: >0.6).
#' @param p_threshold p_threshold (default: <0.05).
#' @param use_p_adj use the p.adjust instead of p.value (default: TRUE), if p.adjust not in the corr object, use p.value.
#' @param delete_single should delete single vertexes?
#'
#' @return an metanet object
#' @export
#'
#' @family build
#' @examples
#' data("otutab", package = "pcutils")
#' t(otutab) -> totu
#' metadata[, 3:10] -> env
#' c_net_calculate(totu) -> corr
#' c_net_build(corr, r_threshold = 0.65) -> co_net
#'
#' c_net_calculate(totu, env) -> corr2
#' c_net_build(corr2) -> co_net2
c_net_build <- function(corr, r_threshold = 0.6, p_threshold = 0.05, use_p_adj = TRUE, delete_single = TRUE) {
  stopifnot(inherits(corr, "corr"))
  # set thresholds to construct
  if ("r" %in% names(corr)) {
    occor.r <- corr$r
  } else {
    stop("No r in the input corr object.")
  }

  if (("p.adjust" %in% names(corr)) && use_p_adj) {
    occor.p <- corr$p.adjust
  } else {
    if ("p.value" %in% names(corr)) {
      occor.p <- corr$p.value
      message("Have not do p-value adjustment! use the p.value to build network.")
    } else {
      occor.p <- occor.r
      occor.p[occor.p != 0] <- 0
      message("No p.value or p.adjust in the input corr object, use the r to build network.")
    }
  }

  if (any(is.na(occor.r))) {
    occor.r[is.na(occor.r)] <- 0
    warning("NA values in the input corr object, set to 0.")
  }
  occor.r[occor.p > p_threshold | abs(occor.r) < r_threshold] <- 0

  # make igraph
  if (t_flag(occor.r)) {
    go <- igraph::graph_from_adjacency_matrix(as.matrix(occor.r),
      mode = "undirected", weighted = TRUE, diag = FALSE
    )
    igraph::graph.attributes(go)$n_type <- "single"
  } else {
    inset <- intersect(rownames(occor.r), colnames(occor.r))
    if (length(inset) > 0) {
      insets <- paste0(inset, collapse = ", ")
      warning("some nodes (row and column) has same name, please check: ", insets)
    }
    go <- igraph::graph_from_incidence_matrix(as.matrix(occor.r),
      directed = FALSE, weighted = TRUE
    )
    igraph::graph.attributes(go)$n_type <- "bipartite"
  }

  # delete single vertexes?
  if (delete_single) go <- igraph::delete.vertices(go, V(go)[igraph::degree(go) == 0])

  # set vertex attributes
  V(go)$v_group <- ifelse(V(go)$name %in% rownames(occor.r), "v_group1", "v_group2")
  V(go)$v_class <- ifelse(V(go)$name %in% rownames(occor.r), "v_class1", "v_class2")
  V(go)$size <- ceiling(60 / sqrt(length(V(go)))) + 1
  V(go)$label <- V(go)$name

  # abs edges weight
  go.weight <- E(go)$weight
  E(go)$cor <- go.weight
  E(go)$weight <- abs(go.weight)

  # time-consuming
  if (TRUE) {
    tmp_e <- get_e(go)
    if ("p.value" %in% names(corr)) {
      # E(go)$p.value <-get_e(go)%>%dplyr::select(from,to)%>%apply(., 1, \(x)occor.r$p.value[x[1],x[2]])
      tmp <- reshape2::melt(corr$p.value, varnames = c("from", "to"), value.name = "p.value", as.is = TRUE)
      tmp_e <- dplyr::left_join(tmp_e, tmp, by = c("from", "to"))
    }
    if ("p.adjust" %in% names(corr)) {
      # E(go)$p.adjust <-get_e(go)%>%dplyr::select(from,to)%>%apply(., 1, \(x)occor.r$p.adjust[x[1],x[2]])
      tmp <- reshape2::melt(corr$p.adjust, varnames = c("from", "to"), value.name = "p.adjust", as.is = TRUE)
      tmp_e <- dplyr::left_join(tmp_e, tmp, by = c("from", "to"))
    }
    # 应该直接expand，再left_join快很多
    igraph::edge.attributes(go) <- as.list(tmp_e)
  }

  # set edges type
  E(go)$e_type <- ifelse(go.weight > 0, "positive", "negative")
  # set edges width
  E(go)$width <- E(go)$weight

  if (!t_flag(occor.r)) {
    # set edges from_to
    anno_edge(go, get_v(go)[, c("name", "v_group")], verbose = FALSE) -> go
    # set edge intra-inter
    tmp_e <- igraph::edge.attributes(go)
    E(go)$e_class <- ifelse(tmp_e$v_group_from == tmp_e$v_group_to, "intra", "inter")
  }

  c_net_update(go, initialize = TRUE) -> go

  return(go)
}


#' Multi-omics network build
#'
#' @param ... some omics abundance tables
#' @param mode "full"
#' @param method "spearman" or "pearson"
#' @param filename the prefix of saved .corr file or FALSE
#' @param p.adjust.method see \code{\link[stats]{p.adjust}}
#' @param r_threshold r_threshold (default: >0.6)
#' @param p_threshold p_threshold (default: <0.05)
#' @param use_p_adj use the p.adjust instead of p-value (default: TRUE)
#' @param delete_single should delete single vertexes?
#'
#' @return metanet
#' @export
#' @family build
#' @examples
#' data("multi_test")
#' multi1 <- multi_net_build(list(Microbiome = micro, Metabolome = metab, Transcriptome = transc))
#' multi1 <- c_net_set(multi1, micro_g, metab_g, transc_g,
#'   vertex_class = c("Phylum", "kingdom", "type")
#' )
#' multi1 <- c_net_set(multi1, data.frame("Abundance1" = colSums(micro)),
#'   data.frame("Abundance2" = colSums(metab)), data.frame("Abundance3" = colSums(transc)),
#'   vertex_size = paste0("Abundance", 1:3)
#' )
#' c_net_plot(multi1)
multi_net_build <- function(..., mode = "full", method = "spearman",
                            filename = FALSE, p.adjust.method = NULL,
                            r_threshold = 0.6, p_threshold = 0.05, use_p_adj = TRUE, delete_single = TRUE) {
  tables <- list(...)
  if (all(class(tables[[1]]) == "list")) tables <- tables[[1]]
  tabs_name <- names(tables)

  tables <- check_tabs(tables)
  if (mode == "full") {
    all_totu <- do.call(cbind, tables)
    message("Calculating ", nrow(all_totu), " samples and ", ncol(all_totu), " features of ", length(tables), " groups.\n")
    all_corr <- c_net_calculate(all_totu, method = method, filename = filename, p.adjust.method = p.adjust.method)

    c_net_build(all_corr,
      r_threshold = r_threshold, p_threshold = p_threshold,
      use_p_adj = use_p_adj, delete_single = delete_single
    ) -> multi_net

    igraph::graph.attributes(multi_net)$n_type <- "multi_full"

    get_v(multi_net) -> tmp_v
    if (is.null(tabs_name)) tabs_name <- paste0("omic", seq_len(length(tables)))
    position <- rep(tabs_name, vapply(tables, ncol, numeric(1)))
    names(position) <- lapply(tables, colnames) %>% do.call(c, .)

    tmp_v$v_class <- tmp_v$v_group <- vapply(tmp_v$name, \(x)position[x], character(1))
    tmp_v %>% as.list() -> igraph::vertex_attr(multi_net)

    # set edges from_to
    suppressMessages(anno_edge(multi_net, tmp_v[, c("name", "v_group")], verbose = FALSE) -> multi_net)
    # set edge intra-inter
    tmp_e <- igraph::edge.attributes(multi_net)
    E(multi_net)$e_class <- ifelse(tmp_e$v_group_from == tmp_e$v_group_to, "intra", "inter")
    c_net_update(multi_net, initialize = TRUE, verbose = FALSE) -> multi_net
  }
  multi_net
}


default_v_color <- c(
  "#a6bce3", "#fb9a99", "#fdbf6f", "#1f78b4", "#b2df8a",
  "#cab2d6", "#33a02c", "#e31a1c", "#ff7f00", "#6a3d9a",
  "#F8CC00", "#b15928"
)
default_v_color_num <- c(
  "#FEE0D2", "#FCBBA1", "#FC9272", "#FB6A4A", "#EF3B2C", "#CB181D",
  "#A50F15", "#67000D"
)
default_e_color <- c(
  "#a6cee3", "#78c679", "#c2a5cf", "#ff7f00", "#1f78b4",
  "#810f7c", "#F8CC00", "#006d2c", "#4d4d4d", "#8c510a",
  "#d73027", "#7f0000", "#41b6c4", "#e7298a", "#54278f"
)
default_e_color_num <- c(
  "#E5F5E0", "#C7E9C0", "#A1D99B", "#74C476", "#41AB5D", "#238B45",
  "#006D2C", "#00441B"
)

default_v_shape <- c("circle" = 21, "square" = 22, "diamond" = 23, "triangle1" = 24, "triangle2" = 25) # "triangle", "diamond", "hexagon", "star"

color_generate <- function(v_class, n_break = 5, mode = "v",
                           v_color = default_v_color, v_color_num = default_v_color_num,
                           e_color = default_e_color, e_color_num = default_e_color_num) {
  # ！！！现在的legend数字不好看，考虑pretty改进
  if (is.numeric(v_class)) {
    # n_color=n_break
    # n_label=seq(min(v_class, na.rm = TRUE), max(v_class, na.rm = TRUE), length.out = n_break)

    n_break <- pretty(v_class, n_break)
    n_color <- length(n_break) - 1
    n_label <- n_break[seq_len(n_color)] %>% format(., trim = TRUE)

    if (mode == "v") {
      return(as.character(cut(v_class,
        breaks = n_break,
        labels = pcutils::get_cols(n_color, v_color_num)
      )))
    } else if (mode == "e") {
      return(as.character(cut(v_class,
        breaks = n_break,
        labels = pcutils::get_cols(n_color, e_color_num)
      )))
    } else if (mode == "label") {
      return(as.character(cut(v_class,
        breaks = n_break,
        labels = n_label
      )))
    }
  } else {
    if (mode == "v") {
      return(pcutils::tidai(v_class, pcutils::get_cols(nlevels(factor(v_class)), v_color), fac = TRUE))
    } else if (mode == "e") {
      v_class <- droplevels(as.factor(v_class))
      if (all(levels(v_class) %in% c("negative", "positive"))) {
        ncols <- c(negative = "#E85D5D", positive = "#48A4F0")[levels(v_class)]
      } else if (all(levels(v_class) %in% c("inter-module", "intra-module"))) {
        ncols <- c("inter-module" = "#FA789A", "intra-module" = "#A6CEE3")[levels(v_class)]
      } else {
        ncols <- e_color
      }
      return(pcutils::tidai(v_class, pcutils::get_cols(nlevels(factor(v_class)), ncols), fac = TRUE))
    } else if (mode == "label") {
      return(v_class)
    }
  }
}

#' Update a metanet object or transform igraph object to metanet object
#'
#' @param go a metanet object or igraph object
#' @param node_break node_break if v_class is numeric, default: 5
#' @param initialize initialize?
#' @param edge_break edge_break if e_type is numeric, default: 5
#' @param verbose verbose?
#' @param uniq_v_class if TRUE, add prefix to v_class if multiple v_class belong to same v_group.
#'
#' @aliases as.metanet
#' @export
#' @family build
#' @return metanet
c_net_update <- function(go, node_break = 5, edge_break = 5, initialize = FALSE, verbose = TRUE, uniq_v_class = FALSE) {
  stopifnot(inherits(go, "igraph"))
  if (length(go) == 0) {
    message("Empty graph, return empty metanet object.")
    return(go)
  }
  v_class <- NULL
  # name
  if (!"name" %in% igraph::vertex_attr_names(go)) V(go)$name <- as.character(seq_len(length(go)))
  if (!"label" %in% igraph::vertex_attr_names(go)) V(go)$label <- V(go)$name
  get_v(go) -> tmp_v
  # v_size
  if (!"size" %in% colnames(tmp_v)) tmp_v$size <- 1

  # ！！！从这里开始仔细修改，考虑对应关系
  # v_shape
  if (!"v_group" %in% colnames(tmp_v)) {
    tmp_v$v_group <- "v_group1"
  } else {
    tmp_v$v_group <- as.character(tmp_v$v_group)
    if (length(unique(tmp_v$v_group)) > 10) warning("Too many groups, please check the 'v_group'.")
  }
  flag <- TRUE
  if ("shape" %in% colnames(tmp_v)) {
    if (initialize) {
      if (e_match(tmp_v[, c("v_group", "shape")])) {
        flag <- FALSE
      } else if (verbose) message("'v_group' and 'shape' not match one by one, update 'shape'.")
    } else {
      if (e_match(tmp_v[, c("v_group", "shape")], test = 1)) {
        flag <- FALSE
      } else if (verbose) message("a 'v_group' should not match multiple shapes, update 'shape'.")
    }
  }
  if (flag) tmp_v$shape <- pcutils::tidai(tmp_v$v_group, names(default_v_shape), fac = TRUE)

  # v_color
  if (!"v_class" %in% colnames(tmp_v)) {
    tmp_v$v_class <- "v_class1"
  }

  if (uniq_v_class) {
    # 如果有同样的v_class对应不同v_group，则加上前缀
    dplyr::distinct_all(tmp_v[, c("v_class", "v_group")]) %>% dplyr::count(v_class) -> group_class
    group_class <- setNames(group_class$n, group_class$v_class)
    if (any(group_class > 1)) {
      la <- which(group_class > 1)
      if (verbose) message("some 'v_class' belong to multiple 'v_group': \n", paste0(names(la), collapse = ", "))
      tmp_v$v_class %in% names(la) -> r_index
      tmp_v$v_class[r_index] <- paste0(tmp_v$v_group[r_index], "-", tmp_v$v_class[r_index])
    }
  }

  flag <- TRUE
  if ("color" %in% colnames(tmp_v)) {
    if (initialize) {
      if (e_match(tmp_v[, c("v_class", "color")])) {
        flag <- FALSE
      } else if (verbose) message("'v_class' and 'color' not match one by one, update 'color'.")
    } else {
      if (e_match(tmp_v[, c("v_class", "color")], test = 1)) {
        flag <- FALSE
      } else if (verbose) message("a 'v_class' should not match multiple colors, update 'color'.")
    }
  }

  if (flag) {
    tmp_v$color <- color_generate(tmp_v$v_class, node_break, mode = "v")
    tmp_v$v_class <- color_generate(tmp_v$v_class, node_break, mode = "label")
  }

  as.list(tmp_v) -> igraph::vertex_attr(go)

  # e_color
  if (!"e_type" %in% igraph::edge_attr_names(go)) {
    E(go)$e_type <- "e_type1"
  }
  flag <- TRUE
  if ("color" %in% igraph::edge_attr_names(go)) {
    if (initialize) {
      if (e_match(data.frame(a = E(go)$e_type, b = E(go)$color))) {
        flag <- FALSE
      } else if (verbose) message("'e_type' and 'color' not match one by one, update 'color'.")
    } else {
      if (e_match(data.frame(a = E(go)$e_type, b = E(go)$color), test = 1)) {
        flag <- FALSE
      } else if (verbose) message("a 'e_type' should not match multiple colors, update 'color'.")
    }
  }
  if (flag) {
    E(go)$color <- color_generate(E(go)$e_type, edge_break, mode = "e")
    E(go)$e_type <- color_generate(E(go)$e_type, edge_break, mode = "label")
  }

  # e_lty
  if (!"e_class" %in% igraph::edge_attr_names(go)) {
    E(go)$e_class <- "e_class1"
  }
  flag <- TRUE
  if ("lty" %in% igraph::edge_attr_names(go)) {
    if (initialize) {
      if (e_match(data.frame(a = E(go)$e_class, b = E(go)$lty))) {
        flag <- FALSE
      } else if (verbose) message("'e_class' and 'lty' not match one by one, update 'lty'.")
    } else {
      if (e_match(data.frame(a = E(go)$e_class, b = E(go)$lty), test = 1)) {
        flag <- FALSE
      } else if (verbose) message("a 'e_class' should not match multiple linetypes, update 'lty'.")
    }
  }
  if (flag) E(go)$lty <- pcutils::tidai(E(go)$e_class, 1:4, fac = TRUE)

  # e_width
  if (!"width" %in% igraph::edge_attr_names(go)) {
    E(go)$width <- 1
  }

  class(go) <- c("metanet", "igraph")
  return(go)
}


#' Clean a igraph object
#'
#' @param go igraph, metanet objects
#' @param direct direct?
#'
#' @return a igraph object
#' @export
clean_igraph <- function(go, direct = NULL) {
  stopifnot(inherits(go, "igraph"))
  if (is.null(direct)) direct <- is.directed(go)
  go <- igraph::graph_from_data_frame(
    d = get_e(go)[, c("from", "to")],
    directed = direct,
    vertices = get_v(go)["name"]
  )
  go
}


#' Construct a network from edge_list dataframe
#'
#' @param edgelist first is source, second is target, others are annotation
#' @param vertex_df vertex metadata data.frame
#' @param direct logical
#' @param e_type set e_type
#' @param e_class set e_class
#' @return metanet
#' @export
#' @family build
#' @examples
#' data(edgelist)
#' edge_net <- c_net_from_edgelist(arc_count, vertex_df = arc_taxonomy)
#' edge_net <- c_net_set(edge_net, vertex_class = "Phylum", edge_width = "n")
#' c_net_plot(edge_net)
c_net_from_edgelist <- function(edgelist, vertex_df = NULL, direct = FALSE, e_type = NULL, e_class = NULL) {
  node_type <- c("from", "to")
  if (!all(c("from", "to") %in% colnames(edgelist))) {
    message("No 'from' and 'to' in the colnames(edgelist), use the first two columns as the 'from' and 'to'.")
    node_type <- colnames(edgelist)[1:2]
    colnames(edgelist)[1:2] <- c("from", "to")
  }
  edgelist <- data.frame(edgelist[, c("from", "to"), drop = FALSE], edgelist[, setdiff(colnames(edgelist), c("from", "to")), drop = FALSE])
  name <- NULL
  vertices <- data.frame(
    name = c(unique(edgelist$from), unique(edgelist$to)),
    `_type` = c(rep(node_type[1], length(unique(edgelist$from))), rep(node_type[2], length(unique(edgelist$to)))),
    check.names = FALSE, stringsAsFactors = FALSE
  )
  vertices <- dplyr::distinct(vertices, name, .keep_all = TRUE)
  if (!is.null(vertex_df)) {
    if (!"name" %in% colnames(vertex_df)) {
      vertex_df$name <- rownames(vertex_df)
      message("No 'name' in the colnames(vertex_df), use rownames(vertex_df) as the 'name'.")
    }
    vertex_df <- data.frame(vertex_df[, "name", drop = FALSE], vertex_df[, setdiff(colnames(vertex_df), "name"), drop = FALSE])
    vertices <- dplyr::full_join(vertices, vertex_df, by = "name")
    # vertices <- vertex_df
  }

  go <- igraph::graph_from_data_frame(edgelist, directed = direct, vertices = vertices)
  if (length(V(go)) == 0) {
    return(c_net_update(go, initialize = TRUE))
  }

  if (!is.null(e_type)) E(go)$e_type <- edgelist[, e_type]
  if (!is.null(e_class)) E(go)$e_class <- edgelist[, e_class]
  go <- c_net_update(go, initialize = TRUE)
  if (!"v_group" %in% colnames(vertices)) go <- c_net_set(go, vertex_group = "_type")
  if (!"v_class" %in% colnames(vertices)) go <- c_net_set(go, vertex_class = "_type")
  go
}

update_c_net_rda <- function() {
  otutab <- metadata <- taxonomy <- NULL
  data("otutab", package = "pcutils", envir = environment())
  t(otutab) -> totu
  metadata[, 3:10] -> env
  c_net_calculate(totu) -> corr
  c_net_build(corr, r_threshold = 0.65) -> co_net
  c_net_build(corr, r_threshold = 0.69) -> co_net_rmt

  c_net_calculate(totu, env) -> corr2
  c_net_build(corr2) -> co_net2

  co_net <- c_net_set(co_net, taxonomy, data.frame("Abundance" = colSums(totu)),
    vertex_class = "Phylum", vertex_size = "Abundance"
  )
  co_net2 <- c_net_set(co_net2, taxonomy, data.frame(name = colnames(env), env = colnames(env)),
    vertex_class = c("Phylum", "env")
  )
  co_net2 <- c_net_set(co_net2, data.frame("Abundance" = colSums(totu)), vertex_size = "Abundance")
  save(co_net, co_net2, co_net_rmt, file = "data/c_net.rda")
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import ggplot2
#' @import igraph
#' @importFrom dplyr select filter left_join arrange count mutate group_by summarise pull rename
#' @importFrom igraph V E
#' @importFrom utils data combn head tail read.csv write.csv packageName
#' @importFrom stats aggregate median var sd setNames runif relevel coef fitted cor coefficients formula lm pt
#' @importFrom stats time na.omit kmeans p.adjust density approx ks.test smooth.spline
#' @importFrom pcutils lib_ps mmscale get_cols trans guolv hebing update_param tidai change_fac_lev list_to_dataframe
#' @importFrom graphics legend hist par text symbols
#' @importFrom tibble rownames_to_column column_to_rownames
#' @importFrom reshape2 acast melt
## usethis namespace: end
NULL

## quiets concerns of R CMD check re: the .'s that appear in pipelines
if (getRversion() >= "2.15.1") utils::globalVariables(c("."))

#' Show MetaNet logo
#'
#' @return picture
#' @export
show_MetaNet_logo <- function() {
  pcutils::read.file(system.file("figures/MetaNet.png", package = "MetaNet"), all_yes = TRUE)
}

# Load the value of the option on package startup
.onAttach <- function(libname, pkgname) {
  add_metanet_shapes()
  packageStartupMessage("\033[38;5;246mIf you use the MetaNet package in published research, please cite:\n\nPeng, C., Huang, Z., Wei, X., Jiang, L., Zhu, X., Liu, Z., Chen, Q., Shen, X., Gao, P., and Jiang, C. (2025). MetaNet: a scalable and integrated tool for reproducible omics network analysis. Preprint at bioRxiv, https://doi.org/10.1101/2025.06.26.661636 https://doi.org/10.1101/2025.06.26.661636. \033[39m")
}

#' Pipe operator
#'
#' See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' @return The result of calling `rhs(lhs)`.
NULL

#' Assignment pipe
#'
#' See \code{magrittr::\link[magrittr:pipe]{\%<>\%}} for details.
#'
#' @name %<>%
#' @keywords internal
#' @export
#' @importFrom magrittr %<>%
#' @usage lhs \%<>\% rhs
#' @param lhs A value or the magrittr placeholder.
#' @param rhs A function call using the magrittr semantics.
#' @return The result of calling `rhs(lhs)`.
NULL

# flag determined by the correlation table from one table or two tables
t_flag <- \(corr){
  if (!nrow(corr) == ncol(corr)) {
    return(FALSE)
  }
  if (!all(rownames(corr) == colnames(corr))) {
    return(FALSE)
  }
  return(TRUE)
}

# 多列表是否一一对应
# test 对应的列需要跟剩下的列具有唯一映射
e_match <- function(df, test = NULL) {
  dis_df <- dplyr::distinct_all(df) %>% as.data.frame()
  if (is.null(test)) {
    return(all_same(c(nrow(dis_df), apply(dis_df, 2, \(i)length(unique(i))))))
  } else {
    return(all_same(c(nrow(dis_df), nrow(dplyr::distinct_all(dis_df[, test, drop = FALSE])))))
  }
}

all_same <- \(x){
  return(all(x == x[1]))
}

# choose the last not na value
condance <- \(aa){
  aa <- as.data.frame(aa)
  if (any(is.na(aa[, length(aa)]))) {
    res <- apply(aa, 1, \(x){
      tmp <- x[!is.na(x)]
      if (length(tmp) == 0) {
        return(NA)
      }
      return(tmp[length(tmp)])
    })
  } else {
    res <- aa[, length(aa), drop = TRUE]
  }
  res
}

# bind two df with same columns, the last df will replace first df
cbind_new <- \(df, df1){
  if (ncol(df) < 1) {
    return(df1)
  }
  if (ncol(df1) < 1) {
    return(df)
  }
  inter <- intersect(colnames(df1), colnames(df))
  la <- setdiff(colnames(df), inter)
  cbind(df[, la, drop = FALSE], df1)
}

twocol2vector <- \(df){
  if (ncol(df) < 2) {
    return(NULL)
  }
  if (!e_match(df, test = 1)) {
    stop("The columns are not one-to-one correspondence.")
  }
  df <- dplyr::distinct_all(df)
  setNames(df[, 2], df[, 1])
}

custom_sort <- function(x) {
  # 尝试将字符转换为数字并排序
  converted <- suppressWarnings(as.numeric(x))
  if (!any(is.na(converted))) {
    # 如果转换成功，则返回按数字排序的结果
    return(converted[order(converted)])
  } else {
    # 如果转换失败，则返回按字符排序的结果
    return(stringr::str_sort(x, numeric = TRUE))
  }
}

paste_df <- function(df, collapse = ",") {
  apply(df, 1, paste, collapse = collapse)
}

paste_df2pielist <- function(df, collapse = ",") {
  pcutils::strsplit2(df, split = collapse) %>%
    pcutils::t2() %>%
    as.list() %>%
    lapply(function(x) as.numeric(x))
}

deprecated <- function(old, new) {
  assign(old, new, envir = asNamespace(packageName()))
}

#' @export c_net_cal
deprecated("c_net_cal", c_net_calculate)

#' @export c_net_module
deprecated("c_net_module", module_detect)

#' @export c_net_lay
deprecated("c_net_lay", c_net_layout)

#' @export c_net_neighbors
deprecated("c_net_neighbors", c_net_ego)

#' @export as.metanet
deprecated("as.metanet", c_net_update)

#' @export c_net_skeleton
deprecated("c_net_skeleton", get_group_skeleton)

# ========4.plot========


#' Plot a metanet
#'
#' @param x metanet object
#' @param ... add
#'
#' @return plot
#' @exportS3Method
#' @method plot metanet
plot.metanet <- function(x, ...) {
  # 具有n_type的metanet的默认参数记录在这里，用于快速绘图。
  go <- x
  if (is.null(get_n(go)$n_type)) {
    c_net_plot(go, ...)
  } else if (get_n(go)$n_type == "skeleton") {
    skeleton_plot(go, ...)
  } else if (get_n(go)$n_type == "module") {
    default_arg <- list(
      labels_num = 0,
      group_legend_title = "Module"
    )
    do.call(c_net_plot, append(list(go = go), pcutils::update_param(default_arg, list(...))))
  } else if (get_n(go)$n_type == "venn") {
    nice_size <- ceiling(60 / sqrt(length(V(go)))) + 1
    default_arg <- list(
      labels_num = "all",
      vertex_size_range = list("Group" = c(1.5 * nice_size, 1.5 * nice_size), "elements" = c(0.5 * nice_size, 0.5 * nice_size)),
      vertex.shape = "circle",
      legend = FALSE, edge.curved = 0.3,
      edge.color = unique(V(go)$color)
    )
    do.call(c_net_plot, append(list(go = go), pcutils::update_param(default_arg, list(...))))
  } else if (get_n(go)$n_type == "twocol") {
    nice_size <- ceiling(60 / sqrt(length(V(go)))) + 1
    default_arg <- list(
      labels_num = 0,
      vertex.shape = "circle",
      edge_legend = FALSE,
      edge.color = "black"
    )
    do.call(c_net_plot, append(list(go = go), pcutils::update_param(default_arg, list(...))))
  } else if (get_n(go)$n_type == "ko_net") {
    nice_size <- ceiling(60 / sqrt(length(V(go)))) + 1
    default_arg <- list(
      labels_num = "all",
      vertex.shape = "circle",
      vertex_size_range = list("Pathway" = c(1.2 * nice_size, 1.2 * nice_size), "KOs" = c(0.6 * nice_size, 0.6 * nice_size)),
      edge_legend = FALSE,
      edge.color = "black",
      mark_alpha = 0.1
    )
    do.call(c_net_plot, append(list(go = go), pcutils::update_param(default_arg, list(...))))
  } else {
    c_net_plot(go, ...)
  }
}

get_net_main <- function(n_index) {
  main <- ""
  if (!is.null(n_index$n_type)) {
    switch(n_index$n_type,
      "single" = {
        main <- "Correlation network"
      },
      "bipartite" = {
        main <- "Bipartite network"
      },
      "multi_full" = {
        main <- "Multi-omics network"
      },
      "module" = {
        main <- paste0(n_index$n_modules, "-modules network")
      },
      "skeleton" = {
        main <- paste0(n_index$skeleton, " skeleton network")
      },
      "venn" = {
        main <- "Venn network"
      },
      default = {
        main <- ""
      }
    )
  }
  return(main)
}

scale_size_width <- function(tmp_v, tmp_e, vertex_size_range, edge_width_range) {
  {
    v_groups <- unique(tmp_v$v_group)
    nice_size <- ceiling(100 / sqrt(nrow(tmp_v))) + 1

    vertex_size_range_default <- rep(list(c(max(nice_size * 0.4, 3), min(nice_size * 1.6, 12))), length(v_groups))
    names(vertex_size_range_default) <- v_groups

    if (!is.null(vertex_size_range)) {
      if (!is.list(vertex_size_range)) vertex_size_range <- list(vertex_size_range)
      if (is.null(names(vertex_size_range))) {
        vertex_size_range <- rep(vertex_size_range, length(v_groups))
        names(vertex_size_range) <- v_groups
      }
      vertex_size_range <- pcutils::update_param(vertex_size_range_default, vertex_size_range)
    } else {
      vertex_size_range <- vertex_size_range_default
    }

    node_size_text <- setNames(as.list(numeric(length(v_groups))), v_groups)
    for (i in v_groups) {
      node_size_text[[i]] <- c(
        min(tmp_v[tmp_v$v_group == i, "size"], na.rm = TRUE),
        max(tmp_v[tmp_v$v_group == i, "size"], na.rm = TRUE)
      )
      tmp_v[tmp_v$v_group == i, "size"] <- do.call(pcutils::mmscale, append(
        list(tmp_v[tmp_v$v_group == i, "size"]),
        as.list(sort(vertex_size_range[[i]][1:2]))
      ))
    }
  }

  {
    edge_width_range_default <- vertex_size_range_default[[1]] / 6
    if (is.null(edge_width_range)) edge_width_range <- edge_width_range_default
    edge_width_range <- sort(edge_width_range)
    edge_width_text <- c(min(tmp_e$width, na.rm = TRUE), max(tmp_e$width, na.rm = TRUE))
    tmp_e$width <- pcutils::mmscale(tmp_e$width, edge_width_range[1], edge_width_range[2])
  }

  envir <- parent.frame()
  assign("node_size_text", node_size_text, envir)
  assign("edge_width_text", edge_width_text, envir)
  assign("vertex_size_range", vertex_size_range, envir)
  assign("edge_width_range", edge_width_range, envir)
  assign("tmp_e", tmp_e, envir)
  assign("tmp_v", tmp_v, envir)
}

some_custom_paras <- function(tmp_v, tmp_e, ...) {
  params <- list(...)
  params_name <- names(params)

  # 下列都是mapping好的颜色，形状等，无法单独修改某一个vertex或edge的颜色，形状等
  # ！！！考虑增加额外参数，用于单独修改某一个vertex或edge的颜色，形状等
  if (!"label.color" %in% colnames(tmp_v)) tmp_v$label.color <- "black"
  if ("vertex.size" %in% params_name) tmp_v$size <- params[["vertex.size"]]
  if ("vertex.color" %in% params_name) {
    tmp_v$color <- condance(data.frame(
      tmp_v$color,
      pcutils::tidai(tmp_v$v_class, params[["vertex.color"]], fac = TRUE)
    ))
  }
  if ("vertex.shape" %in% params_name) {
    tmp_v$shape <- condance(data.frame(
      tmp_v$shape,
      pcutils::tidai(tmp_v$v_group, params[["vertex.shape"]])
    ))
  }
  if ("vertex.label" %in% params_name) tmp_v$label <- params[["vertex.label"]]
  if ("vertex.label.color" %in% params_name) {
    tmp_v$label.color <- condance(data.frame(
      "black",
      pcutils::tidai(tmp_v$v_group, params[["vertex.label.color"]], fac = TRUE)
    ))
  }

  if ("edge.color" %in% params_name) {
    tmp_e$color <- condance(data.frame(
      tmp_e$color,
      pcutils::tidai(tmp_e$e_type, params[["edge.color"]], fac = TRUE)
    ))
  }
  if ("edge.lty" %in% params_name) {
    tmp_e$lty <- condance(data.frame(
      tmp_e$lty,
      pcutils::tidai(tmp_e$e_class, params[["edge.lty"]], fac = TRUE)
    ))
  }
  if ("edge.width" %in% params_name) tmp_e$width <- params[["edge.width"]]

  envir <- parent.frame()
  assign("tmp_e", tmp_e, envir)
  assign("tmp_v", tmp_v, envir)
}

pie_set_for_plot <- function(tmp_v, pie_value, pie_color) {
  if ("pie" %in% tmp_v$shape) {
    if (!is.null(pie_value)) {
      if (!is.data.frame(pie_value)) stop("pie_value must be a data.frame.")
      if (!"name" %in% colnames(pie_value)) {
        pie_value$name <- rownames(pie_value)
      }
      if (any(duplicated(pie_value$name))) {
        stop(
          "Duplicated name in annotation tables: ",
          paste0(pie_value$name[duplicated(pie_value$name)], collapse = ", ")
        )
      }
      tmp_merge_df <- left_join(tmp_v["name"], pie_value, by = "name")
      pie_value <- tmp_merge_df[, -1]
      pie_parts <- colnames(pie_value)

      pie_value[is.na(pie_value)] <- 0
      pie_value$`__others` <- ifelse(rowSums(pie_value) > 0, 0, 1)
      pie_value_list <- as.list(pcutils::t2(pie_value))

      default_pie_color <- setNames(pcutils::get_cols(length(pie_parts), "col1"), pie_parts)
      if (is.null(pie_color)) pie_color <- default_pie_color
      if (is.null(names(pie_color))) {
        pie_color <- setNames(pie_color, pie_parts)
      } else {
        pie_color <- pcutils::update_param(default_pie_color, pie_color)
      }

      pie_color <- pie_color[pie_parts]
      pie_color <- c(pie_color, `__others` = NA)
    } else {
      pie_value_list <- pie_color <- NULL
    }
  } else {
    pie_value_list <- pie_color <- NULL
  }
  envir <- parent.frame()
  assign("pie_value_list", pie_value_list, envir)
  assign("pie_color", pie_color, envir)
}

get_show_labels <- function(tmp_v, labels_num) {
  name <- size <- NULL
  if (is.null(labels_num)) {
    if (nrow(tmp_v) < 20) {
      labels_num <- "all"
    } else {
      labels_num <- 0
    }
  }
  {
    if (is.character(labels_num)) {
      if (identical(labels_num, "all")) {
        tmp_v %>% dplyr::pull(name) -> toplabel
      } else {
        intersect(tmp_v %>% dplyr::pull(name), labels_num) -> toplabel
      }
    } else {
      if (labels_num >= 1) {
        tmp_v %>%
          dplyr::top_n(labels_num, size) %>%
          dplyr::arrange(-size) %>%
          dplyr::pull(name) %>%
          head(labels_num) -> toplabel
      } else {
        tmp_v %>%
          dplyr::top_frac(labels_num, size) %>%
          dplyr::arrange(-size) %>%
          dplyr::pull(name) %>%
          head(ceiling(labels_num * nrow(tmp_v))) -> toplabel
      }
    }
    tmp_v$label <- ifelse(tmp_v$name %in% toplabel, tmp_v$label, NA)
  }
  return(tmp_v)
}

module_set_for_plot <- function(tmp_v, mark_module, mark_color) {
  if (mark_module) {
    new_modu <- as_module(setNames(tmp_v$module, tmp_v$name))
    new_modu[["others"]] <- NULL

    module_color <- pcutils::get_cols(length(new_modu))
    if (!is.null(mark_color)) module_color <- condance(data.frame(module_color, pcutils::tidai(names(new_modu), mark_color)))

    module_color <- setNames(module_color, names(new_modu))
    module_color <- module_color[names(module_color) != "others"]
  } else {
    new_modu <- module_color <- NULL
  }
  envir <- parent.frame()
  assign("new_modu", new_modu, envir)
  assign("module_color", module_color, envir)
}

get_module_coors <- function(go = NULL, coors = NULL, tmp_v = NULL, ori_coors = NULL, module_label_just = c(0.5, 0.5), rescale_flag = TRUE) {
  X <- Y <- module <- minx <- maxx <- miny <- maxy <- NULL
  if (is.null(go)) {
    if (is.null(tmp_v) && is.null(ori_coors)) message("input `tmp_v` and `ori_coors` when `go` is null.")
  } else {
    tmp_v <- get_v(go)
    ori_coors <- get_coors(coors, go)
  }
  module_coors <- dplyr::left_join(tmp_v[, c("name", "module")], ori_coors, by = "name")
  if (rescale_flag) module_coors <- dplyr::mutate(module_coors, X = mmscale(X, -1, 1), Y = mmscale(Y, -1, 1))
  module_coors <- dplyr::group_by(module_coors, module) %>%
    dplyr::summarise(minx = min(X), maxx = max(X), miny = min(Y), maxy = max(Y))
  module_label_just <- rep(module_label_just, 2)
  module_coors <- mutate(module_coors,
    X = minx + module_label_just[1] * (maxx - minx),
    Y = miny + module_label_just[2] * (maxy - miny)
  )
  return(module_coors)
}

produce_c_net_legends <- function(tmp_v, tmp_e, vertex_frame_width,
                                  legend_position, legend_number, legend_cex,
                                  node_size_text, edge_width_text,
                                  group_legend_title, group_legend_order,
                                  color_legend, color_legend_order,
                                  size_legend, size_legend_title,
                                  edge_legend, edge_legend_title, edge_legend_order,
                                  width_legend, width_legend_title,
                                  lty_legend, lty_legend_title, lty_legend_order,
                                  module_legend, module_legend_title, module_legend_order, module_color, mark_alpha,
                                  pie_legend, pie_legend_title, pie_legend_order, pie_color,
                                  ...) {
  color <- e_type <- lty <- e_class <- v_class <- shape <- left_leg_x <- right_leg_x <- NULL
  v_highlight <- v_highlight_from <- v_highlight_to <- NULL

  legend_position_default <- c(left_leg_x = -2, left_leg_y = 1, right_leg_x = 1.2, right_leg_y = 1)

  if (is.null(legend_position)) legend_position <- legend_position_default
  if (is.null(names(legend_position))) {
    legend_position <- setNames(legend_position, names(legend_position_default)[seq_along(legend_position)])
  }
  legend_position <- pcutils::update_param(legend_position_default, legend_position)
  here_env <- environment()
  lapply(names(legend_position), \(i){
    assign(i, legend_position[i], here_env)
  })

  vgroups <- pcutils::change_fac_lev(tmp_v$v_group, group_legend_order)
  vgroups <- levels(vgroups)

  if (color_legend) {
    # ！！！考虑添加更多的形状，至少是21:25，形状为pie时添加额外legend

    if (is.null(group_legend_title)) {
      group_legend_title <- setNames(vgroups, vgroups)
    } else if (is.null(names(group_legend_title))) {
      group_legend_title <- setNames(rep(group_legend_title, len = length(vgroups)), vgroups)
    }

    this_shape <- unique(tmp_v$shape)
    if (!all(this_shape %in% names(default_v_shape))) {
      default_v_shape <- pcutils::update_param(
        default_v_shape,
        setNames(rep(21, length(this_shape)), this_shape)
      )
    }

    if ("v_highlight" %in% colnames(tmp_v)) {
      tmp_v <- dplyr::arrange(tmp_v, -v_highlight)
    }
    for (g_i in vgroups) {
      if ("count" %in% names(tmp_v)) {
        tmp_v1 <- tmp_v[tmp_v$v_group == g_i, c("v_class", "color", "shape", "count")]
      } else {
        tmp_v1 <- tmp_v[tmp_v$v_group == g_i, c("v_class", "color", "shape")]
      }

      tmp_v1$v_class <- factor(tmp_v1$v_class, levels = custom_sort(unique(tmp_v1$v_class)))

      vclass <- pcutils::change_fac_lev(tmp_v1$v_class, color_legend_order)
      vclass <- levels(vclass)

      node_cols <- dplyr::distinct(tmp_v1, color, v_class)
      node_cols <- setNames(node_cols$color, node_cols$v_class)
      node_shapes <- dplyr::distinct(tmp_v1, shape, v_class)
      node_shapes <- setNames(node_shapes$shape, node_shapes$v_class)

      if (legend_number) {
        eee <- table(tmp_v1$v_class)
        if (!is.null(attributes(tmp_v)$skeleton)) {
          eee <- setNames(tmp_v1$count, tmp_v1$v_class)
          # legend_number <- FALSE
        }
        le_text <- paste(vclass, eee[vclass], sep = ": ")
      } else {
        le_text <- vclass
      }
      if (length(le_text) == 0) le_text <- ""
      legend(left_leg_x, left_leg_y,
        cex = 0.7 * legend_cex, adj = 0, pt.lwd = vertex_frame_width,
        legend = le_text, title.cex = 0.8 * legend_cex,
        title = group_legend_title[g_i], title.font = 2, title.adj = 0,
        col = "black", pt.bg = node_cols[vclass], bty = "n", pch = default_v_shape[node_shapes[vclass]]
      )

      left_leg_y <- left_leg_y - (length(vclass) * 0.12 + 0.2) * legend_cex
    }
  }

  if (module_legend) {
    tmp_v$module <- factor(tmp_v$module, levels = custom_sort(unique(tmp_v$module)))
    modules <- pcutils::change_fac_lev(tmp_v$module, module_legend_order)
    modules <- levels(modules)

    if (legend_number) {
      eee <- table(tmp_v$module)
      le_text <- paste(modules, eee[modules], sep = ": ")
    } else {
      le_text <- modules
    }
    legend(left_leg_x, left_leg_y,
      cex = 0.7 * legend_cex, adj = 0,
      legend = le_text, title.cex = 0.8 * legend_cex,
      title = module_legend_title, title.font = 2, title.adj = 0,
      fill = pcutils::add_alpha(module_color[modules], mark_alpha),
      border = module_color[modules], bty = "n"
    )

    left_leg_y <- left_leg_y - (length(modules) * 0.12 + 0.2) * legend_cex
  }

  if (pie_legend) {
    pie_color <- pie_color[names(pie_color) != "__others"]
    pies <- pcutils::change_fac_lev(names(pie_color), pie_legend_order)
    pies <- levels(pies)

    le_text <- pies
    legend(left_leg_x, left_leg_y,
      cex = 0.7 * legend_cex, adj = 0,
      legend = le_text, title.cex = 0.8 * legend_cex,
      title = pie_legend_title, title.font = 2, title.adj = 0,
      fill = pie_color[pies],
      border = "black", bty = "n"
    )
    left_leg_y <- left_leg_y - (length(pies) * 0.12 + 0.2) * legend_cex
  }

  if (size_legend) {
    legend(
      x = right_leg_x, y = right_leg_y,
      cex = 0.7 * legend_cex, title.font = 2, title = size_legend_title, title.adj = 0,
      legend = c(
        paste(lapply(node_size_text[vgroups], \(i)round(i[1], 3)), collapse = "/ "),
        paste(lapply(node_size_text[vgroups], \(i)round(i[2], 3)), collapse = "/ ")
      ),
      adj = 0, title.cex = 0.8 * legend_cex,
      col = "black", bty = "n", pch = 21, pt.cex = c(min(tmp_v$size), max(tmp_v$size)) * legend_cex / 5
    )
    right_leg_y <- right_leg_y - 0.5 * legend_cex
  }

  if (nrow(tmp_e) == 0) {
    return(invisible())
  }

  if (edge_legend) {
    if ("v_highlight_from" %in% colnames(tmp_e)) {
      tmp_e <- dplyr::arrange(tmp_e, -v_highlight_from, -v_highlight_to)
    }

    tmp_e$e_type <- factor(tmp_e$e_type, levels = custom_sort(unique(tmp_e$e_type)))
    edges <- pcutils::change_fac_lev(tmp_e$e_type, edge_legend_order)
    edges <- levels(edges)
    edge_cols <- dplyr::distinct(tmp_e, color, e_type)
    edge_cols <- setNames(edge_cols$color, edge_cols$e_type)
    if (legend_number) {
      eee <- table(tmp_e$e_type)
      le_text <- paste(edges, eee[edges], sep = ": ")
    } else {
      le_text <- edges
    }
    legend(right_leg_x, right_leg_y,
      cex = 0.7 * legend_cex, title.font = 2, title = edge_legend_title, title.adj = 0,
      legend = le_text, adj = 0, title.cex = 0.8 * legend_cex,
      col = edge_cols[edges], bty = "n", lty = 1
    )
    right_leg_y <- right_leg_y - (length(unique(tmp_e$color)) * 0.12 + 0.2) * legend_cex
  }

  if (width_legend) {
    legend(right_leg_x, right_leg_y,
      cex = 0.7 * legend_cex, title.font = 2, title = width_legend_title, title.adj = 0,
      legend = edge_width_text %>% round(., 3), adj = 0, title.cex = 0.8 * legend_cex,
      col = "black", bty = "n", lty = 1, lwd = c(min(tmp_e$width), max(tmp_e$width))
    )
    right_leg_y <- right_leg_y - 0.5 * legend_cex
  }

  if (lty_legend) {
    edges <- pcutils::change_fac_lev(tmp_e$e_class, lty_legend_order)
    edges <- levels(edges)
    edge_ltys <- dplyr::distinct(tmp_e, lty, e_class)
    edge_ltys <- setNames(edge_ltys$lty, edge_ltys$e_class)

    if (legend_number) {
      eee <- table(tmp_e$e_class)
      le_text <- paste(edges, eee[edges], sep = ": ")
    } else {
      le_text <- edges
    }
    legend(right_leg_x, right_leg_y,
      cex = 0.7 * legend_cex, title.font = 2, title = lty_legend_title, title.adj = 0,
      legend = le_text, adj = 0, title.cex = 0.8 * legend_cex,
      col = "black", bty = "n", lty = edge_ltys[edges]
    )
  }
}

#' Plot a metanet
#'
#' @param go an igraph or metanet object
#' @param coors the coordinates you saved
#' @param ... additional parameters for \code{\link[igraph]{igraph.plotting}}
#' @param labels_num show how many labels, >1 indicates number, <1 indicates fraction, "all" indicates all.
#' @param vertex_size_range the vertex size range, e.g. c(1,10)
#' @param edge_width_range the edge width range, e.g. c(1,10)
#'
#' @param plot_module logical, plot module?
#' @param mark_module logical, mark the modules?
#' @param mark_color mark color
#' @param mark_alpha mark fill alpha, default 0.3
#' @param module_label show module label?
#' @param module_label_cex module label cex
#' @param module_label_color module label color
#' @param module_label_just module label just, default c(0.5,0.5)
#'
#' @param pie_value a dataframe using to plot pie (with rowname or a "name" column)
#' @param pie_color color vector
#'
#' @param legend all legends
#' @param legend_number legend with numbers
#' @param legend_cex character expansion factor relative to current par("cex"), default: 1
#' @param legend_position legend_position, default: c(left_leg_x=-1.9,left_leg_y=1,right_leg_x=1.2,right_leg_y=1)
#'
#' @param group_legend_title group_legend_title, length must same to the numbers of v_group
#' @param group_legend_order group_legend_order vector
#' @param color_legend logical
#' @param color_legend_order color_legend_order vector
#' @param size_legend logical
#' @param size_legend_title size_legend_title
#'
#' @param edge_legend logical
#' @param edge_legend_title edge_legend_title
#' @param edge_legend_order edge_legend_order vector, e.g. c("positive","negative")
#' @param width_legend logical
#' @param width_legend_title width_legend_title
#'
#' @param lty_legend logical
#' @param lty_legend_title lty_legend_title
#' @param lty_legend_order lty_legend_order
#'
#' @param module_legend logical
#' @param module_legend_title module_legend_title
#' @param module_legend_order module_legend_order
#'
#' @param pie_legend logical
#' @param pie_legend_title pie_legend_title
#' @param pie_legend_order pie_legend_order
#'
#' @param seed random seed, default:1234, make sure each plot is the same.
#' @param params_list a list of parameters, e.g. list(edge_legend = TRUE, lty_legend = FALSE), when the parameter is duplicated, the format argument will be used rather than the argument in params_list.
#' @param rescale Logical constant, whether to rescale the coordinates to the (-1,1).
#' @param arrow_size_cex arrow size cex, default 1, relative to the vertex size
#' @param arrow_width_cex arrow width cex, default 1, relative to the vertex size
#' @param label_cex label cex, default 1, relative to the vertex size
#'
#' @family plot
#' @return a network plot
#' @export
#'
#' @examples
#' data("c_net")
#' c_net_plot(co_net)
#' c_net_plot(co_net2)
#' c_net_plot(multi1)
c_net_plot <- function(go, coors = NULL, ..., labels_num = NULL,
                       vertex_size_range = NULL, edge_width_range = NULL,
                       plot_module = FALSE,
                       mark_module = FALSE, mark_color = NULL, mark_alpha = 0.3,
                       module_label = FALSE, module_label_cex = 2, module_label_color = "black",
                       module_label_just = c(0.5, 0.5),
                       pie_value = NULL, pie_color = NULL,
                       legend = TRUE, legend_number = FALSE, legend_cex = 1,
                       legend_position = c(left_leg_x = -2, left_leg_y = 1, right_leg_x = 1.2, right_leg_y = 1),
                       group_legend_title = NULL, group_legend_order = NULL,
                       color_legend = TRUE, color_legend_order = NULL,
                       size_legend = FALSE, size_legend_title = "Node Size",
                       edge_legend = TRUE, edge_legend_title = "Edge type", edge_legend_order = NULL,
                       width_legend = FALSE, width_legend_title = "Edge width",
                       lty_legend = FALSE, lty_legend_title = "Edge class", lty_legend_order = NULL,
                       module_legend = FALSE, module_legend_title = "Module", module_legend_order = NULL,
                       pie_legend = FALSE, pie_legend_title = "Pie part", pie_legend_order = NULL,
                       label_cex = 1, arrow_size_cex = 1, arrow_width_cex = 1,
                       params_list = NULL, rescale = FALSE,
                       seed = 1234) {
  if (!is.null(params_list)) {
    as.list(match.call()[-1]) -> set_params_list
    set_params_list[["params_list"]] <- NULL
    for (i in seq_along(params_list)) {
      if (names(params_list)[i] %in% names(set_params_list)) {
        message("The parameter `", names(params_list)[i], "` is duplicated, the format argument will be used.")
      }
    }
    pcutils::update_param(params_list, set_params_list) -> set_params_list
    do.call(c_net_plot, set_params_list)
    return(invisible())
  }

  if (length(V(go)) == 0) {
    message("The network is empty.")
    return(invisible())
  }
  new_modu <- module_color <- node_size_text <- edge_width_text <- NULL
  set.seed(seed)

  if (!is_metanet(go)) go <- c_net_update(go)
  # go <- c_net_update(go)

  # modules
  if (plot_module) {
    go <- to_module_net(go)
    if (is.null(group_legend_title)) group_legend_title <- "Module"
  }

  # get network type
  get_net_main(get_n(go)) -> main
  get_v(go) -> tmp_v
  get_e(go) -> tmp_e

  # get coordinates
  ori_coors <- get_coors(coors, go, seed = seed)
  coors <- ori_coors[, c("X", "Y")] %>% as.matrix()
  if (is.null(attributes(ori_coors)$curved)) {
    edge_curved <- NULL
  } else {
    edge_curved <- attributes(ori_coors)$curved$curved
  }

  # scale the size and width
  scale_size_width(tmp_v, tmp_e, vertex_size_range, edge_width_range)

  # some custom parameters
  some_custom_paras(tmp_v, tmp_e, ...)

  # set pie
  pie_value_list <- NULL
  pie_set_for_plot(tmp_v, pie_value, pie_color)
  if (is.null(pie_value) || !"pie" %in% tmp_v$shape) pie_legend <- FALSE
  if (is.null(pie_color)) {
    pie_color_list <- NULL
  } else {
    pie_color_list <- list(pie_color)
  }

  # show labels
  tmp_v <- get_show_labels(tmp_v, labels_num)

  # modules set
  module_set_for_plot(tmp_v, mark_module, mark_color)
  if (!mark_module) module_legend <- FALSE

  if (any(igraph::is.loop(go))) go <- clean_multi_edge_metanet(go)
  # main plot
  {
    old_xpd <- graphics::par(mar = c(4, 2, 2, 2), xpd = TRUE)
    on.exit(graphics::par(old_xpd), add = TRUE)

    igraph::plot.igraph(go,
      layout = coors,
      vertex.size = tmp_v$size,
      vertex.color = tmp_v$color,
      vertex.shape = tmp_v$shape,
      vertex.label.color = tmp_v$label.color,
      edge.color = tmp_e$color,
      edge.lty = tmp_e$lty,
      edge.width = tmp_e$width,
      mark.groups = new_modu,
      mark.col = pcutils::add_alpha(module_color[names(new_modu)], mark_alpha),
      mark.border = module_color[names(new_modu)],
      vertex.pie = pie_value_list,
      vertex.pie.color = pie_color_list,
      ...,
      vertex.size2 = tmp_v$size / 2,
      vertex.frame.width = 0.5,
      main = main,
      rescale = rescale,
      vertex.label.font = 1,
      vertex.label.cex = label_cex * 0.05 * tmp_v$size,
      vertex.label = tmp_v$label,
      edge.arrow.size = arrow_size_cex * 0.2 * tmp_e$width,
      edge.arrow.width = arrow_width_cex * 1 * tmp_e$width,
      edge.curved = edge_curved,
      margin = c(0, 0, 0, 0)
    )
  }

  # add module_label
  if (module_label) {
    rescale_flag <- TRUE
    params <- list(...)
    if ("rescale" %in% names(params)) {
      if (!params[["rescale"]]) rescale_flag <- FALSE
    }
    module_coors <- get_module_coors(
      tmp_v = tmp_v, ori_coors = ori_coors,
      module_label_just = module_label_just, rescale_flag = rescale_flag
    )

    n_module <- nrow(module_coors)
    module_label_cex <- rep(module_label_cex, n_module)
    module_label_color <- rep(module_label_color, n_module)
    for (i in seq_len(n_module)) {
      text(
        x = module_coors[i, "X"], y = module_coors[i, "Y"],
        labels = module_coors[i, "module"],
        cex = module_label_cex, col = module_label_color[i]
      )
    }
  }

  if (!legend) {
    return(invisible())
  }

  if ("vertex.frame.width" %in% names(list(...))) {
    vertex_frame_width <- list(...)[["vertex.frame.width"]]
  } else {
    vertex_frame_width <- 0.5
  }
  # produce legends
  if (grepl("skeleton", main)) attributes(tmp_v)$skeleton <- TRUE
  produce_c_net_legends(
    tmp_v, tmp_e, vertex_frame_width,
    legend_position, legend_number, legend_cex,
    node_size_text, edge_width_text,
    group_legend_title, group_legend_order,
    color_legend, color_legend_order,
    size_legend, size_legend_title,
    edge_legend, edge_legend_title, edge_legend_order,
    width_legend, width_legend_title,
    lty_legend, lty_legend_title, lty_legend_order,
    module_legend, module_legend_title, module_legend_order, module_color, mark_alpha,
    pie_legend, pie_legend_title, pie_legend_order, pie_color, ...
  )
}

#' Print method for 'corr' objects
#'
#' @param x 'corr' object
#' @param ... additional arguments
#'
#' @exportS3Method
#' @method print corr
#' @return No value
print.corr <- function(x, ...) {
  if (is.null(x$r)) {
    cat("NULL corr object.\n")
    return(invisible())
  }
  if (attributes(x)$method %in% c("spearman", "pearson", "sparcc")) {
    cat("Correlation table:\n")
  } else {
    cat("Similarity (1-Distance) table:\n")
  }
  cat("Table dimensions:", nrow(x$r), "rows,", ncol(x$r), "columns\n")
  cat("Use method: ", attributes(x)$method, "\n")
}

#' Summary method for 'corr' objects
#'
#' @param object 'corr' object
#' @param ... Additional arguments
#'
#' @return No value
#' @exportS3Method
#' @method summary corr
#'
summary.corr <- function(object, ...) {
  if (is.null(object$r)) {
    cat("NULL corr object.\n")
    return(invisible())
  }
  if (attributes(object)$method %in% c("spearman", "pearson", "sparcc")) {
    cat("Correlation table:\n")
  } else {
    cat("Similarity (1-Distance) table:\n")
  }

  cat("Distribution of r:\n")
  if (t_flag(object$r)) {
    print(summary(object$r[upper.tri(object$r)]))
  } else {
    print(summary(as.vector(object$r)))
  }

  if (!attributes(object)$method %in% c("spearman", "pearson", "sparcc")) {
    return(invisible())
  }
  cat("Distribution of p:\n")
  if (t_flag(object$p)) {
    print(summary(object$p[upper.tri(object$p)]))
  } else {
    print(summary(as.vector(object$p)))
  }
  if ("p.adjust" %in% names(object)) {
    cat("Distribution of p.adjust:\n")
    if (t_flag(object$p.adjust)) {
      print(summary(object$p.adjust[upper.tri(object$p.adjust)]))
    } else {
      print(summary(as.vector(object$p.adjust)))
    }
  }
}

#' Print method for 'coors' objects
#'
#' @param x 'coors' object
#' @param ... additional arguments
#' @return No value
#' @exportS3Method
#' @method print coors
print.coors <- function(x, ...) {
  cat("Coordinates:\n")
  cat("Table dimensions:", nrow(x), "rows,", ncol(x), "columns\n")
  if (!is.null(attributes(x)$curved)) cat("\nWith curved information")
}


#' Print method for 'metanet' objects
#'
#' @param x 'metanet' object
#' @param ... Additional arguments
#' @exportS3Method
#' @method print metanet
#' @return No value
print.metanet <- function(x, ...) {
  pcutils::dabiao(" metanet ", print = TRUE)
  print.igraph(x)
}

#' Print method for 'ggig' objects
#'
#' @param x 'ggig' object
#' @param ... Additional arguments
#'
#' @return No value
#' @exportS3Method
#' @method print ggig
print.ggig <- function(x, ...) {
  pcutils::dabiao(" ggig ", print = TRUE)
  pcutils::dabiao("use `plot()` to visualize ggig object.", char = " ", print = TRUE)
}


#' Print method for 'metanet_compare' objects
#' @param x 'metanet_compare' object
#' @param ... Additional arguments
#' @return No value
#' @exportS3Method
#' @method print metanet_compare
print.metanet_compare <- function(x, ...) {
  pcutils::dabiao("Comparsion result ", print = TRUE)
  print(x$net_similarity)
  pcutils::dabiao("\n\nuse `plot()` to visualize metanet_compare object.", char = " ", print = TRUE)
}

#' Print method for 'robust' objects
#' @param x 'robust' object
#' @param ... Additional arguments
#' @return No value
#' @exportS3Method
#' @method print robust
print.robust <- function(x, ...) {
  pcutils::dabiao(" robust ", print = TRUE)
  pcutils::dabiao("use `plot()` to visualize robust object.", char = " ", print = TRUE)
}

#' Print method for 'robustness' objects
#' @param x 'robustness' object
#' @param ... Additional arguments
#' @return No value
#' @exportS3Method
#' @method print robustness
print.robustness <- function(x, ...) {
  pcutils::dabiao(" robustness ", print = TRUE)
  pcutils::dabiao("use `plot()` to visualize robustness object.", char = " ", print = TRUE)
}

#' Print method for 'vulnerability' objects
#' @param x 'vulnerability' object
#' @param ... Additional arguments
#' @return No value
#' @exportS3Method
#' @method print vulnerability
print.vulnerability <- function(x, ...) {
  pcutils::dabiao(" vulnerability ", print = TRUE)
  pcutils::dabiao("use `plot()` to visualize vulnerability object.", char = " ", print = TRUE)
}

#' Print method for 'cohesion' objects
#' @param x 'cohesion' object
#' @param ... Additional arguments
#' @return No value
#' @exportS3Method
#' @method print cohesion
print.cohesion <- function(x, ...) {
  pcutils::dabiao(" cohesion ", print = TRUE)
  pcutils::dabiao("use `plot()` to visualize cohesion object.", char = " ", print = TRUE)
}

# ==========2.1 manipulate========

#' Set basic attributes from totu table
#'
#' @param go metanet an igraph object
#' @param ... some data.frames to annotate go
#' @param vertex_group choose which column to be vertex_group (map to vertex_shape)
#' @param vertex_class choose which column to be vertex_class (map to vertex_color)
#' @param vertex_size choose which column to be vertex_size (map to vertex_size)
#' @param edge_type choose which column to be edge_type (map to edge_color)
#' @param edge_class choose which column to be edge_class (map to edge_linetype)
#' @param edge_width choose which column to be edge_width (map to edge_width)
#' @param node_break node_break if v_class is numeric, default: 5
#' @param edge_break edge_break if e_type is numeric, default: 5
#' @param initialize initialize, default: TRUE
#'
#' @return a metanet object
#' @export
#' @family build
#' @examples
#' data("otutab", package = "pcutils")
#' t(otutab) -> totu
#' metadata[, 3:10] -> env
#'
#' data("c_net")
#' co_net <- c_net_set(co_net, taxonomy, data.frame("Abundance" = colSums(totu)),
#'   vertex_class = "Phylum", vertex_size = "Abundance"
#' )
#' co_net2 <- c_net_set(co_net2, taxonomy, data.frame(name = colnames(env), env = colnames(env)),
#'   vertex_class = c("Phylum", "env")
#' )
#' co_net2 <- c_net_set(co_net2, data.frame("Abundance" = colSums(totu)), vertex_size = "Abundance")
c_net_set <- function(go, ..., vertex_group = "v_group", vertex_class = "v_class", vertex_size = "size",
                      edge_type = "e_type", edge_class = "e_class", edge_width = "width",
                      node_break = 5, edge_break = 5, initialize = TRUE) {
  size <- e_class <- width <- NULL
  c_net_update(go, verbose = FALSE) -> go
  name <- v_group <- v_class <- e_type <- color <- NULL

  # annotation vertex
  anno_dfs <- list(...)
  if (length(anno_dfs) > 0) {
    anno_dfs2 <- list()
    for (i in seq_len(length(anno_dfs))) {
      x <- anno_dfs[[i]]
      if ("name" %in% colnames(x)) {
        rownames(x) <- x$name
        x <- dplyr::select(x, -name)
      }
      anno_dfs2[[i]] <- x
    }

    if (any(duplicated(lapply(anno_dfs2, names) %>% unlist()))) stop("Duplicated column names in your annotation tables, please check!")

    Reduce(\(x, y)merge(x, y, by = "row.names", all = TRUE) %>%
      tibble::column_to_rownames("Row.names"), anno_dfs2) -> all_anno

    anno_vertex(go, all_anno) -> go
  }
  get_v(go) -> v_index
  get_e(go) -> e_index

  # set something
  # ！！！这里的set要改成跟c_net_update一样的逻辑
  if (!setequal(vertex_group, "v_group")) dplyr::select(v_index, v_group, !!vertex_group) %>% condance() -> v_index$v_group

  if (!setequal(vertex_class, "v_class")) {
    old_color <- twocol2vector(v_index[, c("v_class", "color")])
    new_color_name <- c()

    # 给每一个v_group加上v_class调整颜色
    # 可能某一个group用numeric做v_class，所以要分开上色
    for (i in unique(v_index$v_group)) {
      tmp_index <- v_index[v_index$v_group == i, ]
      tmp_v_class <- dplyr::select(tmp_index, v_class, !!vertex_class) %>% condance()
      if (identical(tmp_v_class, tmp_index$v_class)) {
        new_color_name <- c(new_color_name, unique(tmp_index$v_class))
        next
      }
      if (is.numeric(tmp_v_class)) {
        tmp_v_color <- color_generate(tmp_v_class, n_break = node_break, mode = "v")
        tmp_v_class <- color_generate(tmp_v_class, n_break = node_break, mode = "label")
        v_index[v_index$v_group == i, "color"] <- tmp_v_color
      } else {
        new_color_name <- c(new_color_name, unique(tmp_index$v_class))
      }
      v_index[v_index$v_group == i, "v_class"] <- as.character(tmp_v_class)
    }
    # 总体分类颜色是否改变，没变的话就不该，变了的话全部重新赋
    new_color_name <- unique(new_color_name)
    if (!all(new_color_name %in% names(old_color))) {
      new_color <- setNames(pcutils::get_cols(length(new_color_name), pal = default_v_color), new_color_name)
      v_index$color <- condance(data.frame(
        v_index$color,
        pcutils::tidai(v_index$v_class, new_color)
      ))
    }
  }

  if (!setequal(vertex_size, "size")) dplyr::select(v_index, size, !!vertex_size) %>% condance() -> v_index$size

  if (!setequal(edge_type, "e_type")) {
    tmp_e_type <- dplyr::select(e_index, e_type, !!edge_type) %>% condance()
    if (!identical(tmp_e_type, e_index$e_type)) {
      tmp_e_color <- color_generate(tmp_e_type, edge_break, mode = "e")
      tmp_e_type <- color_generate(tmp_e_type, edge_break, mode = "label")
      e_index$e_type <- tmp_e_type
      e_index$color <- tmp_e_color
    }
  }
  if (!setequal(edge_class, "e_class")) dplyr::select(e_index, e_class, !!edge_class) %>% condance() -> e_index$e_class
  if (!setequal(edge_width, "width")) dplyr::select(e_index, width, !!edge_width) %>% condance() -> e_index$width

  as.list(v_index) -> igraph::vertex.attributes(go)
  as.list(e_index) -> igraph::edge.attributes(go)

  c_net_update(go, initialize = initialize, verbose = FALSE) -> go2
  return(go2)
}


#' Is this object a metanet object?
#'
#' @param go a test object
#'
#' @return logical
#' @export
#' @aliases is.metanet
#' @family manipulate
#' @examples
#' data(c_net)
#' is_metanet(co_net)
is_metanet <- function(go) {
  is.igraph(go) & inherits(go, "metanet")
}

#' Get vertex information
#'
#' @param go metanet object
#' @param index attribute name, default: NULL
#' @family manipulate
#' @return data.frame
#' @export
get_v <- function(go, index = NULL) {
  name <- NULL
  # 规定name只能为字符
  if (is.null(V(go)$name)) V(go)$name <- as.character(V(go))
  # df <- as.data.frame(igraph::vertex.attributes(go))
  igraph::as_data_frame(go, what = "vertices") -> df
  df <- dplyr::select(df, name, dplyr::everything())
  rownames(df) <- NULL
  if (!is.null(index)) {
    return(dplyr::select(df, !!index))
  } else {
    return(df)
  }
}

#' Get edge information
#' @param go metanet object
#' @param index attribute name, default: NULL
#' @return data.frame
#' @family manipulate
#' @export
get_e <- function(go, index = NULL) {
  id <- NULL
  tmp_e <- cbind_new(igraph::as_data_frame(go), data.frame(id = seq_len(igraph::ecount(go))))
  tmp_e <- dplyr::select(tmp_e, id, dplyr::everything())
  if (!is.null(index)) {
    return(dplyr::select(tmp_e, !!index))
  } else {
    return(tmp_e)
  }
}

#' Get network information
#'
#' @param go metanet object
#' @param index attribute name, default: NULL
#' @param simple logical, get simple index
#' @family manipulate
#' @return data.frame
#' @export
get_n <- function(go, index = NULL, simple = FALSE) {
  gls <- igraph::graph.attributes(go)
  if (simple) {
    gls <- lapply(gls, \(x){
      if (inherits(x, "data.frame")) {
        return(NULL)
      }
      if (is.array(x)) {
        return(NULL)
      }
      if (is.list(x)) {
        return(NULL)
      }
      if (length(x) > 1) {
        return(NULL)
      }
      return(x)
    })
  } else {
    gls <- lapply(gls, \(x){
      if (inherits(x, "data.frame")) {
        return(paste0(ncol(x), "-columns df"))
      }
      if (is.array(x)) {
        return(paste0(length(x), "-elements ", class(x)))
      }
      if (is.list(x)) {
        return(paste0(length(x), "-elements ", class(x)))
      }
      if (length(x) > 1) {
        return(paste0(length(x), "-elements vector"))
      }
      return(x)
    })
  }
  df <- as.data.frame(do.call(cbind, gls))
  if (!is.null(index)) {
    return(dplyr::select(df, !!index))
  } else {
    return(df)
  }
}

#' Filter a network according to some attributes
#'
#' @param go metanet object
#' @param ... some attributes of vertex and edge
#' @param mode "v" or "e"
#'
#' @return metanet
#' @export
#' @family manipulate
#' @examples
#' data("multi_net")
#' c_net_filter(multi1, v_group %in% c("omic1", "omic2"))
c_net_filter <- function(go, ..., mode = "v") {
  if (mode == "v") {
    go1 <- filter_v(go, ...)
  } else if (mode == "e") {
    go1 <- filter_e(go, ...)
  } else {
    stop("mode should be 'v' or 'e'")
  }
  if (length(V(go1)) == 0) {
    message("The network is empty.")
  }
  go1
}

filter_v <- function(go, ...) {
  get_v(go) -> tmp_v
  tmp_v <- dplyr::filter(tmp_v, ...)
  tmp_v$name -> vid
  igraph::subgraph(go, vid) -> go1
  class(go1) <- c("metanet", "igraph")
  go1
}

filter_e <- function(go, ...) {
  get_e(go) -> tmp_e
  tmp_e <- dplyr::filter(tmp_e, ...)
  tmp_e$id -> eid
  igraph::subgraph.edges(go, eid) -> go1
  class(go1) <- c("metanet", "igraph")
  go1
}


#' Union two networks
#'
#' @param go1 metanet object
#' @param go2 metanet object
#' @param ... add
#'
#' @return metanet
#' @export
#' @family manipulate
#' @examples
#' g1 <- make_graph(edges = c("1", 2, 2, 3, 3, 4, 4, 5, 5, 1), directed = FALSE) %>% as.metanet()
#' g2 <- make_graph(edges = c("4", 5, 5, 6, 6, 7, 7, 8, 8, 4), directed = FALSE) %>% as.metanet()
#' par(mfrow = c(1, 3))
#' plot(c_net_union(g1, g2))
#' plot(c_net_intersect(g1, g2))
#' plot(c_net_difference(g1, g2))
c_net_union <- function(go1, go2, ...) {
  c_net_two_do(go1, go2, func = igraph::union, ...)
}

#' Intersect two networks
#'
#' @inheritParams c_net_union
#' @return metanet
#' @export
c_net_intersect <- function(go1, go2, ...) {
  c_net_two_do(go1, go2, func = igraph::intersection, keep.all.vertices = FALSE, ...)
  # common_nodes <- intersect(V(go1)$name, V(go2)$name)
  # c_net_filter(go1, name%in%common_nodes)
}

#' Difference two networks
#'
#' @inheritParams c_net_union
#' @return metanet
#' @export
c_net_difference <- function(go1, go2, ...) {
  c_net_two_do(go1, go2, func = igraph::difference, ...)
}

c_net_two_do <- function(go1, go2, func = igraph::union, ...) {
  stopifnot(is_directed(go1) == is_directed(go2))

  tmp_v1 <- get_v(go1)
  tmp_v2 <- get_v(go2)
  cols <- intersect(colnames(tmp_v1), colnames(tmp_v2))
  tmp_v <- rbind(tmp_v1[cols], tmp_v2[cols])
  message("Duplicated vertexes: ", sum(duplicated(tmp_v$name)), "\nUse the attributes of the first network.")
  tmp_v <- tmp_v[!duplicated(tmp_v$name), ]

  tmp_e1 <- get_e(go1)
  tmp_e2 <- get_e(go2)
  cols <- intersect(colnames(tmp_e1), colnames(tmp_e2))
  tmp_e <- rbind(tmp_e1[cols], tmp_e2[cols])
  if (!is_directed(go1)) {
    # 无方向性
    tmp_e2 <- tmp_e
    tmp_e2$from <- tmp_e$to
    tmp_e2$to <- tmp_e$from
    tmp_e <- rbind(tmp_e, tmp_e2)
  }
  message("Duplicated edges: ", sum(duplicated(tmp_e[, c("from", "to")])), "\nUse the attributes of the first network.")
  tmp_e <- tmp_e[!duplicated(tmp_e[, c("from", "to")]), ]

  go <- func(go1, go2, ...)

  go <- clean_igraph(go)
  go <- c_net_annotate(go, tmp_v, mode = "v")
  go <- c_net_annotate(go, tmp_e, mode = "e")
  go <- c_net_annotate(go, list(n_type = "combine_net"), mode = "n")
  go <- c_net_update(go, initialize = TRUE)
  go
}


#' Annotate a metanet
#'
#' @param go metanet object
#' @param anno_tab a dataframe using to annotate (mode v, e), or a list (mode n)
#' @param mode "v" for vertex, "e" for edge, "n" for network
#' @param verbose logical
#'
#' @return a annotated metanet object
#' @export
#' @family manipulate
#' @examples
#' data("c_net")
#' anno <- data.frame("name" = "s__Pelomonas_puraquae", new_atr = "new")
#' co_net_new <- c_net_annotate(co_net, anno, mode = "v")
#' get_v(co_net_new, c("name", "new_atr"))
#'
#' anno <- data.frame("from" = "s__Pelomonas_puraquae", "to" = "s__un_g__Rhizobium", new_atr = "new")
#' co_net_new <- c_net_annotate(co_net, anno, mode = "e")
#' get_e(co_net_new, c("from", "to", "new_atr"))
#'
#' co_net_new <- c_net_annotate(co_net, list(new_atr = "new"), mode = "n")
#' get_n(co_net_new)
c_net_annotate <- function(go, anno_tab, mode = "v", verbose = TRUE) {
  mode <- match.arg(mode, c("v", "e", "n"))
  if (mode == "v") {
    anno_vertex(go, anno_tab, verbose = verbose) -> go
  } else if (mode == "e") {
    anno_edge(go, anno_tab, verbose = verbose) -> go
  } else if (mode == "n") {
    igraph::graph.attributes(go) <-
      pcutils::update_param(igraph::graph.attributes(go), anno_tab)
  }
  go
}


#' Use data.frame to annotate vertexes of metanet
#'
#' @param go metanet object
#' @param verbose logical
#' @param anno_tab a dataframe using to annotate (with rowname or a "name" column)
#'
#' @return a annotated metanet object
#' @aliases anno_node
#' @export
#' @family manipulate
#' @examples
#' data("c_net")
#' data("otutab", package = "pcutils")
#' anno_vertex(co_net, taxonomy)
anno_vertex <- function(go, anno_tab, verbose = TRUE) {
  if (is.null(anno_tab)) {
    return(go)
  }
  get_v(go) -> v_atr
  if (!"name" %in% colnames(anno_tab)) rownames(anno_tab) -> anno_tab$name
  if (any(duplicated(anno_tab$name))) {
    stop(
      "Duplicated name in annotation tables: ",
      paste0(anno_tab$name[duplicated(anno_tab$name)], collapse = ", ")
    )
  }
  v_atr <- dplyr::left_join(v_atr, anno_tab, by = "name", suffix = c(".x", ""))
  grep(".x", colnames(v_atr), value = TRUE) %>% gsub(".x", "", .) -> du
  if (length(du) > 0) message(length(du), (" attributes will be overwrited:\n"), paste0(du, collapse = ", "), "\n")
  v_atr %>% dplyr::select(!dplyr::ends_with(".x")) -> v_atr

  as.list(v_atr) -> igraph::vertex.attributes(go)
  return(go)
}

#' Use dataframe to annotate edges of an igraph
#'
#' @param go metanet an igraph object
#' @param verbose logical
#' @param anno_tab a dataframe using to annotate (with rowname or a name column)
#'
#' @return a annotated igraph object
#' @export
#' @family manipulate
#' @examples
#' data("c_net")
#' anno <- data.frame("from" = "s__Pelomonas_puraquae", "to" = "s__un_g__Rhizobium", new_atr = "new")
#' anno_edge(co_net, anno) -> anno_net
anno_edge <- function(go, anno_tab, verbose = TRUE) {
  name <- NULL
  if (is.null(anno_tab)) {
    return(go)
  }
  get_e(go) -> e_atr
  if (all(c("from", "to") %in% colnames(anno_tab))) {
    if (anyDuplicated(anno_tab[, c("from", "to")])) {
      message("Duplicated edges in annotation tables: ")
    }
    if ("id" %in% colnames(anno_tab)) anno_tab$id <- NULL
    e_atr <- dplyr::left_join(e_atr, anno_tab, by = c("from", "to"), suffix = c(".x", ""), multiple = "first")
    grep(".x", colnames(e_atr), value = TRUE) %>% gsub(".x", "", .) -> du
    if (length(du) > 0) {
      if (verbose) message(length(du), (" attributes will be overwrited:\n"), paste0(du, collapse = ","), "\n")
    }
    e_atr %>% dplyr::select(!dplyr::ends_with(".x")) -> e_atr
  } else {
    if (verbose) message("No 'from' and 'to' columns in annotation table, will use 'name_from' and 'name_to' instead.")
    if (!"name" %in% colnames(anno_tab)) rownames(anno_tab) -> anno_tab$name
    anno_tab %>% dplyr::select(name, dplyr::everything()) -> anno_tab
    if (any(duplicated(anno_tab$name))) {
      stop(
        "Duplicated name in annotation tables: ",
        paste0(anno_tab$name[duplicated(anno_tab$name)], collapse = ", ")
      )
    }
    # from
    tmp <- anno_tab
    colnames(tmp) <- paste0(colnames(anno_tab), "_from")
    e_atr <- dplyr::left_join(e_atr, tmp, by = c("from" = "name_from"), suffix = c(".x", ""))
    grep(".x", colnames(e_atr), value = TRUE) %>% gsub(".x", "", .) -> du
    if (length(du) > 0) {
      if (verbose) message(length(du), (" attributes will be overwrited:\n"), paste0(du, collapse = ","), "\n")
    }
    e_atr %>% dplyr::select(!dplyr::ends_with(".x")) -> e_atr
    # to
    tmp <- anno_tab
    colnames(tmp) <- paste0(colnames(anno_tab), "_to")
    e_atr <- dplyr::left_join(e_atr, tmp, by = c("to" = "name_to"), suffix = c(".x", ""))
    grep(".x", colnames(e_atr), value = TRUE) %>% gsub(".x", "", .) -> du
    if (length(du) > 0) {
      if (verbose) message(length(du), (" attributes will be overwrited:\n"), paste0(du, collapse = ","), "\n")
    }
    e_atr %>% dplyr::select(!dplyr::ends_with(".x")) -> e_atr
  }
  as.list(e_atr) -> igraph::edge.attributes(go)
  return(go)
}

#' Save network file
#'
#' @param go metanet network
#' @param filename filename
#' @param format "data.frame","graphml"
#' @return No value
#' @family manipulate
#' @export
c_net_save <- function(go, filename = "net", format = "data.frame") {
  if (format == "data.frame") {
    get_v(go) %>% write.csv(., paste0(filename, "_nodes.csv"), row.names = FALSE)
    get_e(go) %>%
      dplyr::select(-1) %>%
      write.csv(., paste0(filename, "_edges.csv"), row.names = FALSE)
  } else if (format == "graphml") {
    if ("id" %in% edge.attributes(go)) go <- igraph::delete_edge_attr(go, "id")
    if (!grepl("\\.graphml$", filename)) filename <- paste0(filename, ".graphml")
    igraph::write_graph(go, filename, format = "graphml")
  } else {
    if (!grepl(paste0("\\.", format), filename)) filename <- paste0(filename, ".", format)
    igraph::write_graph(go, filename, format = format)
  }
  message(paste0(filename, " saved sucessfully!"))
}

#' Load network file
#'
#' @inheritParams c_net_save
#'
#' @return metanet
#' @export
#' @family manipulate
c_net_load <- function(filename, format = "data.frame") {
  if (format == "data.frame") {
    nodes <- read.csv(paste0(filename, "_nodes.csv"), stringsAsFactors = FALSE)
    edges <- read.csv(paste0(filename, "_edges.csv"), stringsAsFactors = FALSE)
    c_net_from_edgelist(edges, vertex_df = nodes) -> go
  } else if (format == "cyjs") {
    lib_ps("jsonify", library = FALSE)
    if (!grepl("\\.cyjs$", filename)) filename <- paste0(filename, ".cyjs")
    jsonify::from_json(filename) -> G

    if (!is.data.frame(G$elements$nodes$data)) {
      names <- lapply(G$elements$nodes$data, names)
      comm_name <- Reduce(intersect, names)
      lapply(G$elements$nodes$data, \(i)i[comm_name]) -> G$elements$nodes$data
      G$elements$nodes$data <- list_to_dataframe(G$elements$nodes$data)
    }

    node <- cbind_new(G$elements$nodes$data, G$elements$nodes$position)
    node$y <- -node$y
    node <- node[, colnames(node) != "name"]
    colnames(node)[1] <- "name"

    edge <- G$elements$edges$data
    edge <- edge[, !colnames(edge) %in% c("from", "to")]
    colnames(edge)[1:3] <- c("id", "from", "to")
    c_net_from_edgelist(edge, node) -> go
  } else if (format == "graphml") {
    if (!grepl("\\.graphml$", filename)) filename <- paste0(filename, ".graphml")
    igraph::read_graph(filename, format = "graphml") -> go
    go <- c_net_update(go, initialize = TRUE)
  } else {
    if (!grepl(paste0("\\.", format), filename)) filename <- paste0(filename, ".", format)
    igraph::read_graph(filename, format = format) -> go
    go <- c_net_update(go, initialize = TRUE)
  }
  go
}

#' Summaries two columns information
#' @param df data.frame
#' @param from first column name or index
#' @param to second column name or index
#' @param count (optional) weight column, if no, each equal to 1
#' @param direct consider direct? default: FALSE
#'
#' @return data.frame
#' @export
#' @examples
#' test <- data.frame(
#'   a = sample(letters[1:4], 10, replace = TRUE),
#'   b = sample(letters[1:4], 10, replace = TRUE)
#' )
#' summ_2col(test, direct = TRUE)
#' summ_2col(test, direct = FALSE)
#' if (requireNamespace("circlize")) {
#'   summ_2col(test, direct = TRUE) %>% pcutils::my_circo()
#' }
summ_2col <- function(df, from = 1, to = 2, count = 3, direct = FALSE) {
  if (ncol(df) < 2) stop("need at least two columns")
  if (ncol(df) == 2) {
    tmp <- cbind(df, count = 1)
  } else {
    tmp <- dplyr::select(df, !!from, !!to, !!count)
  }
  cols <- colnames(tmp)
  colnames(tmp) <- c("from", "to", "count")

  if (direct) {
    tmp <- (dplyr::group_by(tmp, from, to) %>% dplyr::summarise(count = sum(count)))
    colnames(tmp) <- cols
    return(as.data.frame(tmp))
  }

  com <- \(group1, group2, levels){
    factor(c(group1, group2), levels = levels) %>% sort()
  }

  group <- factor(c(tmp[, 1], tmp[, 2]))
  tmp1 <- apply(tmp, 1, function(x) com(x[1], x[2], levels(group))) %>%
    t() %>%
    as.data.frame()

  tmp1 <- cbind(tmp1, tmp$count)
  colnames(tmp1) <- c("from", "to", "count")
  tmp1 <- dplyr::group_by(tmp1, from, to) %>% dplyr::summarise(count = sum(count))
  colnames(tmp1) <- cols
  return(as.data.frame(tmp1))
}


#' Plot e_type bar
#'
#' @param go metanet object
#' @param mode "left" or "right"
#' @param degree_threshold degree threshold
#'
#' @returns ggplot
#' @export
#'
#' @examples
#' data("c_net")
#' plot_e_type_bar(co_net, degree_threshold = 10)
plot_e_type_bar <- function(go, mode = c("left", "right")[1], degree_threshold = 0) {
  from <- e_type <- n <- label <- NULL
  get_e(go) -> tmp_e2
  count(tmp_e2, from, e_type) -> plotdat
  count(tmp_e2, from) -> plotdat2
  filter(plotdat2, n > degree_threshold) -> plotdat2
  plotdat <- filter(plotdat, from %in% plotdat2$from)
  plotdat2$label <- plotdat2$from

  plotdat$from <- factor(plotdat$from, levels = dplyr::arrange(plotdat2, n) %>% dplyr::pull(from))
  if (mode == "left") {
    p <- ggplot(plotdat, aes(x = -n, y = from)) +
      geom_col(aes(fill = e_type), width = 0.9, color = "black", linewidth = 0.2) +
      geom_text(aes(x = -0.3, y = from, label = label), data = plotdat2, hjust = 1, size = 3) +
      scale_x_continuous(labels = \(x) -x, expand = c(0.1, 0.1, 0, 0))
  } else {
    p <- ggplot(plotdat, aes(x = n, y = from)) +
      geom_col(aes(fill = e_type), width = 0.9, color = "black", linewidth = 0.2) +
      geom_text(aes(x = 0.3, y = from, label = label), data = plotdat2, hjust = 0, size = 3) +
      scale_x_continuous(labels = \(x) x, expand = c(0, 0, 0.1, 0.1))
  }
  p +
    theme_test() +
    scale_fill_manual(values = setNames(unique(tmp_e2$color), unique(tmp_e2$e_type))) +
    theme(
      axis.ticks.y = element_blank(),
      axis.text.y = element_blank(),
      axis.title.y = element_blank()
    ) +
    xlab("Number of e_type")
}

#' Get skeleton network according to a group
#'
#' @param go network
#' @param Group vertex column name
#' @param count take which column count, default: NULL
#' @param top_N top_N
#'
#' @return skeleton network
#' @export
#' @family topological
#' @aliases c_net_skeleton
#'
#' @examples
#' get_group_skeleton(co_net) -> ske_net
#' skeleton_plot(ske_net)
get_group_skeleton <- function(go, Group = "v_class", count = NULL, top_N = 8) {
  name <- v_group <- n <- NULL
  stopifnot(is_igraph(go))
  direct <- igraph::is_directed(go)

  if (!Group %in% vertex_attr_names(go)) stop("no Group named ", Group, " !")
  get_v(go) -> tmp_v
  tmp_v %>% dplyr::select(name, !!Group) -> nodeGroup
  colnames(nodeGroup) <- c("name", "Group")
  nodeGroup$Group <- as.factor(nodeGroup$Group)
  # summary edges counts in each e_type
  suppressMessages(anno_edge(go, nodeGroup) %>% get_e() -> edge)
  {
    if (is.null(count)) {
      edge$count <- 1
    } else {
      edge$count <- edge[, count]
    }
  }
  bb <- data.frame()
  for (i in unique(edge$e_type)) {
    tmp <- edge[edge$e_type == i, c("Group_from", "Group_to", "count")]
    tmp <- dplyr::mutate_if(tmp, is.factor, as.character)
    # tmp=pcutils:::gettop(tmp,top_N)
    bb <- rbind(bb, data.frame(summ_2col(tmp,
      direct = direct
    ), e_type = i))
  }
  tmp_go <- igraph::graph_from_data_frame(bb, directed = direct)
  nodeGroup <- cbind_new(nodeGroup, data.frame(v_group = tmp_v$v_group))

  # nodeGroup=mutate_all(nodeGroup,as.character)
  # nodeGroup=rbind(nodeGroup,c("others","others","others"))

  dplyr::distinct(nodeGroup, Group, v_group) %>% tibble::column_to_rownames("Group") -> v_group_tab

  V(tmp_go)$v_group <- v_group_tab[V(tmp_go)$name, "v_group"]
  V(tmp_go)$v_class <- V(tmp_go)$name
  V(tmp_go)$size <- stats::aggregate(tmp_v$size, by = list(tmp_v[, Group]), sum) %>%
    tibble::column_to_rownames("Group.1") %>%
    .[V(tmp_go)$name, "x"]
  V(tmp_go)$count <- tmp_v %>%
    dplyr::group_by(!!rlang::sym(Group)) %>%
    dplyr::count() %>%
    tibble::column_to_rownames(Group) %>%
    .[V(tmp_go)$name, "n"]

  tmp_go <- c_net_update(tmp_go, initialize = TRUE)
  get_e(tmp_go) -> tmp_e

  E(tmp_go)$width <- E(tmp_go)$label <- tmp_e$count

  graph.attributes(tmp_go)$n_type <- "skeleton"
  graph.attributes(tmp_go)$skeleton <- Group
  tmp_go
}

#' Skeleton plot
#'
#' @param ske_net skeleton
#' @param split_e_type split by e_type? default: TRUE
#' @param ... additional parameters for \code{\link[igraph]{igraph.plotting}}
#'
#' @export
#' @rdname get_group_skeleton
skeleton_plot <- function(ske_net, split_e_type = TRUE, ...) {
  e_type <- NULL
  params <- list(...)
  tmp_go <- ske_net
  if (get_n(tmp_go)$n_type != "skeleton") stop("Not a skeleton network")
  get_e(tmp_go) -> tmp_e

  if (split_e_type) {
    for (i in unique(tmp_e$e_type)) {
      # main plot
      tmp_go1 <- c_net_filter(tmp_go, e_type == i, mode = "e")
      do.call(c_net_plot, pcutils::update_param(
        list(go = tmp_go1, legend_number = TRUE, edge_width_range = c(1, 5)), params
      ))
    }
  } else {
    tmp_go <- clean_multi_edge_metanet(tmp_go)
    do.call(c_net_plot, pcutils::update_param(
      list(go = tmp_go, legend_number = TRUE, edge_width_range = c(1, 5)), params
    ))
  }
}

# 整理skeleton网络的边，使其尽量不重叠。
# 1.from-to都是自己时，添加edge.loop.angle
# 2.from-to一致时，添加edge.curved
# 3.from-to刚好相反时，添加edge.curved


#' Clean multi edge metanet to plot
#' @param go metanet object
#'
#' @return metanet object
#' @export
#'
#' @examples
#' g <- igraph::make_ring(2)
#' g <- igraph::add_edges(g, c(1, 1, 1, 1, 2, 1))
#' plot(g)
#' plot(clean_multi_edge_metanet(g))
clean_multi_edge_metanet <- function(go) {
  tmp_e <- get_e(go)
  tmp_e$loop.angle <- 0
  # tmp_e$curved=0

  summ_2col(tmp_e[, c("from", "to")], direct = FALSE) -> e_count
  filter(e_count, count > 1) -> multi_e_count
  for (i in seq_len(nrow(multi_e_count))) {
    from <- multi_e_count$from[i]
    to <- multi_e_count$to[i]
    count <- multi_e_count$count[i]
    if (from == to) {
      tmp_e[tmp_e$from == from & tmp_e$to == to, "loop.angle"] <- seq(0, 2 * pi, length = count + 1)[-(count + 1)]
    }
    # else {
    #   tmp_e[tmp_e$from%in%c(from,to) & tmp_e$to%in%c(from,to),"curved"] <- 0.2 # seq(0,1,length=count)
    # }
  }

  #   summ_2col(tmp_e[,c("from","to")],direct = TRUE) -> e_count
  #   filter(e_count,count>1) -> multi_e_count
  #   for (i in seq_len(nrow(multi_e_count))) {
  #     from=multi_e_count$from[i]
  #     to=multi_e_count$to[i]
  #     count=multi_e_count$count[i]
  #     if(from!=to){
  #       tmp_e[tmp_e$from==from & tmp_e$to==to,"curved"] <- seq(0.2,1,length=count)
  #     }
  #   }

  igraph::edge.attributes(go) <- as.list(tmp_e)
  go
}

#' Link summary of the network
#'
#' @param go igraph or metanet
#' @param group summary which group of vertex attribution in names(vertex_attr(go))
#' @param e_type "positive", "negative", "all"
#' @param topN topN of group, default: 10
#' @param mode 1~2
#' @param colors colors
#' @param plot_param plot parameters
#'
#' @return plot
#' @export
#' @family topological
#' @examples
#' if (requireNamespace("circlize")) {
#'   links_stat(co_net, topN = 10)
#'   module_detect(co_net) -> co_net_modu
#'   links_stat(co_net_modu, group = "module")
#' }
#' if (requireNamespace("corrplot")) {
#'   links_stat(co_net, topN = 10, mode = 2)
#' }
links_stat <- function(go, group = "v_class", e_type = "all",
                       topN = 10, colors = NULL, mode = 1, plot_param = list()) {
  color <- v_class <- shape <- left_leg_x <- from <- to <- n <- NULL
  direct <- is_directed(go)
  go <- c_net_set(go, vertex_class = group)

  get_v(go) -> v_index
  v_index %>% dplyr::select("name", "v_class") -> map

  suppressMessages(anno_edge(go, map) %>% get_e() -> edge)
  # statistics
  if (e_type != "all") edge %>% dplyr::filter(e_type == !!e_type) -> edge
  summ_2col(edge[, paste0("v_class", c("_from", "_to"))], direct = direct) -> bb
  colnames(bb) <- c("from", "to", "count")

  dplyr::group_by(bb, from) %>%
    dplyr::summarise(n = sum(count)) %>%
    dplyr::arrange(-n) %>%
    dplyr::top_n(topN, n) %>%
    dplyr::pull(from) -> nnn

  # plot
  bb2 <- mutate(bb,
    from = ifelse(from %in% nnn, from, "Others"),
    to = ifelse(to %in% nnn, to, "Others")
  ) %>% summ_2col(direct = direct)

  if (mode == 1) {
    do.call(pcutils::my_circo, pcutils::update_param(
      list(
        df = bb2,
        reorder = FALSE,
        pal = colors
      ), plot_param
    ))
  }
  if (mode == 2) {
    tab <- pcutils::df2distance(bb2)
    tab2 <- tab
    tab2[tab2 > 0] <- 1
    tab2[tab2 != 1] <- 0
    # tab2 <- trans(tab, "pa") %>% as.matrix()
    do.call(corrplot::corrplot, pcutils::update_param(
      list(
        corr = tab2,
        type = "lower",
        method = "color",
        col = c("white", "white", "red"),
        addgrid.col = "black",
        cl.pos = "n",
        tl.col = "black"
      ),
      plot_param
    ))
  }
}

# 每个分组可以构建一个网络，每个网络都可以用link_stat得到一些互作的数量（互作强度），可以再看这些数量和分组间某些指标的相关性。

# ========7.stability===========

#' Evaluate the stability of a network
#'
#' @param go_ls an igraph object or igraph list.
#' @param mode "robust_test", "vulnerability", "robustness"
#' @param partial how much percent vertexes be removed in total (default: 0.5, only for robust_test)
#' @param step how many nodes be removed each time? (default: 10, only for robust_test)
#' @param reps simulation number (default: 9)
#' @param threads threads
#' @param verbose verbose
#' @param keystone remove 70%% keystones instead of remove 50%% nodes (default: False, only for robustness)
#'
#' @return a data.frame
#' @export
#'
#' @examples
#' \donttest{
#' data("c_net")
#' if (requireNamespace("ggpmisc")) {
#'   c_net_stability(co_net, mode = "robust_test", step = 20, reps = 9) -> robust_res
#'   plot(robust_res, index = "Average_degree", mode = 2)
#' }
#'
#' c_net_stability(co_net, mode = "vulnerability") -> vulnerability_res
#' plot(vulnerability_res)
#'
#' robustness(co_net) -> robustness_res
#' plot(robustness_res)
#'
#' module_detect(co_net) -> co_net_modu
#' zp_analyse(co_net_modu, mode = 2) -> co_net_modu
#'
#' c_net_stability(co_net_modu, mode = "robustness", keystone = TRUE) -> robustness_res
#' plot(robustness_res)
#' }
c_net_stability <- function(go_ls, mode = "robust_test", partial = 0.5, step = 10, reps = 9,
                            threads = 1, verbose = TRUE, keystone = FALSE) {
  mode <- match.arg(mode, c("robust_test", "vulnerability", "robustness"))
  if (mode == "robust_test") {
    res <- robust_test(go_ls, partial = partial, step = step, reps = reps, threads = threads, verbose = verbose)
  } else if (mode == "vulnerability") {
    res <- vulnerability(go_ls, threads = threads, verbose = verbose)
  } else if (mode == "robustness") {
    res <- robustness(go_ls, keystone = keystone, reps = reps, threads = threads, verbose = verbose)
  }
  return(res)
}

#' Robust_test for a network
#' @rdname c_net_stability
#'
#' @return data.frame (robustness class)
#' @export
#'
robust_test <- function(go_ls, partial = 0.5, step = 10, reps = 9, threads = 1, verbose = TRUE) {
  if ("igraph" %in% class(go_ls)) {
    robustness_res <- robust_test_in(go_ls, partial = partial, step = step, reps = reps, threads = threads, verbose = verbose)
    robustness_res <- data.frame(robustness_res, group = "Network")
  } else {
    if (!is.igraph(go_ls[[1]])) stop("No igraph or igraph-list.")
    robustness_res <- lapply(names(go_ls), \(i){
      tmp <- robust_test_in(go_ls[[i]], partial = partial, step = step, reps = reps, threads = threads, verbose = verbose)
      data.frame(tmp, group = i)
    })
    robustness_res <- do.call(rbind, robustness_res)
    robustness_res$group <- factor(robustness_res$group, levels = names(go_ls))
  }
  class(robustness_res) <- c("robust", "data.frame")
  return(robustness_res)
}

robust_test_in <- function(go, partial = 0.5, step = 10, reps = 9, threads = 1, verbose = TRUE) {
  i <- NULL

  cal_del <- \(go, partial, step, rep){
    nodes <- length(igraph::V(go))
    floor(nodes * partial) -> del_i
    del_i_indexs <- data.frame()
    sequ <- seq(0, del_i, step)
    if (sequ[length(sequ)] < del_i) sequ <- c(sequ, del_i)
    res <- lapply(sequ, \(i){
      # remove i nodes in the network
      remove_node <- sample(1:nodes, i)
      dp <- igraph::delete.vertices(go, remove_node)
      dp <- igraph::delete.vertices(dp, igraph::V(dp)[igraph::degree(dp) == 0])

      # calculate network parameters
      tmp_ind <- (MetaNet::net_par(dp, mode = "n")$n_index)
      data.frame(tmp_ind, i = i)
    })
    del_i_indexs <- do.call(rbind, res)

    # for (i in sequ) {
    #   # remove i nodes in the network
    #   remove_node <- sample(1:nodes, i)
    #   dp <- igraph::delete.vertices(go, remove_node)
    #   dp=igraph::delete.vertices(dp,igraph::V(dp)[igraph::degree(dp)==0])
    #
    #   # calculate network parameters
    #   tmp_ind=(MetaNet::net_par(dp,mode = "n")$n_index)
    #   del_i_indexs <- rbind(
    #     del_i_indexs,
    #     data.frame(tmp_ind, i = i)
    #   )
    # }

    return(data.frame(del_i_indexs, "rep" = rep))
  }

  # parallel
  # main function
  loop <- function(i) {
    cal_del(go, partial, step, i)
  }
  {
    if (threads > 1) {
      pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
      pb <- utils::txtProgressBar(max = reps, style = 3)
      if (verbose) {
        opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
        cl <- snow::makeCluster(threads)
      } else {
        opts <- NULL
      }
      doSNOW::registerDoSNOW(cl)
      res <- foreach::`%dopar%`(
        foreach::foreach(i = 1:reps, .options.snow = opts),
        loop(i)
      )
      snow::stopCluster(cl)
      gc()
    } else {
      res <- lapply(1:reps, loop)
    }
  }
  # simplify method
  del_i_indexs <- do.call(rbind, res)

  del_i_indexs$i_ratio <- del_i_indexs$i / length(V(go)) %>% round(., 3)
  class(del_i_indexs) <- c("robust", class(del_i_indexs))
  return(del_i_indexs)
}

#' Plot robust
#'
#' @param x `robust_test()` result (robust object)
#' @param mode plot mode, 1~3
#' @param indexes indexes selected to show
#' @param use_ratio use the delete nodes ratio rather than nodes number
#' @param ... additional arguments for \code{\link[pcutils]{group_box}}
#'
#' @return a ggplot
#' @method plot robust
#' @exportS3Method
#' @rdname robust_test
plot.robust <- function(x, indexes = c("Natural_connectivity", "Average_degree"), use_ratio = FALSE, mode = 1, ...) {
  i <- group <- variable <- value <- se <- eq.label <- adj.rr.label <- Natural_connectivity <- lm <- NULL
  robust_res <- x
  xlab <- "Removed_nodes"
  if (use_ratio) {
    robust_res$i <- robust_res$i_ratio
    xlab <- "Removed_nodes_ratio"
  }
  robust_res %>%
    dplyr::select(i, group, !!indexes) %>%
    reshape2::melt(id.var = c("i", "group")) -> pdat

  pdat %>%
    dplyr::group_by(i, variable, group) %>%
    dplyr::summarise(mean = mean(value), sd = sd(value), se = sd / sqrt(length(value))) -> sdd

  if (mode == 1) {
    p <- ggplot(sdd, aes(x = i, y = mean, col = group)) +
      geom_line() +
      geom_errorbar(data = sdd, aes(ymax = mean + se, ymin = mean - se)) +
      # geom_smooth(se = FALSE,method = "loess",formula = 'y ~ x') +
      facet_wrap(. ~ variable, scales = "free") +
      labs(x = xlab, y = NULL) +
      theme_bw()
  }
  if (mode == 2) {
    lib_ps("ggpmisc", library = FALSE)
    p <- ggplot(sdd, aes(x = i, y = mean, col = group)) +
      geom_point(size = 0.2, alpha = 0.4) +
      geom_smooth(se = FALSE, method = "loess", formula = "y ~ x") +
      ggpmisc::stat_poly_eq(
        aes(label = paste(after_stat(eq.label), after_stat(adj.rr.label), sep = "~~~~~")),
        formula = y ~ x, parse = TRUE, label.x = "right",
        size = 3,
      ) +
      facet_wrap(. ~ variable, scales = "free") +
      labs(x = xlab, y = NULL) +
      theme_bw()
  }
  if (mode == 3) {
    robust_res %>% dplyr::select(i, rep, group, `Natural_connectivity`) -> pdat

    pdat %>%
      # filter(i<0.4)%>%
      dplyr::group_by(rep, group) %>%
      dplyr::summarise(slope = coefficients(lm(`Natural_connectivity` ~ i))[2]) -> slope

    p <- pcutils::group_box(slope["slope"], group = "group", metadata = slope, alpha = TRUE, ...) + theme_bw()
  }
  return(p)
}

#' Vulnerability calculation
#'
#' @rdname c_net_stability
#'
#' @return a vector
#' @export
#' @description
#' \deqn{Vi=\frac{E-Ei}{E}}
#' E is the global efficiency and Ei is the global efficiency after the removal of the node i and its entire links.
#'
vulnerability <- function(go_ls, threads = 1, verbose = TRUE) {
  if ("igraph" %in% class(go_ls)) {
    vulnerability_res <- vul_max(go_ls, threads = threads, verbose = verbose)
  } else {
    if (!"igraph" %in% class(go_ls[[1]])) stop("No igraph or igraph-list.")
    vulnerability_res <- lapply(names(go_ls), \(i){
      vul_max(go_ls[[i]], threads = threads, verbose = verbose)
    })
    names(vulnerability_res) <- names(go_ls)
  }
  class(vulnerability_res) <- c("vulnerability", class(vulnerability_res))
  return(vulnerability_res)
}

vul_max <- function(go, threads = 1, verbose = TRUE) {
  i <- NULL
  stopifnot(is.igraph(go))
  if (is.null(V(go)$name)) V(go)$name <- V(go)

  # parallel
  # main function
  loop <- function(i) igraph::global_efficiency(igraph::delete_vertices(go, i))
  {
    if (threads > 1) {
      pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
      if (verbose) {
        pb <- utils::txtProgressBar(max = length(V(go)$name), style = 3)
        opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
      } else {
        opts <- NULL
      }
      cl <- snow::makeCluster(threads)
      doSNOW::registerDoSNOW(cl)
      res <- foreach::`%dopar%`(
        foreach::foreach(i = V(go)$name, .options.snow = opts),
        loop(i)
      )
      snow::stopCluster(cl)
      gc()
    } else {
      res <- lapply(V(go)$name, loop)
    }
  }
  # simplify method
  tmpv <- do.call(c, res)

  (igraph::global_efficiency(go) - tmpv) / igraph::global_efficiency(go)
}

#' Plot vulnerability
#'
#' @param x `vulnerability()` result (vulnerability object)
#' @param ... add
#'
#' @return a ggplot
#' @method plot vulnerability
#' @exportS3Method
#' @rdname vulnerability
plot.vulnerability <- function(x, ...) {
  group <- NULL
  vulnerability_res <- x
  if (!is.list(vulnerability_res)) {
    vulnerability_res <- list(Network = vulnerability_res)
  }
  pdat <- data.frame(
    group = factor(names(vulnerability_res), levels = names(vulnerability_res)),
    vulnerability = vapply(vulnerability_res, max, numeric(1))
  )
  ggplot(pdat, aes(group, vulnerability)) +
    geom_col(aes(fill = group), ...) +
    geom_text(aes(group, vulnerability * 1.05, label = round(vulnerability, 3))) +
    theme_bw()
}


#' Robustness after remove 50%% nodes or some hubs, need the metanet contains "cor" attribute.
#'
#' @rdname c_net_stability
#'
#' @export
robustness <- function(go_ls, keystone = FALSE, reps = 9, threads = 1, verbose = TRUE) {
  if ("igraph" %in% class(go_ls)) {
    robustness_res <- robustness_in(go_ls, keystone = keystone, reps = reps, threads = threads, verbose = verbose)
  } else {
    if (!"igraph" %in% class(go_ls[[1]])) stop("No igraph or igraph-list.")
    robustness_res <- lapply(names(go_ls), \(i){
      tmp <- robustness_in(go_ls[[i]], keystone = keystone, reps = reps, threads = threads, verbose = verbose)
      data.frame(tmp, group = i)
    })
    robustness_res <- do.call(rbind, robustness_res)
    robustness_res$group <- factor(robustness_res$group, levels = names(go_ls))
  }
  class(robustness_res) <- c("robustness", class(robustness_res))
  return(robustness_res)
}

robustness_in <- function(go, keystone = FALSE, reps = 9, threads = 1, verbose = TRUE) {
  roles <- name <- from <- to <- i <- NULL

  nodes <- length(V(go))
  floor(nodes * 0.5) -> del_i
  if (keystone) {
    get_v(go) -> tmp_v
    if (!"module" %in% colnames(tmp_v)) stop("no modules, please `module_detect()` first")
    if (!"roles" %in% colnames(tmp_v)) stop("no roles, please `zp_analyse()` first")
    tmp_v %>%
      dplyr::filter(roles == "Module hubs") %>%
      dplyr::pull(name) -> hubs
    floor(length(hubs) * 0.7) -> del_i
  }

  # parallel
  # main function
  loop <- \(i){
    if (!keystone) remove_node <- sample(1:nodes, del_i)
    if (keystone) remove_node <- sample(hubs, del_i)

    dp <- igraph::delete.vertices(go, remove_node)
    dead_s <- "init"
    # calculated the abundance-weighted mean interaction strength (wMIS) of nodes,<=0 will dead
    # repeat until all >0
    while (length(dead_s) > 0) {
      get_e(dp) -> edge_list
      edge_list %>%
        dplyr::select(from, cor) %>%
        rbind(., dplyr::select(edge_list, to, cor) %>% dplyr::rename(from = to)) %>%
        dplyr::group_by(from) %>%
        dplyr::summarise(w_degree = sum(cor)) -> w_degree
      w_degree %>%
        dplyr::filter(w_degree <= 0) %>%
        dplyr::pull(from) -> dead_s
      dp <- igraph::delete.vertices(dp, dead_s)
    }
    tmp_ind <- (MetaNet::net_par(dp, mode = "n")$n_index)
    data.frame(tmp_ind, reps = i)
  }
  {
    if (threads > 1) {
      pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
      if (verbose) {
        pb <- utils::txtProgressBar(max = reps, style = 3)
        opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
      } else {
        opts <- NULL
      }
      cl <- snow::makeCluster(threads)
      doSNOW::registerDoSNOW(cl)
      res <- foreach::`%dopar%`(
        foreach::foreach(i = 1:reps, .options.snow = opts),
        loop(i)
      )
      snow::stopCluster(cl)
      gc()
    } else {
      res <- lapply(1:reps, loop)
    }
  }
  # simplify method
  del_i_indexs <- do.call(rbind, res)
  return(del_i_indexs)
}

#' Plot robustness
#'
#' @param x `robustness()` result (robustness object)
#' @param indexes indexes selected to show
#' @param ... additional arguments for \code{\link[pcutils]{group_box}}
#'
#' @return a ggplot
#' @method plot robustness
#' @exportS3Method
#' @rdname robustness
plot.robustness <- function(x, indexes = "Node_number", ...) {
  robustness_res <- x
  if (!"group" %in% colnames(robustness_res)) robustness_res$group <- "Network"
  pcutils::group_box(robustness_res[, indexes, drop = FALSE],
    group = "group",
    metadata = robustness_res, ...
  ) + theme_bw()
}


#' Cohesion calculation
#'
#' @param otutab otutab
#' @param reps iteration time
#' @param threads threads
#' @param mycor a correlation matrix you want to use, skip the null model build when mycor is not NULL, default: NULL
#' @param verbose verbose
#'
#' @return Cohesion object: a list with two dataframe
#' @export
#' @references
#' Herren, C. M. & McMahon, K. (2017) Cohesion: a method for quantifying the connectivity of microbial communities. doi:10.1038/ismej.2017.91.
#' @examples
#' \donttest{
#' data("otutab", package = "pcutils")
#' # set reps at least 99 when you run.
#' Cohesion(otutab[1:50, ], reps = 19) -> cohesion_res
#' if (requireNamespace("ggpubr")) {
#'   plot(cohesion_res, group = "Group", metadata = metadata, mode = 1)
#'   plot(cohesion_res, group = "Group", metadata = metadata, mode = 2)
#' }
#' }
Cohesion <- function(otutab, reps = 200, threads = 1, mycor = NULL, verbose = TRUE) {
  i <- NULL
  d <- t(otutab)
  rel.d <- d / rowSums(d)

  if (is.null(mycor)) {
    # Create observed correlation matrix
    cor.mat.true <- cor(rel.d)
    nc <- ncol(rel.d)

    loop <- \(which.taxon){
      # create vector to hold correlations from every permutation for each single otu
      ## perm.cor.vec.mat stands for permuted correlations vector matrix
      perm.cor.vec.mat <- vector()

      for (i in 1:reps) {
        # Create empty matrix of same dimension as rel.d
        perm.rel.d <- matrix(numeric(0), dim(rel.d)[1], dim(rel.d)[2])
        rownames(perm.rel.d) <- rownames(rel.d)
        colnames(perm.rel.d) <- colnames(rel.d)

        # For each otu
        for (j in seq_len(dim(rel.d)[2])) {
          # Replace the original taxon vector with a permuted taxon vector
          perm.rel.d[, j] <- sample(rel.d[, j])
        }

        # Do not randomize focal column
        perm.rel.d[, which.taxon] <- rel.d[, which.taxon]

        # Calculate correlation matrix of permuted matrix
        cor.mat.null <- cor(perm.rel.d, perm.rel.d[, which.taxon])

        # For each iteration, save the vector of null matrix correlations between focal taxon and other taxa
        perm.cor.vec.mat <- cbind(perm.cor.vec.mat, cor.mat.null)
      }

      # Save the median correlations between the focal taxon and all other taxa
      apply(perm.cor.vec.mat, 1, median)
    }
    {
      if (threads > 1) {
        pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
        pb <- utils::txtProgressBar(max = reps, style = 3)
        if (verbose) {
          opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
          cl <- snow::makeCluster(threads)
        } else {
          opts <- NULL
        }
        doSNOW::registerDoSNOW(cl)
        res <- foreach::`%dopar%`(
          foreach::foreach(i = 1:nc, .options.snow = opts),
          loop(i)
        )
        snow::stopCluster(cl)
        gc()
      } else {
        res <- lapply(1:nc, loop)
      }
    }

    simplify2array(res) -> med.tax.cors

    obs.exp.cors.mat <- cor.mat.true - med.tax.cors
  } else {
    obs.exp.cors.mat <- mycor
  }

  diag(obs.exp.cors.mat) <- 0

  # Calculate connectedness by averaging positive and negative observed - expected correlations
  pn.mean <- \(vector, mode = "p"){
    if (mode == "p") {
      pos.vals <- vector[which(vector > 0)]
    } else if (mode == "n") pos.vals <- vector[which(vector < 0)]
    p.mean <- mean(pos.vals)
    if (length(pos.vals) == 0) p.mean <- 0
    return(p.mean)
  }
  connectedness.pos <- apply(obs.exp.cors.mat, 2, pn.mean, mode = "p")
  connectedness.neg <- apply(obs.exp.cors.mat, 2, pn.mean, mode = "n")

  # Calculate cohesion by multiplying the relative abundance dataset by associated connectedness
  cohesion.pos <- rel.d %*% connectedness.pos
  cohesion.neg <- rel.d %*% connectedness.neg

  #### Combine vectors into one list and print
  output <- list(
    data.frame(neg = connectedness.neg, pos = connectedness.pos),
    data.frame(neg = cohesion.neg, pos = cohesion.pos)
  )

  names(output) <- c("Connectedness", "Cohesion")

  class(output) <- c("cohesion", class(output))
  return(output)
}


#' Plot cohesion
#'
#' @param x `Cohesion()` result (cohesion object)
#' @param mode plot mode, 1~2
#' @param group group name in colnames(metadata)
#' @param metadata metadata
#' @param ... additional arguments for \code{\link[pcutils]{group_box}} (mode=1) or \code{\link[pcutils]{group_box}} (mode=2)
#'
#' @return a ggplot
#' @method plot cohesion
#' @exportS3Method
#' @rdname Cohesion
plot.cohesion <- function(x, group, metadata, mode = 1, ...) {
  neg <- pos <- NULL
  cohesion_res <- x
  if (mode == 1) p <- pcutils::stackplot(abs(t(cohesion_res$Cohesion)), group = group, metadata = metadata, ...)
  if (mode == 2) {
    co <- cohesion_res$Cohesion %>% dplyr::transmute(`neg:pos` = neg / pos)
    p <- pcutils::group_box(co, group = group, metadata = metadata, ...) +
      ylab("neg:pos cohesion") + theme_bw()
  }
  p
}

# ========3.layout========
#' Layout coordinates
#'
#' @param go igraph or metanet
#' @param method
#'  (1) as_line(), as_arc(), as_polygon(), as_polyarc(), as_polycircle(), as_circle_tree();
#'  (2) as_star(), as_tree(), in_circle(), nicely(), on_grid(), on_sphere(),randomly(), with_dh(), with_fr(), with_gem(), with_graphopt(), with_kk(),with_lgl(), with_mds(),. see \code{\link[igraph]{layout_}};
#'  (3) a character, "auto","backbone","centrality","circlepack","dendrogram",
#'  "eigen","focus","hive","igraph","linear","manual","matrix",
#'  "partition","pmds","stress","treemap","unrooted". see \code{\link[ggraph]{create_layout}}
#' @param order_by order nodes according to a node attribute
#' @param order_ls manual the discrete variable with a vector, or continuous variable with "desc" to decreasing
#' @param seed random seed
#' @param line_curved consider line curved, only for some layout methods like as_line(), as_polygon().default:0
#' @param rescale logical, scale the X, Y to (-1,1)
#' @param ... add
#'
#' @aliases c_net_lay
#' @family layout
#' @return coors object: coordinates for nodes, columns: name, X, Y; curved for edges, columns: from, to, curved;
#' @export
#' @examples
#' library(igraph)
#' c_net_layout(co_net) -> coors
#' c_net_plot(co_net, coors)
#' c_net_plot(co_net, c_net_layout(co_net, in_circle()), vertex.size = 2)
#' c_net_plot(co_net, c_net_layout(co_net, in_circle(), order_by = "v_class"), vertex.size = 2)
#' c_net_plot(co_net, c_net_layout(co_net, in_circle(), order_by = "size", order_ls = "desc"))
#' c_net_plot(co_net, c_net_layout(co_net, as_polygon(3)))
c_net_layout <- function(go, method = igraph::nicely(), order_by = NULL, order_ls = NULL,
                         seed = 1234, line_curved = 0.5, rescale = TRUE, ...) {
  set.seed(seed)
  name <- x <- y <- NULL
  if ("igraph_layout_spec" %in% class(method)) {
    coors <- igraph::layout_(go, method)
    if (all(is.na(coors))) {
      go2 <- clean_igraph(go)
      coors <- igraph::layout_(go2, method)
    }
  } else if ("poly" %in% class(method)) {
    coors <- method(go, group2 = order_by, group2_order = order_ls)
  } else if ("layout" %in% class(method)) {
    coors <- method(go)
  } else if (is.character(method)) {
    message("Use method from `ggraph`: ", method)
    lib_ps("ggraph", library = FALSE)
    data <- ggraph::create_layout(clean_igraph(go), layout = method, ...)
    coors <- data %>% dplyr::select(name, x, y)
    colnames(coors) <- c("name", "X", "Y")
  } else {
    stop("No valid method")
  }

  if (inherits(coors, "coors")) {
    if (rescale) {
      coors <- rescale_coors(coors)
    }
    return(coors)
  }

  # order
  if (is.matrix(coors)) {
    get_v(go) -> tmp_v
    coors <- order_tmp_v_name(tmp_v, coors, order_by = order_by, order_ls = order_ls)
  }

  curved <- NULL
  # if line type, need to consider edge.curved
  if ("line" %in% class(method)) {
    tmp_e <- data.frame(igraph::as_data_frame(go))[, c("from", "to")]
    if (nrow(tmp_e) > 0) {
      curved <- data.frame(tmp_e, curved = line_curved, row.names = NULL)
    }
  }
  coors <- data.frame(coors, row.names = NULL)
  coors <- as_coors(coors, curved)
  if (rescale) {
    coors <- rescale_coors(coors)
  }
  return(coors)
}

order_circle_func <- function(coors_mat) {
  dis <- X <- Y <- NULL
  mid_x <- mean(coors_mat[, 1])
  mid_y <- mean(coors_mat[, 2])
  coors_mat <- data.frame(X = coors_mat[, 1], Y = coors_mat[, 2]) %>%
    dplyr::mutate(dis = sqrt((X - mid_x)^2 + (Y - mid_y)^2)) %>%
    dplyr::arrange(dis) %>%
    dplyr::select_at(1:2) %>%
    as.matrix()
  coors_mat
}

order_tmp_v_name <- function(tmp_v, coors_mat, order_by = NULL, order_ls = NULL) {
  if (is.null(order_by)) {
    coors <- data.frame(name = tmp_v$name, X = coors_mat[, 1], Y = coors_mat[, 2], row.names = NULL)
  } else {
    ordervec <- tmp_v[, order_by]
    if (is.numeric(ordervec)) {
      name <- tmp_v[order(ordervec, decreasing = is.null(order_ls)), "name"]
    } else {
      ordervec <- pcutils::change_fac_lev(ordervec, order_ls)
      name <- tmp_v[order(ordervec), "name"]
    }
    coors <- data.frame(name = name, X = coors_mat[, 1], Y = coors_mat[, 2], row.names = NULL)
  }
  return(coors)
}

is_layout <- \(x){
  any(class(x) %in% c("igraph_layout_spec", "layout"))
}


#' Transfer to a coors object
#'
#' @param coors data.frame
#' @param curved line curved
#'
#' @return coors object
#' @export
#'
as_coors <- function(coors, curved = NULL) {
  if (inherits(coors, "coors")) {
    if (!is.null(curved)) attributes(coors)$curved <- curved
    return(coors)
  }
  if (!identical(colnames(coors), c("name", "X", "Y"))) {
    stop("The input should be a data frame with columns: name, X, Y")
  }
  attributes(coors)$class <- c("coors", class(coors))
  attributes(coors)$curved <- curved
  return(coors)
}


get_coors <- \(coors, go, ...){
  edge_curved <- NULL
  # 1.如果coors是NULL，去graph_attr找一下，没有的话就默认nicely计算
  if (is.null(coors)) {
    if (is.null(igraph::graph_attr(go, "coors"))) {
      coors <- c_net_layout(go, igraph::nicely(), ...)
    } else {
      coors <- igraph::graph_attr(go, "coors")
    }
  }
  # 2.如果是layout函数，那就计算
  if (is_layout(coors)) coors <- c_net_layout(go, coors, ...)
  # 3.如果是一个提供的coors对象（list），那就把curved传到edge里，coors导出为df
  if (inherits(coors, "coors")) {
    if (!is.null(attributes(coors)$curved)) {
      edge_curved <- dplyr::left_join(get_e(go), attributes(coors)$curved, by = c("from", "to"), suffix = c(".x", "")) %>%
        dplyr::select("from", "to", "curved")
      edge_curved[is.na(edge_curved)] <- 0
      edge_curved <- data.frame(edge_curved, row.names = NULL)
    }
  }
  # 4.如果是matrix，变成df
  if (is.data.frame(coors)) {
    if (!"name" %in% colnames(coors)) coors <- as.matrix(coors)
  }
  if (is.matrix(coors)) coors <- data.frame(name = V(go)$name, X = coors[, 1], Y = coors[, 2], row.names = NULL)
  # 5.如果是df了，那就对齐name用于下一步的绘图，
  if (is.data.frame(coors)) {
    coors <- data.frame(coors, check.names = FALSE)
    setdiff(V(go)$name, coors$name) -> tmp_name
    if (length(setdiff(V(go)$name, coors$name)) > 0) {
      warning("Some nodes are not in coors, will be randomly assigned")
      coors <- rbind(
        coors,
        data.frame(
          name = tmp_name,
          X = runif(length(tmp_name), range(coors$X)[1], range(coors$X)[2]),
          Y = runif(length(tmp_name), range(coors$Y)[1], range(coors$Y)[2])
        )
      )
    }
    coors <- coors[match(V(go)$name, coors$name), ]
    coors <- as_coors(coors, edge_curved)
    return(coors)
  }
  stop("coors wrong!")
}

rescale_coors <- function(coors, keep_asp = TRUE) {
  stopifnot(inherits(coors, "coors"))
  if (any(is.na(coors$X)) || any(is.na(coors$Y))) {
    stop("Some coordinates are NA")
  }
  if (keep_asp) {
    diff_x <- diff(range(coors$X))
    diff_y <- diff(range(coors$Y))
    if (diff_x > diff_y) {
      coors$X <- mmscale(coors$X, -1, 1)
      coors$Y <- mmscale(coors$Y, -1 * diff_y / diff_x, 1 * diff_y / diff_x)
    } else {
      coors$X <- mmscale(coors$X, -1 * diff_x / diff_y, 1 * diff_x / diff_y)
      coors$Y <- mmscale(coors$Y, -1, 1)
    }
  } else {
    coors$X <- mmscale(coors$X, -1, 1)
    coors$Y <- mmscale(coors$Y, -1, 1)
  }
  coors
}

combine_coors <- function(..., list = NULL) {
  name <- from <- to <- NULL
  list <- c(list(...), list)
  if (!all(vapply(list, \(i)inherits(i, "coors"), logical(1)))) stop("some input are not coors object")
  coors <- lapply(list, \(i)i) %>% do.call(rbind, .)
  curved <- lapply(list, \(i)attributes(i)$curved) %>% do.call(rbind, .)
  if (any(duplicated(coors$name))) {
    warning("some duplicated name in coors")
    coors <- dplyr::distinct(coors, name, .keep_all = TRUE)
  }
  if (!is.null(curved)) {
    curved2 <- dplyr::distinct(curved, from, to, .keep_all = TRUE)
    if (nrow(curved2) != nrow(curved)) warning("some duplicates in attributes(i)$curved")
    curved <- curved2
  }
  coors <- as_coors(coors, curved)
  return(coors)
}

#' Transform the layout of a 'coors' object
#'
#' This function applies various transformations to a 'coors' object, including
#' scaling, aspect ratio adjustment, rotation, mirroring, and pseudo-3D perspective transformation.
#'
#' @param coors An object of class 'coors', containing node coordinates.
#' @param scale A numeric value to scale the layout (default = 1).
#' @param aspect_ratio A numeric value to adjust the Y-axis scaling (default = 1).
#' @param rotation A numeric value in degrees to rotate the layout (default = 0).
#' @param mirror_x A logical value indicating whether to mirror along the X-axis (default = FALSE).
#' @param mirror_y A logical value indicating whether to mirror along the Y-axis (default = FALSE).
#' @param shear_x A numeric value to apply a shear transformation in the X direction (default = 0).
#' @param shear_y A numeric value to apply a shear transformation in the Y direction (default = 0).
#'
#' @return A transformed 'coors' object with updated coordinates.
#' @export
transform_coors <- function(coors, scale = 1, aspect_ratio = 1,
                            rotation = 0, mirror_x = FALSE, mirror_y = FALSE,
                            shear_x = 0, shear_y = 0) {
  stopifnot(inherits(coors, "coors"))

  # 复制原始数据
  new_coor <- coors

  mid_x <- mean(new_coor$X)
  mid_y <- mean(new_coor$Y)

  new_coor$X <- new_coor$X - mid_x
  new_coor$Y <- new_coor$Y - mid_y

  # 放大/缩小
  new_coor$X <- new_coor$X * scale
  new_coor$Y <- new_coor$Y * scale * aspect_ratio

  # 旋转（角度转换为弧度）
  theta <- rotation * pi / 180
  new_x <- new_coor$X * cos(theta) - new_coor$Y * sin(theta)
  new_y <- new_coor$X * sin(theta) + new_coor$Y * cos(theta)
  new_coor$X <- new_x
  new_coor$Y <- new_y

  # 透视投影（shear变换）
  new_coor$X <- new_coor$X + shear_x * new_coor$Y
  new_coor$Y <- new_coor$Y + shear_y * new_coor$X

  # 镜像变换
  if (mirror_x) new_coor$X <- -new_coor$X
  if (mirror_y) new_coor$Y <- -new_coor$Y
  # 还原到原始位置
  new_coor$X <- new_coor$X + mid_x
  new_coor$Y <- new_coor$Y + mid_y

  coors <- new_coor
  return(coors)
}

#' Layout as a line
#'
#' @param angle anticlockwise rotation angle
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_line()(co_net)
#' c_net_plot(co_net, coors = as_line(pi / 2))
as_line <- function(angle = 0) {
  fun <- \(go){
    nv <- length(V(go))
    data.frame(
      x = seq(-cos(angle), cos(angle), len = nv),
      y = seq(-sin(angle), sin(angle), len = nv)
    ) %>%
      as.matrix() %>%
      round(., 4)
  }
  class(fun) <- c("line", "layout", "function")
  fun
}

#' Layout as a arc
#'
#' @param angle anticlockwise rotation angle
#' @param arc the radian of arc
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_arc()(co_net)
#' c_net_plot(co_net, coors = as_arc(pi / 2))
as_arc <- function(angle = 0, arc = pi) {
  fun <- \(go){
    # (0,0) is the midpoint of circle
    nv <- length(V(go))
    theta <- seq(-arc / 2 + angle, arc / 2 + angle, len = nv)
    coor <- data.frame(x = cos(theta), y = sin(theta))
    as.matrix(coor) %>% round(., 4)
  }
  class(fun) <- c("layout", "function")
  fun
}

#' Layout as a polygon
#'
#' @param n how many edges of this polygon
#' @param line_curved line_curved 0~0.5
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_polygon()(co_net)
as_polygon <- function(n = 3, line_curved = 0.5) {
  fun <- \(go, group2 = NULL, group2_order = NULL){
    V(go)$poly_group <- rep(paste0("omic", seq_len(n)), len = length(go))
    if (n < 2) stop("n should bigger than 1")
    g_layout_polygon(go,
      group = "poly_group", group2 = group2, group2_order = group2_order,
      line_curved = line_curved
    ) -> oridata
    oridata
  }
  class(fun) <- c("poly", "layout", "function")
  fun
}

#' Layout as a polyarc
#'
#' @param n how many arcs of this poly_arc
#' @param space the space between each arc, default: pi/3
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_polyarc()(co_net)
as_polyarc <- \(n = 3, space = pi / 3){
  fun <- \(go, group2 = NULL, group2_order = NULL){
    V(go)$poly_group <- rep(paste0("omic", 1:n), len = length(go))
    if (n < 2) stop("n should bigger than 1")
    g_layout_polyarc(go, "poly_group", space = space, group2 = group2, group2_order = group2_order) -> oridata
    oridata
  }
  class(fun) <- c("poly", "layout", "function")
  fun
}

#' Layout as a polycircle
#'
#' @param n how many circles of this polycircle
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_polycircle()(co_net)
as_polycircle <- \(n = 5){
  fun <- \(go, group2 = NULL, group2_order = NULL){
    V(go)$poly_group <- rep(paste0("omic", 1:n), distribute_points_in_concentric_circles(length(go), n))
    if (n < 2) stop("n should bigger than 1")
    g_layout_polycircle(go, "poly_group", group2 = group2, group2_order = group2_order) -> oridata
    oridata
  }
  class(fun) <- c("poly", "layout", "function")
  fun
}


distribute_points_in_concentric_circles <- function(n, k = NULL, mode = c("linear", "perimeter")) {
  mode <- match.arg(mode)

  if (is.null(k)) {
    # 自动决定层数：sqrt(n) 是个常用经验
    k <- ceiling(sqrt(n))
  }

  layer_weights <- switch(mode,
    linear = 1:k, # 权重 1, 2, ..., k
    perimeter = 1:k # 半径与层号成比例，周长也是
  )

  # 归一化
  weight_sum <- sum(layer_weights)
  raw_counts <- n * layer_weights / weight_sum

  # 四舍五入并微调使总和为n
  counts <- round(raw_counts)
  diff_n <- n - sum(counts)
  if (diff_n != 0) {
    # 按最大余数或最小绝对值误差来调整
    adjust_order <- order(abs(raw_counts - counts), decreasing = TRUE)
    for (i in seq_len(abs(diff_n))) {
      idx <- adjust_order[i]
      counts[idx] <- counts[idx] + sign(diff_n)
    }
  }

  return(counts)
}

#' Layout as a multi_layer
#'
#' @param n how many arcs of this multi_layer
#' @param layout see method in \code{\link[MetaNet]{c_net_layout}}
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
#' @examples
#' as_multi_layer()(co_net)
as_multi_layer <- \(n = 3, layout = on_grid()){
  fun <- \(go, group2 = NULL, group2_order = NULL){
    V(go)$poly_group <- rep(paste0("omic", 1:n), len = length(go))
    if (n < 2) stop("n should bigger than 1")
    g_layout_multi_layer(go, layout = layout, group = "poly_group", group2 = group2, group2_order = group2_order) -> oridata
    oridata
  }
  class(fun) <- c("poly", "layout", "function")
  fun
}

#' Layout as a multi_layer
#'
#' @param n how many arcs of this multi_layer
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
as_poly_sector <- \(n = 3){
  fun <- \(go, group2 = NULL, group2_order = NULL){
    V(go)$poly_group <- rep(paste0("omic", 1:n), len = length(go))
    if (n < 2) stop("n should bigger than 1")
    g_layout_poly_sector(go, group = "poly_group", group2 = group2, group2_order = group2_order) -> oridata
    oridata
  }
  class(fun) <- c("poly", "layout", "function")
  fun
}


#' Layout as a circle_tree
#'
#' @return A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
#' @export
#' @family layout
as_circle_tree <- \(){
  fun <- \(go){
    name <- x <- y <- NULL
    lib_ps("ggraph", library = FALSE)
    data <- ggraph::create_layout(clean_igraph(go), layout = "igraph", algorithm = "tree", circular = TRUE)
    coors <- data %>% dplyr::select(name, x, y)
    colnames(coors) <- c("name", "X", "Y")
    coors
  }
  class(fun) <- c("layout", "function")
  fun
}

big_layout <- \(zoom1, layout1, nodeGroup){
  tmp_g <- igraph::make_ring(nlevels(nodeGroup$group))
  V(tmp_g)$name <- levels(nodeGroup$group)
  c_net_update(tmp_g) -> tmp_da_net
  da <- get_coors(layout1, tmp_da_net)
  rownames(da) <- da$name
  da <- da[, c("X", "Y")]

  # center of each group
  scale_f <- (ceiling(max(da) - min(da)))
  scale_f <- ifelse(scale_f == 0, 1, scale_f / 2)
  da <- da / scale_f * zoom1
  return(da)
}

#' Layout with group
#'
#' @param go igraph or metanet object
#' @param group group name (default: module)
#' @param zoom1 big network layout size
#' @param zoom2 average sub_network layout size, or numeric vector, or "auto"
#' @param layout1 layout1 method, one of
#'      (1) a dataframe or matrix: rowname is group, two columns are X and Y
#'      (2) function: layout method for \code{\link{c_net_layout}} default: in_circle()
#' @param layout2 one of functions: layout method for \code{\link{c_net_layout}}, or a list of functions.
#' @param show_big_layout show the big layout to help you adjust.
#' @param ... add
#' @param group_order group_order
#' @param rescale logical, scale the X, Y to (-1,1)
#'
#' @return coors
#' @export
#' @family g_layout
#' @examples
#' \donttest{
#' data("c_net")
#' module_detect(co_net, method = "cluster_fast_greedy") -> co_net_modu
#' g_layout(co_net_modu, group = "module", zoom1 = 30, zoom2 = "auto", layout2 = as_line()) -> oridata
#' plot(co_net_modu, coors = oridata)
#' }
g_layout <- function(go, group = "module", group_order = NULL, layout1 = in_circle(), zoom1 = 20, layout2 = in_circle(),
                     zoom2 = 3, show_big_layout = FALSE, rescale = TRUE, ...) {
  name <- NULL

  stopifnot(is_igraph(go))
  if (!group %in% igraph::vertex_attr_names(go)) stop("no group named ", group, " !")
  get_v(go) %>% dplyr::select(name, !!group) -> nodeGroup
  colnames(nodeGroup) <- c("ID", "group")
  nodeGroup$group <- factor(nodeGroup$group)
  stopifnot(nlevels(nodeGroup$group) > 1)

  if (!is.null(group_order)) nodeGroup$group <- pcutils::change_fac_lev(nodeGroup$group, group_order)

  da <- big_layout(zoom1, layout1, nodeGroup)
  if (show_big_layout) {
    message("Big layout:")
    print(da)
    graphics::plot.new()
    plot(da, pch = 21, bty = "n", bg = get_cols(nrow(da), "col2"), main = "Big layout coordinates")
    graphics::text(da$X * 0.8, da$Y * 0.9, rownames(da))
    return(invisible())
  }

  # layout of vertexes in one group
  {
    layoutls <- list()
    if (is_layout(layout2)) {
      layoutls <- rep(list(layout2), nlevels(nodeGroup$group))
    } else if (all(class(layout2) == "list")) {
      if (is.null(names(layout2))) {
        layoutls <- rep(layout2, len = nlevels(nodeGroup$group))
      } else {
        for (i in levels(nodeGroup$group)) {
          if (i %in% names(layout2)) {
            layoutls[[i]] <- layout2[[i]]
          } else {
            layoutls[[i]] <- igraph::in_circle()
            warning("layout of ", i, " not set, use in_circle()")
          }
        }
      }
    }
    if (is.null(names(layoutls))) names(layoutls) <- levels(nodeGroup$group)

    zoom2 <- rep(zoom2, nlevels(nodeGroup$group))
    if (zoom2[1] == "auto") zoom2 <- ceiling((table(nodeGroup$group))^(1 / 3))
    names(zoom2) <- levels(nodeGroup$group)
  }

  # get coors
  all_coors <- setNames(vector("list", nlevels(nodeGroup$group)), levels(nodeGroup$group))
  for (i in levels(nodeGroup$group)) {
    # print(i)
    nodeGroup[nodeGroup[, "group"] == i, "ID"] -> tmpid
    igraph::subgraph(go, tmpid) -> tmp_net

    get_coors(layoutls[[i]], tmp_net, ...) -> coors
    data <- coors
    if (nrow(data) == 1) {
      data <- data.frame(name = data$name, X = 0, Y = 0) %>% as_coors()
    }
    if ("igraph_layout_spec" %in% class(layoutls[[i]])) {
      data[, c("X", "Y")] <- igraph::norm_coords(as.matrix(data[, c("X", "Y")]))
    }
    data[, "X"] <- data[, "X"] * zoom2[i] + da[i, "X"]
    data[, "Y"] <- data[, "Y"] * zoom2[i] + da[i, "Y"]

    coors <- data
    all_coors[[i]] <- coors
  }
  coors <- combine_coors(list = all_coors)
  if (rescale) {
    coors <- rescale_coors(coors)
  }
  return(coors)
}

#' Layout with group as a polygon
#'
#' @param go igraph
#' @param group group name (default:v_group)
#' @param group2 group2 name, will order nodes in each group according to group2_order
#' @param group2_order group2_order
#' @param line_curved line_curved 0~1
#' @param group_order group_order
#'
#' @return coors
#' @export
#' @family g_layout
#' @examples
#' g_layout_polygon(multi1) -> oridata
#' c_net_plot(multi1, oridata)
#' g_layout_polyarc(multi1, group2 = "v_class", group2_order = c(LETTERS[4:1])) -> oridata
#' c_net_plot(multi1, oridata)
#' g_layout_polycircle(co_net2, group2 = "v_class") -> oridata
#' c_net_plot(co_net2, oridata)
#' g_layout_multi_layer(co_net2, group2 = "v_class") -> oridata
#' c_net_plot(co_net2, oridata)
g_layout_polygon <- function(go, group = "v_group", group_order = NULL, group2 = NULL, group2_order = NULL, line_curved = 0.5) {
  n <- length(unique(igraph::vertex.attributes(go)[[group]]))

  if (n < 2) stop("n should bigger than 1")
  `_internal_group` <- NULL
  get_v(go) %>% dplyr::pull(!!group) -> group1
  group1_level <- table(group1)
  V(go)$`_internal_group` <- group1

  angle_ls <- -pi / 2 + (seq(0, n - 1, 1)) * 2 * pi / n
  names(angle_ls) <- pcutils::change_fac_lev(names(group1_level), group_order) %>% levels()

  layout2_ls <- list()
  for (i in names(group1_level)) {
    c_net_filter(go, `_internal_group` == i) %>%
      c_net_layout(
        method = as_line(angle_ls[i]), order_by = group2,
        order_ls = group2_order, rescale = FALSE
      ) -> layout2_ls[[i]]
  }

  g_layout(go,
    group_order = group_order,
    group = group, zoom1 = 1, zoom2 = 0.9 * (ifelse(n > 2, tan(pi / n), 2)),
    layout2 = layout2_ls, order_by = group2, order_ls = group2_order
  ) -> oridata

  if (is.data.frame(oridata$curved)) oridata$curved$curved <- line_curved
  oridata
}


#' Layout with group as a polyarc
#'
#' @param space the space between each arc, default: pi/4
#' @param scale_node_num scale with the node number in each group
#'
#' @rdname g_layout_polygon
#' @export
g_layout_polyarc <- function(go, group = "v_group", group_order = NULL,
                             group2 = NULL, group2_order = NULL, space = pi / 4,
                             scale_node_num = TRUE) {
  get_v(go) -> tmp_v
  group1 <- as.factor(tmp_v[, group])
  n <- nlevels(group1)
  if (n < 2) stop("n should bigger than 1")

  if (!is.null(group_order)) group1 <- pcutils::change_fac_lev(group1, group_order)
  # consider each group numbers!!!
  g_num <- table(group1)
  sep <- space / n

  if (scale_node_num) {
    arc_r <- (2 * pi - space) * as.numeric(g_num) / length(group1)
  } else {
    arc_r <- rep((2 * pi - space) / n, n)
  }

  names(arc_r) <- levels(group1)

  # coordinate
  coors <- data.frame()
  theta1 <- 0
  for (i in names(arc_r)) {
    tmp_t <- seq(theta1, theta1 + arc_r[i], len = g_num[i])
    tmp_v1 <- tmp_v[tmp_v[, group] == i, ]
    tmp_coor <- order_tmp_v_name(tmp_v1, data.frame(X = cos(tmp_t), Y = sin(tmp_t)), group2, group2_order)

    coors <- rbind(coors, tmp_coor)
    theta1 <- theta1 + arc_r[i] + sep
  }
  coors <- as_coors(coors)
  coors <- rescale_coors(coors)
  coors
}


#' Layout with group as a polycircle
#'
#' @rdname g_layout_polygon
#' @export
g_layout_polycircle <- function(go, group = "v_group", group_order = NULL, group2 = NULL, group2_order = NULL) {
  name <- NULL
  n <- length(unique(igraph::vertex.attributes(go)[[group]]))
  if (n < 2) stop("n should bigger than 1")
  get_v(go) %>% dplyr::select(name, !!group) -> nodeGroup

  if (is.null(group_order)) {
    group_order <- table(nodeGroup[, group]) %>%
      sort() %>%
      names()
  }

  g_layout(go,
    group_order = group_order,
    group = group, layout1 = matrix(0, nrow = n, ncol = 2),
    zoom1 = 1, zoom2 = 1:n,
    layout2 = igraph::in_circle(), order_by = group2, order_ls = group2_order
  ) -> oridata
  oridata
}

#' Layout with group as a multi_layer
#'
#' @param layout see method in \code{\link[MetaNet]{c_net_layout}}
#'
#' @rdname g_layout_polygon
#' @export
g_layout_multi_layer <- function(go, layout = igraph::in_circle(), group = "v_group", group_order = NULL,
                                 group2 = NULL, group2_order = NULL, scale_node_num = TRUE) {
  n <- length(unique(igraph::vertex.attributes(go)[[group]]))
  if (n < 2) stop("n should bigger than 1")

  get_v(go) %>%
    dplyr::pull(!!group) %>%
    as.factor() -> group1
  if (!is.null(group_order)) group1 <- pcutils::change_fac_lev(group1, group_order)
  group1_level <- table(group1)
  `_internal_group` <- NULL
  V(go)$`_internal_group` <- group1

  # layout of vertexes in one group
  {
    layoutls <- list()
    if (is_layout(layout)) {
      layoutls <- rep(list(layout), nlevels(group1))
    } else if (all(class(layout) == "list")) {
      if (is.null(names(layout))) {
        layoutls <- rep(layout, len = nlevels(group1))
      } else {
        for (i in levels(group1)) {
          if (i %in% names(layout)) {
            layoutls[[i]] <- layout[[i]]
          } else {
            layoutls[[i]] <- igraph::in_circle()
            warning("layout of ", i, " not set, use in_circle()")
          }
        }
      }
    }
    if (is.null(names(layoutls))) names(layoutls) <- levels(group1)
  }

  layout2_ls <- list()
  for (i in names(group1_level)) {
    c_net_filter(go, `_internal_group` == i) %>%
      c_net_layout(method = layoutls[[i]], order_by = group2, order_ls = group2_order) %>%
      transform_coors(shear_x = 3, aspect_ratio = 0.2) -> layout2_ls[[i]]
  }

  if (scale_node_num) {
    zoom2 <- group1_level / mean(group1_level)
  } else {
    zoom2 <- rep(1, nlevels(group1))
  }

  g_layout(go,
    group_order = group_order,
    group = group, layout1 = data.frame(X = 0, Y = seq(-1, 1, length = length(group1_level))),
    zoom1 = 1, zoom2 = zoom2,
    layout2 = layout2_ls
  ) -> oridata
  oridata
}

#' Layout with group nicely
#'
#' @param go igraph or metanet
#' @param group group name (default: module)
#' @param mode circlepack, treemap, backbone, stress
#' @param ... add
#' @param group_zoom zoom for each group
#'
#' @export
#' @return coors
#' @family g_layout
#' @examples
#' \donttest{
#' data("c_net")
#' module_detect(co_net, method = "cluster_fast_greedy") -> co_net_modu
#' if (requireNamespace("ggraph")) {
#'   plot(co_net_modu, coors = g_layout_nice(co_net_modu, group = "module"))
#'   plot(co_net_modu, coors = g_layout_nice(co_net_modu, group = "module", mode = "treemap"))
#' }
#' }
g_layout_nice <- function(go, group = "module", mode = "circlepack", group_zoom = 1, ...) {
  name <- x <- y <- NULL
  lib_ps("ggraph", library = FALSE)
  stopifnot(is_igraph(go))

  mode <- match.arg(mode, c("circlepack", "treemap", "backbone", "stress"))

  if (!group %in% vertex_attr_names(go)) stop("no group named ", group, " !")
  get_v(go) %>% dplyr::select(name, !!group) -> nodeGroup
  colnames(nodeGroup) <- c("ID", "group")
  nodeGroup$group <- as.factor(nodeGroup$group)

  edge <- data.frame(from = paste("group_", nodeGroup$group, sep = ""), to = nodeGroup$ID)

  directed <- TRUE
  if (mode %in% c("backbone")) directed <- FALSE

  mygraph <- igraph::graph_from_data_frame(edge, directed = directed)
  data <- ggraph::create_layout(mygraph, layout = mode, ...)
  coors <- data %>% dplyr::select(name, x, y)
  colnames(coors) <- c("name", "X", "Y")
  coors <- as_coors(coors)

  for (i in unique(nodeGroup$group)) {
    tmp_id <- nodeGroup[nodeGroup[, "group"] == i, "ID"]
    transform_coors(coors[coors$name %in% tmp_id, ], scale = group_zoom) -> coors[coors$name %in% tmp_id, ]
  }

  coors <- rescale_coors(coors)
  return(coors)
}

#' @rdname g_layout_nice
#' @export
get_big_lay_nice <- function(go, group = "module", mode = "circlepack", ...) {
  coors <- g_layout_nice(go, group, mode, ...)
  # 提取circlepack的分组坐标
  big_lay <- coors[grepl("group_", coors$name), ]
  big_lay$name <- gsub("group_", "", big_lay$name)
  return(big_lay)
}

#' @rdname g_layout_nice
#' @export
g_layout_circlepack <- \(go, group = "module", ...){
  g_layout_nice(go, group = group, mode = "circlepack", ...)
}

#' @rdname g_layout_nice
#' @export
g_layout_treemap <- \(go, group = "module", ...){
  g_layout_nice(go, group = group, mode = "treemap", ...)
}

#' @rdname g_layout_nice
#' @export
g_layout_backbone <- \(go, group = "module", ...){
  g_layout_nice(go, group = group, mode = "backbone", ...)
}

#' @rdname g_layout_nice
#' @export
g_layout_stress <- \(go, group = "module", ...){
  g_layout_nice(go, group = group, mode = "stress", ...)
}


#' Generate spatial layout using `spatstat`
#'
#' @param go igraph or metanet object
#' @param win A spatstat window object (owin), e.g. disc(), owin(poly=...); Or sf object.
#' @param type Type of distribution: "random", "regular"
#' @param mode "surface", "boundary"
#' @param jitter for surface-regular, defalut 0
#' @param curved Optional curved attribute for coors
#' @param order_by order nodes according to a node attribute
#' @param order_ls manual the discrete variable with a vector, or continuous variable with "desc" to decreasing
#' @param seed random seed
#' @param rescale rescale the coordinates to (0,1)
#' @param order_circle order nodes from the center of a circle
#'
#' @return A coors object (data.frame with class "coors" and attribute "curved")
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("spatstat.geom") && requireNamespace("spatstat.random")) {
#'   poly_x <- c(0, 2, 2, 0)
#'   poly_y <- c(0, 0, 1, 1)
#'   win_poly <- spatstat.geom::owin(poly = list(x = poly_x, y = poly_y))
#'   plot(win_poly)
#'   coors1 <- spatstat_layout(co_net, win_poly, type = "regular", mode = "surface")
#'   plot(co_net, coors = coors1)
#'   coors2 <- spatstat_layout(co_net2, win_poly, type = "random", mode = "boundary")
#'   plot(co_net2, coors = coors2)
#'
#'   if (requireNamespace("sf")) {
#'     nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
#'     poly <- nc[1, ]
#'     coors <- spatstat_layout(go = multi1, win = poly, type = "regular", mode = "surface")
#'     plot(multi1, coors = coors)
#'   }
#' }
#' }
spatstat_layout <- function(go, win,
                            type = c("random", "regular"),
                            mode = c("surface", "boundary"),
                            jitter = 0,
                            curved = NULL,
                            order_by = NULL, order_ls = NULL, order_circle = FALSE,
                            seed = 1234, rescale = TRUE) {
  lib_ps("spatstat.geom", library = FALSE)
  lib_ps("spatstat.random", library = FALSE)

  type <- match.arg(type)
  mode <- match.arg(mode)

  if (inherits(win, "sf")) {
    win <- sf_to_owin(win)
  }
  if (!inherits(win, "owin")) stop("Input 'win' must be a spatstat 'owin' object.")

  n_nodes <- length(go)
  set.seed(seed)
  pts <- switch(mode,
    surface = switch(type,
      random = generate_random_points(n_nodes, win),
      regular = generate_regular_points(n_nodes, win, jitter = jitter)
    ),
    boundary = switch(type,
      random = generate_boundary_points_random(n_nodes, win),
      regular = generate_boundary_points_regular(n_nodes, win)
    )
  )
  if (!is.null(attributes(pts)$rejects)) {
    pts$x <- c(pts$x, attributes(pts)$rejects$x)
    pts$y <- c(pts$y, attributes(pts)$rejects$y)
  }

  if (!order_circle) {
    coors <- data.frame(X = pts$x, Y = pts$y) %>%
      dplyr::arrange_all() %>%
      as.matrix()
  } else {
    coors <- order_circle_func(as.matrix(data.frame(X = pts$x, Y = pts$y)))
  }

  # order
  get_v(go) -> tmp_v
  coors <- order_tmp_v_name(tmp_v, coors, order_by = order_by, order_ls = order_ls)

  coors <- as_coors(coors, curved = curved)
  if (rescale) coors <- rescale_coors(coors)
  coors
}

generate_regular_points <- function(n_nodes, win, max_iter = 20, jitter = 0) {
  xlen <- diff(win$xrange)
  ylen <- diff(win$yrange)
  ratio <- xlen / ylen

  for (factor in seq(1.1, 5, length.out = max_iter)) {
    ny <- ceiling(sqrt(n_nodes / ratio) * factor)
    nx <- ceiling(ny * ratio)

    gx <- seq(win$xrange[1], win$xrange[2], length.out = nx)
    gy <- seq(win$yrange[1], win$yrange[2], length.out = ny)
    grid <- expand.grid(x = gx, y = gy)

    if (jitter > 0) {
      gx_step <- mean(diff(gx))
      gy_step <- mean(diff(gy))
      grid$x <- grid$x + runif(nrow(grid), -jitter * gx_step, jitter * gx_step)
      grid$y <- grid$y + runif(nrow(grid), -jitter * gy_step, jitter * gy_step)
    }

    inside <- spatstat.geom::inside.owin(grid$x, grid$y, win)
    valid_pts <- grid[inside, ]
    if (nrow(valid_pts) >= n_nodes) {
      selected <- valid_pts[sample(nrow(valid_pts), n_nodes), ]
      return(spatstat.geom::ppp(selected$x, selected$y, window = win))
    }
  }
  stop("Failed to generate enough regularly spaced points inside window after multiple attempts.")
}


generate_random_points <- function(n_nodes, win) {
  spatstat.random::runifpoint(n_nodes, win = win)
}

generate_boundary_points_random <- function(n_nodes, win) {
  bnd <- spatstat.geom::edges(win)
  pts <- spatstat.random::runifpointOnLines(n_nodes, bnd)
  spatstat.geom::ppp(pts$x, pts$y, window = win)
}

generate_boundary_points_regular <- function(n_nodes, win) {
  bnd <- spatstat.geom::edges(win)
  len <- spatstat.geom::lengths_psp(bnd)
  total_len <- sum(len)
  t_seq <- seq(0, total_len, length.out = n_nodes + 1)[-1]

  pts_x <- numeric(n_nodes)
  pts_y <- numeric(n_nodes)
  cum_len <- cumsum(len)

  for (i in seq_along(t_seq)) {
    t <- t_seq[i]
    seg_idx <- which(t <= cum_len)[1]
    t0 <- if (seg_idx == 1) 0 else cum_len[seg_idx - 1]
    frac <- (t - t0) / len[seg_idx]
    seg <- bnd[seg_idx]
    pts_x[i] <- seg$ends$x0 + frac * (seg$ends$x1 - seg$ends$x0)
    pts_y[i] <- seg$ends$y0 + frac * (seg$ends$y1 - seg$ends$y0)
  }

  spatstat.geom::ppp(pts_x, pts_y, window = win)
}

sf_to_owin <- function(sf_obj) {
  lib_ps("sf", library = FALSE)
  lib_ps("spatstat.geom", library = FALSE)

  # 确保输入是 sf 对象
  if (!inherits(sf_obj, "sf")) stop("Input must be an 'sf' object.")
  if (!sf::st_is(sf_obj, "POLYGON") && !sf::st_is(sf_obj, "MULTIPOLYGON")) {
    stop("sf object must be a POLYGON or MULTIPOLYGON")
  }

  # 修复方向问题
  sf_obj <- sf::st_make_valid(sf_obj) # 处理无效几何问题
  # 强制外环为逆时针，内环为顺时针
  sf_obj <- sf::st_set_geometry(sf_obj, sf::st_geometry(sf_obj))
  sf_obj <- sf::st_simplify(sf_obj) # 简化几何

  # 将 sf 对象转为 polylist 坐标
  poly_coords <- sf::st_coordinates(sf_obj)[, 1:2]
  polylist <- list(x = poly_coords[, 1], y = poly_coords[, 2])

  # 创建 spatstat 的 owin 对象
  win <- spatstat.geom::owin(poly = list(polylist))
  return(win)
}


create_sector_window <- function(r0 = 0, r1 = 1, theta_start = 0, theta_end = pi) {
  lib_ps("spatstat.geom", library = FALSE)
  # theta in radians
  n <- 100
  angles <- seq(theta_start, theta_end, length.out = n)
  x_outer <- r1 * cos(angles)
  y_outer <- r1 * sin(angles)
  x_inner <- r0 * cos(rev(angles))
  y_inner <- r0 * sin(rev(angles))
  x <- c(x_outer, x_inner)
  y <- c(y_outer, y_inner)
  spatstat.geom::owin(poly = list(x = x, y = y))
}


#' Layout with group
#'
#' @param go igraph
#' @param group group name (default:v_group)
#' @param group2 group2 name, will order nodes in each group according to group2_order
#' @param group2_order group2_order
#' @param group_order group_order
#' @param space the space between each arc, default: pi/4
#' @param scale_node_num scale with the node number in each group
#' @param type Type of distribution: "random", "regular"
#' @param mode "surface", "boundary"
#' @param jitter for surface-regular, defalut 0
#' @param curved Optional curved attribute for coors
#'
#' @return coors
#' @export
#' @family g_layout
g_layout_poly_sector <- function(go,
                                 group = "v_group", group_order = NULL,
                                 group2 = NULL, group2_order = NULL,
                                 space = pi / 4, jitter = 0,
                                 scale_node_num = TRUE,
                                 type = c("regular", "random"),
                                 mode = c("surface", "boundary"),
                                 curved = NULL) {
  type <- match.arg(type)
  mode <- match.arg(mode)

  get_v(go) -> tmp_v
  group1 <- as.factor(tmp_v[[group]])
  if (!is.null(group_order)) group1 <- pcutils::change_fac_lev(group1, group_order)

  n <- nlevels(group1)
  if (n < 2) stop("n should be greater than 1")

  g_num <- table(group1)
  sep <- space / n

  # 分配每组所占的弧度
  if (scale_node_num) {
    arc_r <- (2 * pi - space) * as.numeric(g_num) / sum(g_num)
  } else {
    arc_r <- rep((2 * pi - space) / n, n)
  }
  names(arc_r) <- levels(group1)

  coors_all <- list()
  theta1 <- 0
  for (i in names(arc_r)) {
    tmp_nodes <- tmp_v[tmp_v[[group]] == i, ]
    sub_go <- igraph::make_empty_graph(n = nrow(tmp_nodes)) %>%
      igraph::set_vertex_attr("name", value = tmp_nodes$name)

    win <- create_sector_window(r0 = 0, r1 = 1, theta_start = theta1, theta_end = theta1 + arc_r[i])
    tmp_layout <- spatstat_layout(sub_go, win = win, type = type, mode = mode, curved = curved, rescale = FALSE, jitter = jitter)

    tmp_layout <- order_tmp_v_name(tmp_nodes, tmp_layout[, c("X", "Y")], group2, group2_order)
    coors_all[[i]] <- tmp_layout
    theta1 <- theta1 + arc_r[i] + sep
  }

  # 合并所有 layout
  coors_df <- do.call(rbind, coors_all)
  as_coors(coors_df, curved = curved) %>% rescale_coors()
}

# =========2.2RMT optimize=====

#' Get RMT threshold for a correlation matrix
#'
#' @param occor.r a corr object or a correlation matrix
#' @param min_threshold min_threshold
#' @param max_threshold max_threshold
#' @param step step
#' @param plot plot pngs?
#' @param gif render a .gif file?
#' @param verbose verbose
#' @param out_dir output dir
#'
#' @return a r-threshold
#' @export
#' @references
#' J. Zhou, Y. Deng, FALSE. Luo, Z. He, Q. Tu, X. Zhi, (2010) Functional Molecular Ecological Networks, doi:10.1128/mBio.00169-10.
#' <https://matstat.org/content_en/RMT/RMThreshold_Intro.pdf>
#' @examples
#' \donttest{
#' data(otutab, package = "pcutils")
#' t(otutab) -> totu
#' c_net_calculate(totu) -> corr
#' rmt(corr)
#' # recommend: 0.69
#' c_net_build(corr, r_threshold = 0.69) -> co_net_rmt
#' }
RMT_threshold <- function(occor.r, out_dir, min_threshold = 0.5, max_threshold = 0.8,
                          step = 0.02, plot = FALSE, gif = FALSE, verbose = FALSE) {
  nwd <- getwd()
  on.exit(setwd(nwd))

  setwd(out_dir)
  if (inherits(occor.r, "corr")) occor.r <- occor.r$r
  if (plot) {
    if (!dir.exists("./RMT_temp")) dir.create("./RMT_temp")
  }
  diag(occor.r) <- 0

  if (max_threshold >= max(abs(occor.r))) max_threshold <- (max(abs(occor.r)) - step)
  if (min_threshold >= max_threshold) min_threshold <- max_threshold - 10 * step

  thres_seq <- seq(min_threshold, max_threshold, step)

  res <- data.frame()
  for (i in seq_len(length(thres_seq))) {
    threshold <- thres_seq[i]
    if (!verbose) pcutils::dabiao(paste0("Calculating", i, ":  threshold =", signif(threshold, 3)), print = TRUE)
    corr_r1 <- occor.r
    corr_r1[abs(corr_r1) < threshold] <- 0
    # calculate eigenvalues
    rand.mat <- corr_r1
    eigenvalues <- eigen(rand.mat, only.values = TRUE)$values
    eigenvalues <- eigenvalues[order(eigenvalues)] / max(abs(eigenvalues))
    eigenvalues <- pcutils::remove.outliers(unique(eigenvalues))

    # get the NNDS
    { # uf <- rm.unfold.gauss(eigenvalues,pop.up = TRUE)
      dens <- density(eigenvalues, kernel = "gaussian")
      midpoints <- \(x)(x[-length(x)] + 0.5 * diff(x))
      scale.function <- approx(dens$x, dens$y, xout = midpoints(eigenvalues))
      ev.spacing <- diff(eigenvalues)
      ev.spacing <- ev.spacing * scale.function$y
      ev.spacing <- ev.spacing / mean(ev.spacing)
    }

    ev.spacing <- ev.spacing[ev.spacing <= 3]
    # test whether fit possion?
    p_ks_test <- ks.test(unique(ev.spacing), "pexp", 1)$p.value
    # get sse
    # sse = rm.sse(ev.spacing)
    sse <- get_sse(ev.spacing)
    log_sse <- log(sse)

    # maximum likelihood
    evs <- ev.spacing[ev.spacing != 0]
    N <- length(evs)
    log_LE <- -sum(evs) / N
    log_LW <- log(pi / 2) + sum(log(evs)) / N - 0.25 * pi * sum(evs^2) / N

    # save png
    if (plot) {
      histo <- hist(ev.spacing, breaks = seq(min(ev.spacing), max(ev.spacing), len = 51), plot = FALSE)
      grDevices::png(paste0("RMT_temp/rmt_nnsd", i, ".png"), height = 600, width = 700, res = 130)
      nnsd_plot(
        histo = histo, title = "Eigenvalue spacing distribution (NNSD)", threshold = threshold,
        dis_GOE = log_LW, dis_possion = log_LE, p_ks_test = p_ks_test
      )
      grDevices::dev.off()
    }
    res <- rbind(res, data.frame(threshold, p_ks_test, log_sse, log_LW, log_LE))
  }
  if (plot) {
    message("The Intermediate files saved in ", out_dir, "/RMT_temp/.")
  } else {
    gif <- FALSE
  }
  # transfer to gif
  if (gif) {
    lib_ps("gifski", library = FALSE)
    gifski::gifski(paste0("RMT_temp/rmt_nnsd", seq_len(length(thres_seq)), ".png"),
      gif_file = "RMT_temp/rmt_nnsd.gif"
    )
  }
  r_threshold <- (res[which(res$log_LW == min(res$log_LW)), "threshold"] +
    res[which(res$log_LE == max(res$log_LE)), "threshold"]) / 2
  res <- list(res = res, r_threshold = r_threshold)
  class(res) <- c("rmt_res", class(res))
  return(res)
}

#' Plot a rmt_res
#'
#' @param x rmt_res
#' @param ... Additional arguments
#'
#' @return ggplot
#' @exportS3Method
#' @method plot rmt_res
plot.rmt_res <- function(x, ...) {
  threshold <- value <- variable <- xi <- y <- NULL
  res <- x$res
  linedf <- data.frame(
    variable = c("p_ks_test", "log_sse", "log_LW", "log_LE"),
    xi = c(
      res[which(res$p_ks_test == max(res$p_ks_test))[1], "threshold"],
      res[which(res$log_sse == min(res$log_sse))[1], "threshold"],
      res[which(res$log_LW == min(res$log_LW))[1], "threshold"],
      res[which(res$log_LE == max(res$log_LE))[1], "threshold"]
    ),
    x = max(res$threshold) - min(res$threshold),
    y = apply(res[, -1], 2, max)
  )

  reshape2::melt(res, "threshold") -> md

  # filter(threshold<0.77)%>%
  p <- ggplot(md, aes(threshold, value)) +
    geom_point(aes(col = variable)) +
    geom_line(aes(col = variable)) +
    scale_color_manual(values = get_cols(4, "col1")) +
    facet_wrap(. ~ variable, scales = "free_y") +
    theme_bw() +
    xlab(NULL) +
    geom_text(data = linedf, aes(x = xi - 0.02 * x, y = 0.5 * y, label = xi)) +
    geom_vline(data = linedf, aes(xintercept = xi), linetype = 2, col = "red") +
    theme(legend.position = "none")

  message(paste("recommend r_threshold: ", mean(linedf$xi)))
  return(p)
}

nnsd_plot <- \(histo = histo, title = title, threshold = threshold,
  dis_GOE = dis_GOE, dis_possion = dis_possion, p_ks_test = p_ks_test) {
  plot(histo, freq = FALSE, col = "#F4FCA1", main = title, font.main = 1, xlab = "eigenvalue spacing", ylab = "PDF of eigenvalue spacing")
  {
    actual.ymax <- par("yaxp")[2]
    x0 <- -log(actual.ymax * 0.98)
    possion_dis <- \(x)exp(-x)
    graphics::curve(possion_dis,
      from = max(x0, min(histo$breaks)),
      to = max(histo$breaks), n = 1001, add = TRUE, type = "l", lty = 1, col = "#EB34FF", lwd = 2
    )
  }
  {
    GOE <- function(x) pi / 2 * x * exp(-pi / 4 * x^2)
    graphics::curve(GOE,
      from = min(histo$breaks),
      to = max(histo$breaks), n = 1001, add = TRUE, type = "l",
      lty = 1, col = "blue", lwd = 2
    )
  }

  if ((!is.na(dis_GOE)) && (!is.na(dis_possion))) {
    graphics::mtext(side = 3, paste(
      "Distance to GOE =", signif(dis_GOE, 3),
      "\nDistance to Possion =", signif(dis_possion, 3), "; ks_test p.value for possion =", signif(p_ks_test, 3)
    ), col = "#878787", cex = 0.6)
  }

  if (!is.na(threshold)) graphics::mtext(side = 4, paste("threshold =", signif(threshold, 4)))

  graphics::legend("topright", inset = 0.05, c("Possion", "GOE"), col = c("#EB34FF", "blue"), lty = 1, lwd = 2, cex = 0.8)
}

trapez <- \(x, y){
  ind <- 2:length(x)
  as.double((x[ind] - x[ind - 1]) %*% (y[ind] + y[ind - 1])) / 2
}

get_sse <- \(ev.spacing){
  dens <- density(ev.spacing)
  N <- 20
  x <- seq(min(ev.spacing), max(ev.spacing), len = 1000)
  A <- exp(-min(ev.spacing)) - exp(-max(ev.spacing))
  xs <- numeric(N + 1)
  xs[1] <- min(ev.spacing)
  for (i in 1:N) xs[i + 1] <- -log(exp(-xs[i]) - A / N)
  area <- numeric(N)
  for (i in 1:N) {
    xsec <- x[(x > xs[i]) & (x < xs[i + 1])]
    xsec <- c(xs[i], xsec, xs[i + 1])
    ysec <- approx(dens$x, dens$y, xout = xsec)$y
    area[i] <- trapez(xsec, ysec)
  }
  sse <- sum((area[i] - A / N)^2)
  sse
}

#' Get RMT threshold for a correlation matrix roughly
#'
#' @export
#' @return recommend threshold
#' @rdname RMT_threshold
rmt <- function(occor.r, min_threshold = 0.5, max_threshold = 0.85, step = 0.01) {
  if (inherits(occor.r, "corr")) occor.r <- occor.r$r
  NNSD <- \(x)abs(diff(x))

  s <- seq(0, 3, 0.1)
  poisson_d <- exp(-s)
  nnsdpois <- density(NNSD(poisson_d))

  ps <- c()
  threshold <- c()

  for (i in seq(min_threshold, max_threshold, step)) {
    corr_r1 <- occor.r
    corr_r1[abs(corr_r1) < i] <- 0
    {
      eigen_res <- sort(eigen(corr_r1)$value)
      # spline to eigen_res
      check <- tryCatch(ssp <- smooth.spline(eigen_res, control.spar = list(low = 0, high = 3)),
        error = \(e) {
          TRUE
        }
      )
      if (rlang::is_true(check)) next
      nnsdw <- density(NNSD(ssp$y))
      chival <- sum((nnsdw$y - nnsdpois$y)^2 / nnsdpois$y / 1e3)
    }

    ps <- c(ps, chival)
    threshold <- c(threshold, i)
    if (((i * 100) %% 5 == 0)) {
      message(paste0("Calculating: ", i))
    }
  }

  res <- data.frame(threshold, ps)
  recommend_thres <- res[which.min(res[, 2]), 1]
  p <- ggplot(res, aes(threshold, ps)) +
    geom_point() +
    geom_vline(xintercept = recommend_thres, linetype = 2, col = "red") +
    geom_text(x = recommend_thres + 0.01, y = 0.5 * max(res$ps), label = recommend_thres) +
    theme_bw(base_size = 15)
  print(p)

  res1 <- res[(res$threshold < (recommend_thres + 0.05)) & (res$threshold > (recommend_thres - 0.05)), ]
  p <- ggplot(res1, aes(threshold, ps)) +
    geom_point() +
    geom_vline(xintercept = recommend_thres, linetype = 2, col = "red") +
    geom_text(x = recommend_thres + 0.01, y = 0.5 * max(res1$ps), label = recommend_thres) +
    theme_bw(base_size = 15)
  print(p)

  message("We recommend r-threshold: ", recommend_thres, ", you can calculate again in a smaller region")
  recommend_thres
}

# =======1.calculate========
#' Calculate correlation for one or two t(otutab), or distance for one t(otutab).
#'
#' @param totu t(otutab), row are samples, column are features.
#' @param totu2 t(otutab2) or NULL, row are samples, column are features.
#' @param method "spearman" (default), "pearson", "sparcc", or distance index from \code{\link[vegan]{vegdist}}.
#' @param filename the prefix of saved .corr file or FALSE.
#' @param p.adjust.method see \code{\link[stats]{p.adjust}}
#' @param p.adjust.mode see \code{\link{p.adjust.table}}
#' @param threads threads, default: 1.
#' @param verbose verbose, default: TRUE.
#'
#' @return a corr object with 3 elements:
#' \item{r}{default: spearman correlation}
#' \item{p.value}{default: p-value of spearman correlation}
#' \item{p.adjust}{default p.adjust.method = NULL}
#' @family calculate
#' @export
#' @aliases c_net_cal
#' @examples
#' data("otutab", package = "pcutils")
#' t(otutab) -> totu
#' c_net_calculate(totu) -> corr
#' metadata[, 3:10] -> env
#' c_net_calculate(totu, env) -> corr2
c_net_calculate <- function(totu, totu2 = NULL, method = "spearman", filename = FALSE,
                            p.adjust.method = NULL, p.adjust.mode = "all", threads = 1, verbose = TRUE) {
  if (!is.null(totu2)) {
    tls <- check_tabs(totu, totu2)
    totu <- tls[[1]]
    totu2 <- tls[[2]]
  } else {
    totu <- check_tabs(totu)
  }

  if (method %in% c("spearman", "pearson")) {
    corr <- fast_cor(totu, totu2, method = method)
  } else if (method == "sparcc") {
    if (!is.null(totu2)) warning("sparcc only take the totu, ignore the totu2.")
    message("sparcc is not supported on CRAN version. Please install `zdk123/SpiecEasi` from github and use the function `sparcc`.")
    # corr <- par_sparcc(totu, threads = threads, verbose = verbose)
    corr <- NULL
  } else if (method %in% c(
    "manhattan", "euclidean", "canberra", "bray",
    "kulczynski", "gower", "morisita", "horn", "mountford",
    "jaccard", "raup", "binomial", "chao", "altGower", "cao",
    "mahalanobis", "clark", "chisq", "chord", "hellinger",
    "aitchison", "robust.aitchison"
  )) {
    corr <- cal_sim(totu, totu2, method = method)
  } else {
    stop("method should be one of 'spearman', 'pearson', 'sparcc', or distance index from vegan.")
  }

  if (!is.null(p.adjust.method)) {
    p.adjust <- p.adjust.table(corr$p.value, p.adjust.method)
    res <- list(r = corr$r, p.value = corr$p.value, p.adjust = p.adjust)
  } else {
    res <- list(r = corr$r, p.value = corr$p.value)
  }

  class(res) <- "corr"
  attributes(res)$method <- method

  # save the correlation result
  if (is.logical(filename)) {
    if (filename) filename <- paste0("c_net_", date())
  }
  if (is.character(filename)) {
    save_corr(res, filename)
  }
  return(res)
}

#' Check tables and extract common samples
#'
#' @param ... tables
#' @return formatted tables
#' @export
#' @examples
#' data("otutab", package = "pcutils")
#' check_tabs(otutab)
check_tabs <- function(...) {
  tables <- list(...)
  if (all(class(tables[[1]]) == "list")) tables <- tables[[1]]

  names(tables) <- NULL

  if (length(tables) > 1) {
    comm <- Reduce(intersect, lapply(tables, rownames))
    if (length(comm) < 2) stop("There are ", length(comm), " common sample! Can not calculate correlation.")
    if (all(lapply(tables, \(i)identical(rownames(i), comm)) %>% unlist())) {
      message("All samples matched.")
    } else {
      message("Extract ", length(comm), " commmon samples.")
    }
    dup <- lapply(tables, colnames) %>% do.call(c, .)
    dup <- dup[duplicated(dup)]
    if (length(dup) > 0) {
      stop("Duplicated colnames found: ", paste0(dup, collapse = ", "), "\nPlease check colnames of input tables.")
    } else {
      message("All features are OK.")
    }
    tables <- lapply(tables, \(i)i[comm, , drop = FALSE])
  }

  if (length(tables) == 1) tables <- tables[[1]]
  return(tables)
}


#' Save a corr object
#'
#' @param corr a corr object
#' @param filename filename without extension, default: "corr"
#'
#' @return a .corr file
#' @export
#'
save_corr <- function(corr, filename = "corr") {
  stopifnot(inherits(corr, "corr"))
  if (!grepl("\\.corr$", filename)) filename <- paste0(filename, ".corr")
  if (t_flag(corr$r)) {
    # 节约一半的储存空间
    corr$r[upper.tri(corr$r)] -> r
    corr$p.value[upper.tri(corr$p.value)] -> p.value
    if (!is.null(corr$p.adjust)) {
      corr$p.adjust[upper.tri(corr$p.adjust)] -> p.adj
      if (all(p.value == p.adj)) {
        p.adj <- FALSE
      }
    } else {
      p.adj <- NULL
    }

    corr_names <- rownames(corr$r)
    saveRDS(
      list(
        r = r, p.value = p.value, p.adjust = p.adj,
        corr_names = corr_names, corr_attr = attributes(corr)
      ),
      file = filename
    )
  } else {
    saveRDS(corr, file = filename)
  }
}

#' Read a corr object
#'
#' @param filename filename of .corr
#'
#' @return a corr object
#' @export
#' @family calculate
read_corr <- function(filename) {
  # r <- read.csv(paste0(filename, "_r.csv"), row.names = 1, check.names = FALSE)
  # p.value <- read.csv(paste0(filename, "_p.csv"), row.names = 1, check.names = FALSE)
  # p.adjust <- read.csv(paste0(filename, "_p_adj.csv"), row.names = 1, check.names = FALSE)
  if (!grepl("\\.corr$", filename)) filename <- paste0(filename, ".corr")
  in_corr <- readRDS(filename)
  if ("corr_names" %in% names(in_corr)) {
    corr_names <- in_corr$corr_names
    r <- in_corr$r
    p.value <- in_corr$p.value
    p.adj <- in_corr$p.adj
    corr_attr <- in_corr$corr_attr

    new_p.adjust <- new_p <- new_r <- matrix(0, nrow = length(corr_names), ncol = length(corr_names), dimnames = list(corr_names, corr_names))
    new_r[upper.tri(new_r)] <- r
    new_r <- new_r + t(new_r)
    diag(new_r) <- 1
    new_p[upper.tri(new_p)] <- p.value
    new_p <- new_p + t(new_p)
    if (is.null(p.adj)) {
      new_p.adjust <- NULL
    } else if (is.logical(p.adj)) {
      new_p.adjust <- new_p
    } else {
      new_p.adjust[upper.tri(new_p.adjust)] <- p.adj
      new_p.adjust <- new_p.adjust + t(new_p.adjust)
      diag(new_p.adjust) <- 1
    }
    in_corr <- list()
    in_corr$r <- new_r
    in_corr$p.value <- new_p
    if (!is.null(new_p.adjust)) in_corr$p.adjust <- new_p.adjust
    attributes(in_corr) <- corr_attr
  }
  return(in_corr)
}

#' Plot a correlation heatmap
#'
#' @param corr a corr object, must contain 'r' and 'p.value' matrices.
#' @param show.p whether to show p-values as significance stars on the heatmap.
#' @param cluster_rows whether to cluster rows.
#' @param cluster_cols whether to cluster columns.
#' @param ... additional arguments passed to `pheatmap::pheatmap`.
#' @return plot of the correlation heatmap.
#' @export
plot_corr_heatmap <- function(corr,
                              show.p = FALSE,
                              cluster_rows = TRUE,
                              cluster_cols = TRUE,
                              ...) {
  # 检查必要包
  lib_ps("pheatmap", library = FALSE)

  # 检查输入结构
  if (!inherits(corr, "corr") ||
    !all(c("r", "p.value") %in% names(corr))) {
    stop("`corr` must be a corr object with 'r' and 'p.value' matrices.")
  }
  p.mat <- corr$p.value

  # 准备显著性标记（可选）
  if (show.p) {
    # 定义显著性符号
    signif_codes <- cut(
      p.mat,
      breaks = c(0, 0.001, 0.01, 0.05, 1),
      labels = c("***", "**", "*", ""),
      include.lowest = TRUE
    )

    # 将相关系数矩阵转换为带标记的字符矩阵
    r_with_stars <- matrix(
      # sprintf("%.2f%s", corr$r, signif_codes),
      signif_codes,
      nrow = nrow(corr$r),
      dimnames = dimnames(corr$r)
    )

    # 移除多余的+号（如0.10* -> 0.1*）
    r_with_stars <- gsub("0\\.(\\d)", ".\\1", r_with_stars)
  }

  # 绘制热图
  pheatmap::pheatmap(
    mat = corr$r,
    display_numbers = if (show.p) r_with_stars else FALSE,
    cluster_rows = cluster_rows,
    cluster_cols = cluster_cols,
    number_color = "black",
    border_color = if (nrow(corr$r) < 20) "gray60" else NA,
    fontsize_number = 10,
    ...
  )
}

#' Fast correlation calculation
#'
#' @param totu t(otutab), row are samples, column are features.
#' @param totu2 t(otutab) or NULL, row are samples, column are features.
#' @param method "spearman" or "pearson"
#'
#' @export
#'
#' @return a list with 2 elements:
#' \item{r}{default: spearman correlation}
#' \item{p.value}{default: p-value of spearman correlation}
#' @family calculate
#' @examples
#' data("otutab", package = "pcutils")
#' t(otutab[1:100, ]) -> totu
#' fast_cor(totu, method = "spearman") -> corr
fast_cor <- function(totu, totu2 = NULL, method = c("pearson", "spearman")) {
  method <- match.arg(method, c("pearson", "spearman"))
  totu <- as.matrix(totu)
  if (!is.null(totu2)) {
    totu2 <- as.matrix(totu2)
    r <- stats::cor(totu, totu2, method = method)
    df <- dim(totu)[1] - 2
    t <- r * sqrt(df / (1 - r^2))
    p <- -2 * expm1(stats::pt(abs(t), df, log.p = TRUE))
  } else {
    r <- stats::cor(totu, method = method)
    p <- r
    p[p != 0] <- 0
    r_tri <- r[upper.tri(r)]
    df <- dim(totu)[1] - 2
    t <- r_tri * sqrt(df / (1 - r_tri^2))
    p[upper.tri(p)] <- -2 * expm1(stats::pt(abs(t), df, log.p = TRUE))
    p <- p + t(p)
  }
  return(list(r = r, p.value = p))
}

Hmisc_cor <- function(totu, totu2 = NULL, method = c("spearman", "pearson")[1]) {
  lib_ps("Hmisc", library = FALSE)
  totu <- as.matrix(totu)
  if (!is.null(totu2)) {
    totu2 <- as.matrix(totu2)
    tmp <- Hmisc::rcorr(totu, totu2, type = method)
    r <- tmp$r[1:ncol(totu), (ncol(totu) + 1):ncol(tmp$r)]
    p <- tmp$P[1:ncol(totu), (ncol(totu) + 1):ncol(tmp$r)]
    return(list(r = r, p.value = p))
  }
  tmp <- Hmisc::rcorr(totu, type = method)
  p <- tmp$P
  p[is.na(p)] <- 0
  return(list(r = tmp$r, p.value = p))
}

#' Calculate similarity for one t(otutab)
#'
#' @param totu t(otutab), row are samples, column are features.
#' @param method Dissimilarity index, see \code{\link[vegan]{vegdist}}.
#' @param totu2 t(otutab) or NULL, row are samples, column are features.
#'
#' @family calculate
#' @return similarity = 1-distance
#' @export
#' @seealso \code{\link[vegan]{vegdist}}
#' @examples
#' if (requireNamespace("vegan")) {
#'   data("otutab", package = "pcutils")
#'   t(otutab) -> totu
#'   cal_sim(totu) -> sim_corr
#' }
cal_sim <- function(totu, totu2 = NULL, method = "bray") {
  lib_ps("vegan", library = FALSE)
  if (is.null(totu2)) {
    vegan::vegdist(t(totu), method = method) %>% as.matrix() -> dist
  } else {
    n1 <- ncol(totu)
    n2 <- ncol(totu2)
    dist <- matrix(NA, nrow = n1, ncol = n2, dimnames = list(colnames(totu), colnames(totu2)))
    for (i in seq_len(n1)) {
      for (j in seq_len(n2)) {
        dist[i, j] <- vegan::vegdist(rbind(totu[, i], totu2[, j]), method = method)
      }
    }
  }

  sim <- 1 - dist
  p <- dist
  message("p-value is not supported for distance index, all set as 0.")
  p[p != 0] <- 0

  return(list(r = sim, p.value = p))
}

cal_KLD <- function(totu, totu2 = NULL, method = "KLD") {
  # Kullback-Leibler divergence
  # KLD = function(p, q) {
  #   sum(p * log(p / q))
  # }
  # p = c(0.1, 0.2, 0.3, 0.4)
  # q = c(0.2, 0.3, 0.2, 0.3)
  # KLD(p, q)
  lib_ps("philentropy", library = FALSE)
  dat <- t(totu) / rowSums(t(totu))
  philentropy::KL(dat, unit = "log")
}

#' p.adjust apply on a correlation table (matrix or data.frame)
#'
#' @param pp table of p-values
#' @param method see \code{\link[stats]{p.adjust}}, default: "BH".
#' @param mode "all" for all values; "rows" adjust each row one by one; "columns" adjust each column one by one. Default: "all".
#'
#' @return a table of adjusted p-values
#' @export
#'
#' @family calculate
#' @examples
#' matrix(abs(rnorm(100, 0.01, 0.1)), 10, 10) -> pp
#' p.adjust.table(pp, method = "BH", mode = "all") -> pp_adj
p.adjust.table <- \(pp, method = "BH", mode = "all"){
  mode <- match.arg(mode, c("all", "rows", "columns"))
  pp <- as.matrix(pp)
  if (mode == "all") {
    if (t_flag(pp)) {
      lp <- lower.tri(pp)
      pa <- pp[lp]
      pa <- p.adjust(pa, method)
      pp[lower.tri(pp, diag = FALSE)] <- pa
      pp[upper.tri(pp, diag = FALSE)] <- 0
      pp <- pp + t(pp)
    } else {
      pp1 <- p.adjust(pp, method)
      pp <- matrix(pp1, nrow(pp), ncol(pp), dimnames = list(rownames(pp), colnames(pp)))
    }
  } else if (mode == "rows") {
    pp <- t(apply(pp, 1, p.adjust, method = method))
  } else if (mode == "columns") {
    pp <- apply(t(pp), 1, p.adjust, method = method)
  }
  return(pp)
}

#' Extract ego-centric subnetwork with preserved class attributes
#'
#' Wrapper around \code{igraph::make_ego_graph()} that ensures output retains
#' "metanet" and "igraph" class structure. Supports single or multiple center nodes.
#'
#' @param graph An igraph object with potential "metanet" class
#' @param nodes Center node(s) for subnetwork extraction (vertex IDs or names)
#' @param order Integer specifying the order of neighbors to include
#' @param mode Character scalar, either "in", "out" or "all" for directed networks
#'
#' @return metanet
#' @aliases c_net_neighbors
#'
#' @examples
#' library(igraph)
#' c_net_plot(co_net)
#' c_net_plot(c_net_ego(co_net, "s__Kribbella_catacumbae"))
#' nodes <- c("s__Kribbella_catacumbae", "s__Verrucosispora_andamanensis")
#' c_net_plot(c_net_ego(co_net, nodes))
#' @export
c_net_ego <- function(graph, nodes, order = 1, mode = "all") {
  # Input validation
  if (!inherits(graph, "igraph")) {
    stop("Input must be an igraph object")
  }

  # Extract ego networks
  ego_graphs <- make_ego_graph(
    graph = graph,
    order = order,
    nodes = nodes,
    mode = mode
  )

  # Merge if multiple centers
  if (length(nodes) > 1) {
    subg <- suppressMessages(Reduce(c_net_union, ego_graphs))
  } else {
    subg <- ego_graphs[[1]]
  }

  # Force class structure
  class(subg) <- c("metanet", "igraph")

  return(subg)
}


#' Highlight specific nodes in a network
#'
#' Adds highlight markers to specified nodes and grays out non-highlighted nodes.
#' Preserves all existing vertex/edge attributes and class structure.
#'
#' @param graph An igraph/metatnet object
#' @param nodes Vector of node names to highlight
#' @param gray_color Color for non-highlighted nodes (default: "gray80")
#' @param edges a data.frame of edges to highlight, colnames must be "from" and "to"
#'
#' @return metanet
#'
#' @examples
#' par(mfrow = c(1, 3))
#' nodes <- c("s__Kribbella_catacumbae", "s__Verrucosispora_andamanensis")
#' nodes <- V(c_net_ego(co_net, nodes))$name
#' g_hl <- c_net_highlight(co_net, nodes = nodes)
#' plot(g_hl) # Highlighted nodes keep colors, others turn gray
#' get_e(co_net) %>% head(20) -> hl_edges
#' g_hl2 <- c_net_highlight(co_net, edges = hl_edges[, 2:3])
#' c_net_plot(g_hl2)
#' g_hl3 <- c_net_highlight(co_net, nodes = nodes, edges = hl_edges[, 2:3])
#' c_net_plot(g_hl3)
#' @export
c_net_highlight <- function(graph, nodes = NULL, edges = NULL, gray_color = "gray80") {
  # 验证输入
  if (!inherits(graph, "igraph")) stop("Input must be an igraph object")
  if (!is_metanet(graph)) graph <- as.metanet(graph)
  # 备份原始颜色（如果不存在则创建）
  if ("color_original" %in% vertex_attr_names(graph)) {
    V(graph)$color <- V(graph)$color_original
  } else {
    V(graph)$color_original <- V(graph)$color
  }
  if ("color_original" %in% edge_attr_names(graph)) {
    E(graph)$color <- E(graph)$color_original
  } else {
    E(graph)$color_original <- E(graph)$color
  }

  E(graph)$v_highlight_from <- E(graph)$v_highlight_to <- V(graph)$v_highlight <- FALSE

  graph2 <- graph
  if (!is.null(nodes)) {
    # 转换节点标识为统一格式
    nodes <- which(V(graph2)$name %in% nodes)
    if (length(nodes) == 0) {
      message("No nodes found in the network.")
    }
    # 添加高亮标记
    V(graph2)$v_highlight[nodes] <- TRUE
    # 处理相应边
    graph2 <- suppressMessages(anno_edge(graph2, get_v(graph2)[, c("name", "v_highlight")]))
  }

  graph3 <- graph
  if (!is.null(edges)) {
    stopifnot(is.data.frame(edges))
    stopifnot(identical(c("from", "to"), colnames(edges)))
    if (!igraph::is_directed(graph3)) {
      edges2 <- edges
      edges2$from <- edges$to
      edges2$to <- edges$from
      edges <- rbind(edges, edges2)
    }
    edges$v_highlight_from <- TRUE
    edges$v_highlight_to <- TRUE
    graph3 <- suppressMessages(anno_edge(graph3, edges))
    E(graph3)$v_highlight_from[is.na(E(graph3)$v_highlight_from)] <- FALSE
    E(graph3)$v_highlight_to[is.na(E(graph3)$v_highlight_to)] <- FALSE

    nodes <- c(E(graph3)$from[E(graph3)$v_highlight_from], E(graph3)$to[E(graph3)$v_highlight_to])
    nodes <- which(V(graph3)$name %in% nodes)
    V(graph3)$v_highlight[nodes] <- TRUE
  }

  V(graph)$v_highlight <- V(graph2)$v_highlight | V(graph3)$v_highlight
  E(graph)$v_highlight_from <- E(graph2)$v_highlight_from | E(graph3)$v_highlight_from
  E(graph)$v_highlight_to <- E(graph2)$v_highlight_to | E(graph3)$v_highlight_to

  # 修改非高亮节点颜色
  V(graph)$color[!V(graph)$v_highlight] <- gray_color
  E(graph)$color[!(E(graph)$v_highlight_from & E(graph)$v_highlight_to)] <- gray_color

  return(graph)
}


#' Batch drawing multiple network diagrams
#'
#' @param graph_ls a list containing igraph objects
#' @param nrow nrow
#' @param ncol ncol
#' @param multi_params_list a list of parameters for each network
#'
#' @return No value
#'
#' @examples
#' plot_multi_nets(list(co_net, co_net2),
#'   multi_params_list = list(
#'     list(vertex.color = "skyblue"),
#'     list(vertex.color = "green3")
#'   )
#' )
#' @export
plot_multi_nets <- function(graph_ls,
                            nrow = NULL,
                            ncol = NULL,
                            multi_params_list = NULL) {
  # 参数验证
  if (!all(sapply(graph_ls, inherits, "igraph"))) {
    stop("All elements in graph_ls must be igraph objects")
  }

  # 计算布局行列数
  n <- length(graph_ls)
  if (is.null(nrow) && is.null(ncol)) {
    nrow <- floor(sqrt(n))
    ncol <- ceiling(n / nrow)
  } else if (is.null(nrow)) {
    nrow <- ceiling(n / ncol)
  } else if (is.null(ncol)) {
    ncol <- ceiling(n / nrow)
  }

  # 保存原始图形参数
  old_par <- par(no.readonly = TRUE)
  on.exit(par(old_par))

  # 设置多图布局
  par(mfrow = c(nrow, ncol), mar = c(1, 1, 2, 1))

  if (length(multi_params_list) == 0) multi_params_list <- vector("list", n)
  if (length(multi_params_list) != n) multi_params_list <- rep(multi_params_list, n)

  # 批量绘图
  for (i in seq_along(graph_ls)) {
    go <- graph_ls[[i]] # 注意这里只能用go，属于是参数环境问题
    c_net_plot(go, params_list = multi_params_list[[i]])
  }
}

#' Calculate Similarity Between Two Graphs via Adjacency Matrices
#'
#' Computes the similarity between two igraph objects using their adjacency matrices.
#' Supports Frobenius norm-based similarity and cosine similarity.
#'
#' @param g1 An igraph object representing the first graph.
#' @param g2 An igraph object representing the second graph.
#' @param method A character string specifying the similarity method:
#'               "frobenius" (default) or "cosine".
#' @return A numeric value between 0 (no similarity) and 1 (identical graphs).
#' @export
#' @examples
#' library(igraph)
#' g1 <- graph_from_edgelist(matrix(c(1, 2, 2, 3), ncol = 2, byrow = TRUE), directed = FALSE)
#' g2 <- graph_from_edgelist(matrix(c(1, 2, 2, 4), ncol = 2, byrow = TRUE), directed = FALSE)
#' adjacency_similarity(g1, g2, method = "frobenius") # Output: 0.5
#' adjacency_similarity(g1, g2, method = "cosine") # Output: 0.5
adjacency_similarity <- function(g1, g2, method = "frobenius") {
  if (!is_metanet(g1)) g1 <- as.metanet(g1)
  if (!is_metanet(g2)) g2 <- as.metanet(g2)
  # 获取邻接矩阵
  adj1 <- as.matrix(igraph::as_adjacency_matrix(g1))
  adj2 <- as.matrix(igraph::as_adjacency_matrix(g2))

  # 统一节点集合
  all_nodes <- union(rownames(adj1), rownames(adj2))

  # 初始化全零矩阵
  adj1_fixed <- matrix(0,
    nrow = length(all_nodes), ncol = length(all_nodes),
    dimnames = list(all_nodes, all_nodes)
  )
  adj2_fixed <- matrix(0,
    nrow = length(all_nodes), ncol = length(all_nodes),
    dimnames = list(all_nodes, all_nodes)
  )

  # 填充已知边
  adj1_fixed[rownames(adj1), colnames(adj1)] <- adj1
  adj2_fixed[rownames(adj2), colnames(adj2)] <- adj2

  # 计算相似性
  if (method == "frobenius") {
    diff_norm <- norm(adj1_fixed - adj2_fixed, "F")
    max_norm <- sqrt(nrow(adj1_fixed) * ncol(adj2_fixed))
    similarity <- 1 - diff_norm / max_norm
  } else if (method == "cosine") {
    similarity <- sum(adj1_fixed * adj2_fixed) /
      (norm(adj1_fixed, "F") * norm(adj2_fixed, "F"))
  } else {
    stop("Method must be 'frobenius' or 'cosine'.")
  }

  return(similarity)
}

#' Compare Two Networks
#'
#' @param g1 network1
#' @param g2 network2
#'
#' @returns A list containing the following elements:
#' - `g1`: The first network.
#' - `g2`: The second network.
#' - `g_union`: The union of the two networks.
#' - `g_inter`: The intersection of the two networks.
#' - `net_par_df`: A data frame containing the network parameters.
#' - `net_similarity`: A list containing the similarity metrics.
#'
#' @export
#'
#' @examples
#' data("c_net")
#' set.seed(12)
#' co_net_p1 <- c_net_filter(co_net, name %in% sample(V(co_net)$name, 300))
#' co_net_p2 <- c_net_filter(co_net, name %in% sample(V(co_net)$name, 300))
#' c_net_compare(co_net_p1, co_net_p2) -> c_net_comp
#' plot(c_net_comp)
c_net_compare <- function(g1, g2) {
  g_union <- suppressMessages(c_net_union(g1, g2))
  g_inter <- suppressMessages(c_net_intersect(g1, g2))

  lapply(list(g1, g2, g_union, g_inter), function(g) {
    net_par(g, mode = "n", only_topological = TRUE)$n_index
  }) %>%
    do.call(rbind, .) %>%
    pcutils::t2() -> net_par_df
  colnames(net_par_df) <- c("g1", "g2", "g_union", "g_inter")

  # 计算相似性
  net_similarity <- c(net_par_df[c("Node_number", "Edge_number"), 4] / net_par_df[c("Node_number", "Edge_number"), 3], adjacency_similarity(g1, g2))
  names(net_similarity) <- c("node_jaccard", "edge_jaccard", "adjacency_similarity")

  res <- list(
    g1 = g1, g2 = g2, g_union = g_union, g_inter = g_inter,
    net_par_df = net_par_df, net_similarity = net_similarity
  )
  class(res) <- c("metanet_compare")
  res
}

#' Plot a metanet_compare
#'
#' @param x metanet_compare object
#' @param ... add
#' @param coors_com coors object
#' @param mains a vector of two strings for the main titles of the two networks
#'
#' @return plot
#' @exportS3Method
#' @method plot metanet_compare
plot.metanet_compare <- function(x, coors_com = NULL, mains = NULL, ...) {
  c_net_comp <- x
  if (is.null(coors_com)) {
    coors_com <- c_net_layout(c_net_comp$g_union)
  }
  if (is.null(mains)) {
    mains <- c("g1", "g2")
  }

  c_net_highlight(c_net_comp$g1, V(c_net_comp$g_inter)$name) -> c_net_comp_g1
  c_net_highlight(c_net_comp$g2, V(c_net_comp$g_inter)$name) -> c_net_comp_g2
  plot_multi_nets(list(c_net_comp_g1, c_net_comp_g2),
    nrow = 1, ncol = 2,
    multi_params_list = list(
      list(coors = coors_com, main = mains[1], legend = FALSE),
      list(coors = coors_com, main = mains[2], legend = FALSE)
    )
  )
}

# ========6.modules=========

#' Generate a n-modules network
#'
#' @param module_number number of modules
#' @param n_node_in_module number of nodes in each modules
#' @param intra_module_density intra_module_density, recommend bigger than 20*inter_module_density, default:0.3
#' @param inter_module_density inter_module_density, default:0.01
#'
#' @description this is just a random generation method, the module number of result is not exactly the module_number, you can change the inter_module_density and intra_module_density to get the proper result.
#' @return n-modules metanet
#' @export
#'
#' @family module
#' @examples
#' g1 <- module_net()
#' get_n(g1)
#' plot(g1, mark_module = TRUE)
#' plot(g1, coors = g_layout(g1, zoom2 = 15))
#' plot(g1, coors = g_layout_polyarc(g1, group = "module"))
#' plot(g1, coors = g_layout_polygon(g1, group = "module"))
module_net <- function(module_number = 3, n_node_in_module = 30,
                       intra_module_density = 0.3,
                       inter_module_density = 0.01) {
  n_node_in_module <- rep(n_node_in_module, length = module_number)
  mat <- matrix(0, nrow = sum(n_node_in_module), ncol = sum(n_node_in_module))
  start <- c(1, cumsum(n_node_in_module[-length(n_node_in_module)]) + 1)
  end <- cumsum(n_node_in_module)
  seqls <- lapply(1:module_number, \(i)(start[i]:end[i]))
  # generate intra_module_edges
  for (i in seq_len(module_number)) {
    seq <- seqls[[i]]
    mat[seq, seq] <- as.matrix(igraph::get.adjacency(igraph::erdos.renyi.game(n_node_in_module[i], intra_module_density)))
  }
  # generate inter_module_edges
  idls <- combn(1:module_number, 2) %>% split(., col(.))
  for (ids in idls) {
    seq1 <- seqls[[ids[1]]]
    seq2 <- seqls[[ids[2]]]
    mat[seq1, seq2] <- sample(c(1, 0), length(seq1) * length(seq2),
      replace = TRUE,
      prob = c(inter_module_density, 1 - inter_module_density)
    )
  }

  g <- igraph::graph.adjacency(mat, mode = "undirected")
  # plot(g)
  c_net_update(g, initialize = TRUE, verbose = FALSE) -> g1
  g1 <- module_detect(g1)
  g1 <- to_module_net(g1)
  g1
}

#' Detect the modules
#'
#' @param go an igraph object
#' @param method cluster_method: "cluster_walktrap", "cluster_edge_betweenness", "cluster_fast_greedy", "cluster_spinglass"
#' @param n_node_in_module transfer the modules less than n_node_in_module to "others"
#' @param delete logical, delete others modules? default:FALSE, the others module will be "others".
#'
#' @return an igraph object
#' @export
#' @aliases c_net_module
#' @family module
#' @examples
#' data("c_net")
#' module_detect(co_net) -> co_net_modu
module_detect <- function(go, method = "cluster_fast_greedy", n_node_in_module = 0, delete = FALSE) {
  stopifnot(is_igraph(go))
  if (!is_metanet(go)) go <- c_net_update(go, initialize = TRUE, verbose = FALSE)

  if ("original_module" %in% vertex_attr_names(go)) message("'module' already exsited, start a new module detection!")
  ms <- c("cluster_walktrap", "cluster_edge_betweenness", "cluster_fast_greedy", "cluster_spinglass")
  method <- match.arg(method, ms)
  if ("weight" %in% edge_attr_names(go)) E(go)$weight <- abs(igraph::E(go)$weight)

  switch(method,
    "cluster_walktrap" = {
      wc <- igraph::cluster_walktrap(go, weights = NULL)
    },
    "cluster_edge_betweenness" = {
      wc <- igraph::cluster_edge_betweenness(go, weights = NULL)
    },
    "cluster_fast_greedy" = {
      wc <- igraph::cluster_fast_greedy(go, weights = NULL)
    },
    "cluster_spinglass" = {
      wc <- igraph::cluster_spinglass(go, weights = NULL)
    }
  )

  # add components
  V(go)$components <- igraph::components(go)$membership %>% as.character()
  V(go)$module <- igraph::membership(wc) %>% as.character()
  V(go)$original_module <- V(go)$module

  if (n_node_in_module > 0) {
    go <- filter_n_module(go, n_node_in_module = n_node_in_module, delete = delete)
    go <- c_net_update(go, initialize = TRUE, verbose = FALSE)
  }

  graph_attr(go)$communities <- wc
  graph_attr(go)$modularity <- modularity(wc)
  rand.g <- erdos.renyi.game(length(V(go)), length(E(go)), type = "gnm")
  rand_m <- modularity(cluster_fast_greedy(rand.g))
  relative_modularity <- (modularity(wc) - rand_m) / rand_m # relative modularity
  graph_attr(go)$relative_modularity <- relative_modularity

  return(go)
}

as_module <- \(x){
  x <- list(membership = x)
  vids <- names(x$membership)
  modus <- tapply(vids, x$membership, simplify = FALSE, function(x) x)
  return(modus)
}

#' Filter some modules as others
#'
#' @param go_m metanet with modules
#' @param n_node_in_module transfer the modules less than n_node_in_module to "others"
#' @param keep_id keep modules ids, will not be "others"
#' @param delete logical, delete others modules? default:FALSE, the others module will be "others".
#' @family module
#' @return metanet with modules
#' @export
#' @examples
#' data("c_net")
#' module_detect(co_net) -> co_net_modu
#' filter_n_module(co_net_modu, n_node_in_module = 30) -> co_net_modu
#' if (requireNamespace("ggtree") && requireNamespace("treeio")) plot_module_tree(co_net_modu)
#' combine_n_module(co_net_modu, 20) -> co_net_modu1
#' if (requireNamespace("ggtree") && requireNamespace("treeio")) plot_module_tree(co_net_modu1)
filter_n_module <- function(go_m, n_node_in_module = 0, keep_id = NULL, delete = FALSE) {
  module <- NULL
  if (!"original_module" %in% vertex_attr_names(go_m)) stop("'module' do not exsited, please do a `module_detect` first!")
  members <- V(go_m)$original_module
  table(members) %>% sort(decreasing = TRUE) -> s_members

  # filter modules whose nodes bigger than n_node_in_module
  keep_id1 <- names(s_members[s_members >= n_node_in_module])
  keep_id <- base::union(as.character(keep_id), as.character(keep_id1))
  new_module <- ifelse(members %in% keep_id, members, "others")

  V(go_m)$module <- new_module
  if (delete) go_m <- c_net_filter(go_m, module != "others")
  return(go_m)
}

#' Combine or cut modules to module_number
#'
#' @param module_number number of modules
#' @export
#' @rdname filter_n_module
combine_n_module <- function(go_m, module_number = 5) {
  get_community(go_m) -> comm
  igraph::cut_at(comm, module_number) -> new_modu
  V(go_m)$module <- as.character(new_modu)
  graph.attributes(go_m)$communities$membership <- as.numeric(new_modu)
  go_m
}

#' Transformation a network to a module network
#'
#' @param edge_type "module", "module_from", "module_to"
#' @param go metanet
#'
#' @export
#' @family module
#' @return metanet with modules
to_module_net <- function(go, edge_type = c("module", "module_from", "module_to")[1]) {
  if (!"module" %in% vertex_attr_names(go)) stop("no 'module', please `module_detect()` first or set the V(net)$module.")
  edge_type <- match.arg(edge_type, c("module", "module_from", "module_to"))
  go <- anno_edge(go, get_v(go)[, c("name", "module")], verbose = FALSE)

  tmp_e <- igraph::edge.attributes(go)
  if (edge_type == "module") {
    E(go)$e_type <- ifelse(tmp_e$module_from == tmp_e$module_to, "intra-module", "inter-module")
  } else if (edge_type == "module_from") {
    E(go)$e_type <- tmp_e$module_from
  } else if (edge_type == "module_to") {
    E(go)$e_type <- tmp_e$module_to
  }

  # 刷新颜色
  # go=delete_edge_attr(go,"color")
  V(go)$v_class <- V(go)$module
  go <- c_net_update(go, initialize = TRUE, verbose = FALSE)
  V(go)$color <- ifelse(V(go)$module == "others", "grey", V(go)$color)

  n_mod <- unique(V(go)$module)
  igraph::graph.attributes(go)$n_type <- "module"
  igraph::graph.attributes(go)$n_modules <- length(n_mod[n_mod != "others"])
  go
}

#' Get community
#' @param go_m module metanet
#'
#' @export
#' @family module
#' @return community
get_community <- function(go_m) {
  if (is.null(igraph::graph_attr(go_m)$communities)) stop("No community find, please do module_net() first.")
  igraph::graph_attr(go_m)$communities
}

#' Get module
#' @param go_m module metanet
#'
#' @export
#' @family module
#' @return module
get_module <- function(go_m) {
  if (!"module" %in% vertex_attr_names(go_m)) stop("no modules, please `module_detect()` first")
  setNames(V(go_m)$module, V(go_m)$name)
}

#' Get module_eigen
#' @param go_m module metanet
#'
#' @export
#' @family module
#' @return module_eigen
get_module_eigen <- function(go_m) {
  graph_attr(go_m, "module_eigen")
}

#' Summary module index
#' @param go_m module metanet
#' @param var variable name
#' @param module which column name is module. default: "module"
#' @param ... add
#' @family module
#' @export
#' @return ggplot
#' @examples
#' data("c_net")
#' module_detect(co_net, n_node_in_module = 30) -> co_net_modu
#' summary_module(co_net_modu, var = "v_class", module = "module")
#' summary_module(co_net_modu, var = "Abundance", module = "module")
summary_module <- function(go_m, var = "v_class", module = "module", ...) {
  tmp_v <- get_v(go_m)
  if ((length(module) > 1) || (length(var) > 1)) stop("var or module should be one column!")
  a <- tmp_v %>% dplyr::select(!!module, !!var)
  colnames(a)[1] <- "module"

  i <- var
  if (is.numeric(a[, i])) {
    pcutils::group_box(a[i], group = "module", metadata = a, ...)
  } else {
    table(a[, i], a$module) %>%
      as.data.frame() %>%
      reshape2::acast(Var1 ~ Var2, value.var = "Freq") %>%
      as.data.frame() -> tab
    pcutils::stackplot(tab, legend_title = var, ...) + labs(x = "Module")
  }
}

#' Plot module tree
#' @param go_m module metanet
#' @param module which column name is module. default: "module"
#' @param community community object, default: NULL, use the community of go_m
#' @param label.size label.size
#' @return ggplot
#'
#' @export
#' @rdname filter_n_module
plot_module_tree <- function(go_m, module = "module", community = NULL, label.size = 2) {
  tmp_v <- get_v(go_m)
  mdata <- tmp_v[, c("name", module)]

  lib_ps("ggtree", "treeio", library = FALSE)
  # modules tree
  if (is.null(community)) {
    get_community(go_m) %>% treeio::as.phylo() -> mcl
  } else {
    community %>% treeio::as.phylo() -> mcl
  }
  mcl <- dplyr::left_join(mcl, mdata, by = c("label" = "name"))
  p <- ggtree::ggtree(mcl, linewidth = 0.3) +
    ggtree::geom_tiplab(aes(color = module), show.legend = FALSE, size = label.size) +
    scale_color_manual(values = pcutils::get_cols(length(unique(mdata$module)), "col3"))

  ggtree::gheatmap(p, mdata %>% tibble::column_to_rownames("name")) +
    scale_fill_manual(values = pcutils::get_cols(length(unique(mdata$module)), "col3"), name = NULL)
}

#' Calculate the eigenvalue of each module and correlation of nodes and eigenvalue (node_eigen_cor).
#'
#' @param go_m module metanet
#' @param totu original abundance table
#' @param cor_method "pearson", "kendall", "spearman"
#'
#' @export
#' @return module metanet with module_eigen
#' @rdname module_expression
module_eigen <- function(go_m, totu, cor_method = "spearman") {
  modules <- get_module(go_m)
  totu <- totu[, names(modules)]

  res <- lapply(levels(factor(modules)), \(i){
    if (i == "others") {
      return(NULL)
    }
    totu1 <- totu[, modules == i]
    # PCA
    # pc <- prcomp(totu1)
    # pc$x[, 1]
    # cor(pc$x[, 1],rowMeans(totu1))

    # SVD
    totu_scale <- scale(totu1)
    svd <- svd(totu_scale)
    if (cor(svd$u[, 1], rowMeans(totu_scale)) < 0) {
      return(-svd$u[, 1])
    } else {
      return(svd$u[, 1])
    }
  })
  names(res) <- levels(factor(modules))
  # simplify method
  eigen_res <- do.call(cbind, res) %>% data.frame(., check.names = FALSE)
  # colnames(eigen_res)=paste0("ME_",colnames(eigen_res))
  rownames(eigen_res) <- rownames(totu)

  lapply(colnames(eigen_res), \(i)cor(totu[, modules == i], eigen_res[, i], method = cor_method)) %>%
    do.call(rbind, .) %>%
    as.data.frame() -> node_eigen_cor
  colnames(node_eigen_cor) <- "node_eigen_cor"
  go_m <- anno_vertex(go_m, node_eigen_cor, verbose = FALSE)
  graph.attributes(go_m)$module_eigen <- eigen_res
  go_m
}

#' Plot the expression of each modules
#'
#' @param go_m module metanet
#' @param totu original abundance table used for module_eigen().
#' @param group group variable for totu
#' @param r_threshold the threshold for node_eigen_cor, default: 0.6.
#' @param x_order order the x axis.
#' @param facet_param parameters parse to \code{\link[ggplot2]{facet_wrap}}, e.g. nrow=2.
#' @param plot_eigen plot the eigen value line.
#' @family module
#' @export
#'
#' @examples
#' data("otutab", package = "pcutils")
#' t(otutab) -> totu
#' data("c_net")
#' module_detect(co_net, n_node_in_module = 30) -> co_net_modu
#' module_eigen(co_net_modu, totu) -> co_net_modu
#' module_expression(co_net_modu, totu)
module_expression <- function(go_m, totu, group = NULL, r_threshold = 0.6,
                              x_order = NULL, facet_param = NULL, plot_eigen = FALSE) {
  node_eigen_cor <- variable <- value <- name <- module <- rowname <- NULL
  if (is.null(graph_attr(go_m, "module_eigen"))) stop("Please do module_eigen() first")

  graph_attr(go_m, "module_eigen") %>%
    rownames_to_column() %>%
    reshape2::melt(id.vars = "rowname", variable.name = "module") -> module_eigen

  get_v(go_m) -> tmp_v

  if (!is.null(group)) totu <- pcutils::hebing(totu, group, 1)

  totu_scale <- scale(totu)
  pdat <- cbind(tmp_v[, c("name", "module", "node_eigen_cor")], t(totu_scale[, tmp_v$name]))
  pdat <- dplyr::filter(pdat, node_eigen_cor > r_threshold)
  pdat_m <- reshape2::melt(pdat, id.vars = c("name", "module", "node_eigen_cor"))
  if (!is.null(x_order)) pdat_m$variable <- pcutils::change_fac_lev(pdat_m$variable, level = x_order)
  pdat_m$module <- factor(pdat_m$module)

  p1 <- ggplot() +
    geom_line(data = pdat_m, aes(
      x = variable, y = value, group = name, color = module,
      size = node_eigen_cor, alpha = node_eigen_cor
    ), size = 0.8) +
    MetaNet_theme +
    theme(plot.margin = unit(c(1, 2, 1, 1), "lines")) +
    do.call(facet_wrap, append(list(facets = ~module), pcutils::update_param(list(scales = "free_y", ncol = 2), facet_param))) +
    labs(x = NULL, y = NULL) +
    # facet_wrap(facets = ~module,nrow = nrow,scales = "free_y")+
    scale_alpha_continuous(range = c(0, 0.6)) +
    scale_x_discrete(expand = c(0, 0)) +
    scale_color_manual(values = get_cols(nlevels(pdat_m$module), default_v_color))

  if (plot_eigen) p1 <- p1 + geom_line(data = module_eigen, aes(x = rowname, y = value, col = module, group = module), size = 2, alpha = 1)

  return(p1)
}

#' Zi-Pi calculate
#'
#' @param go_m igraph object after `module_detect()`
#' @param mode use 7-group (mode=1) or 4-group (mode=2), default: mode=2
#' @param use_origin use original_module, default:TRUE, if FALSE, use module
#'
#' @return igraph
#' @export
#' @family module
#' @references 1. Guimerà, R. & Amaral, L. Functional cartography of complex metabolic networks. (2005) doi:10.1038/nature03288.
#' @examples
#' data("c_net")
#' module_detect(co_net) -> co_net_modu
#' zp_analyse(co_net_modu) -> co_net_modu
#' zp_plot(co_net_modu)
#' zp_plot(co_net_modu, mode = 3)
#'
zp_analyse <- function(go_m, mode = 2, use_origin = TRUE) {
  go_m -> go1
  v_index <- get_v(go_m)
  if (!"module" %in% names(v_index)) stop("no modules, please `module_detect()` first")
  if ("roles" %in% names(v_index)) message("areadly has roles, overwrite!")

  if (!"original_module" %in% names(v_index)) {
    if (use_origin) {
      use_origin <- FALSE
    }
  }
  # use original_module to do zp_analyse
  if (use_origin) {
    {
      go1 <- anno_vertex(go1, data.frame(row.names = V(go1)$name, module = V(go1)$original_module %>% as.numeric()), verbose = FALSE)
    } %>% suppressMessages()
  } else {
    if ("others" %in% v_index$module) message("Consider others as one module!")
    {
      go1 <- anno_vertex(go1, data.frame(
        row.names = V(go1)$name,
        module = tidai(v_index$module, seq_along(unique(v_index$module)))
      ), verbose = FALSE)
    } %>% suppressMessages()
  }

  within <- within_module_deg_z_score(go1)
  v_index$Ki <- within$Ki
  v_index$Zi <- within$Zi
  pc <- part_coeff(go1)
  v_index$Pi <- pc$Pi

  if (mode == 1) {
    lab <- c("Ultra-peripherals", "Peripherals", "Non-hub connectors", "Non-hub kinless nodes", "Provincial hubs", "Connector hubs", "Kinless hubs")
    backs <- data.frame(
      x1 = c(0, 0.05, 0.62, 0.8, 0, 0.3, 0.75),
      x2 = c(0.05, 0.62, 0.8, 1, 0.3, 0.75, 1),
      y1 = c(-Inf, -Inf, -Inf, -Inf, 2.5, 2.5, 2.5),
      y2 = c(2.5, 2.5, 2.5, 2.5, Inf, Inf, Inf),
      lab = factor(lab, levels = lab)
    )
  } else if (mode == 2) {
    lab <- c("Peripherals", "Network hubs", "Module hubs", "Connectors")
    backs <- data.frame(
      x1 = c(0, 0.62, 0, 0.62),
      x2 = c(0.62, 1, 0.62, 1),
      y1 = c(-Inf, 2.5, 2.5, -Inf),
      y2 = c(2.5, Inf, Inf, 2.5),
      lab = factor(lab, levels = lab)
    )
  }
  deter_role <- \(x, y, backs = backs){
    for (i in seq_len(nrow(backs))) {
      flag <- dplyr::between(as.numeric(x), backs$x1[i], backs$x2[i]) && dplyr::between(as.numeric(y), backs$y1[i], backs$y2[i])
      if (is.na(flag)) {
        return(NA)
      } else if (flag) {
        # if((backs$x1[i]<=x)&(backs$x2[i]>=x)&(backs$y1[i]<=y)&(backs$y2[i]>=y)){
        role <- backs$lab[i]
        break
      }
    }
    return(role)
  }
  v_index$roles <- apply(v_index, 1, \(x)deter_role(x["Pi"], x["Zi"], backs))

  vertex.attributes(go_m) <- as.list(v_index)
  return(go_m)
}


#' calculate Zi
#'
#' @param g igraph object
#' @param A adjacency matrix
#' @param weighted logical, default: FALSE
#'
#' @return within_module_deg_z_score
#' @noRd
#' @references https://github.com/cwatson/brainGraph/blob/master/R/vertex_roles.R
within_module_deg_z_score <- function(g, A = NULL, weighted = FALSE) {
  stopifnot(is_igraph(g))
  if (is.null(A)) {
    if (isTRUE(weighted)) {
      A <- as_adj(g, sparse = FALSE, names = TRUE, attr = "weight")
    } else {
      A <- as_adj(g, sparse = FALSE, names = TRUE)
    }
  }
  memb <- vertex_attr(g, "module") %>% as.numeric()
  N <- max(memb)
  nS <- tabulate(memb)
  z <- Ki <- rep.int(0, dim(A)[1L])
  Ksi <- sigKsi <- rep.int(0, N)
  names(z) <- names(Ki) <- rownames(A)
  for (S in seq_len(N)) {
    x <- rowSums(as.matrix(A[memb == S, memb == S]))
    Ki[memb == S] <- x
    Ksi[S] <- sum(x) / nS[S]
    sigKsi[S] <- sqrt(sum((x - Ksi[S])^2) / (nS[S] - 1))
  }
  z <- (Ki - Ksi[memb]) / sigKsi[memb]
  z[is.infinite(z)] <- 0
  z[is.nan(z)] <- 0
  Zi <- z
  df <- data.frame(Ki, Zi, row.names = names(Ki))
  return(df)
}

# calculate Pi
part_coeff <- function(g, A = NULL, weighted = FALSE) {
  stopifnot(is_igraph(g))
  if (is.null(A)) {
    if (isTRUE(weighted)) {
      A <- as_adj(g, sparse = FALSE, attr = "weight")
    } else {
      A <- as_adj(g, sparse = FALSE)
    }
  }
  memb <- vertex_attr(g, "module") %>% as.numeric()
  Ki <- colSums(A)
  Kis <- t(rowsum(A, memb))
  Pi <- 1 - ((1 / Ki^2) * rowSums(Kis^2))
  names(Pi) <- rownames(A)
  Pi <- data.frame(Pi)
  return(Pi)
}


#' Zi-Pi plot of vertexes
#'
#' @param go igraph object after zp_analyse()
#' @param label show label or not
#' @param mode plot style, 1~3
#'
#' @return a ggplot object
#' @export
#' @rdname zp_analyse
zp_plot <- function(go, label = TRUE, mode = 1) {
  v_class <- value <- size <- roles <- x1 <- x2 <- y1 <- y2 <- Pi <- Zi <- name <- NULL
  lib_ps("ggrepel", library = FALSE)
  get_v(go) -> taxa.roles
  if (!"roles" %in% names(taxa.roles)) stop("no roles, please zp_analyse() first")

  if (mode == 3) {
    reshape2::melt(taxa.roles, measure.vars = c("Zi", "Pi")) -> taxa.roles1
    p <- ggplot(taxa.roles1, aes(x = v_class, y = value, col = v_class, size = size, shape = roles)) +
      geom_point() +
      facet_grid(variable ~ ., scales = "free_y") +
      theme_bw() +
      labs(x = NULL, y = NULL) +
      guides(col = "none") +
      theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust = 1)) +
      scale_color_manual(values = setNames(unique(taxa.roles$color), unique(taxa.roles$v_class)))
    return(p)
  }

  mode <- ifelse(nlevels(V(go)$roles) == 7, 1, 2)
  if (mode == 1) {
    lab <- c("Ultra-peripherals", "Peripherals", "Non-hub connectors", "Non-hub kinless nodes", "Provincial hubs", "Connector hubs", "Kinless hubs")
    CPCOLS <- c("#FCF6EFFC", "#EEBCF5", "#EDEDA4", "#FAA371", "#FC5D6096", "#9BC799B9", "#94CCF2AC")
    names(CPCOLS) <- lab
    backs <- data.frame(
      x1 = c(0, 0.05, 0.62, 0.8, 0, 0.3, 0.75),
      x2 = c(0.05, 0.62, 0.8, 1, 0.3, 0.75, 1),
      y1 = c(-Inf, -Inf, -Inf, -Inf, 2.5, 2.5, 2.5),
      y2 = c(2.5, 2.5, 2.5, 2.5, Inf, Inf, Inf),
      lab = factor(lab, levels = lab)
    )
  } else if (mode == 2) {
    lab <- c("Peripherals", "Network hubs", "Module hubs", "Connectors")
    CPCOLS <- c("#FCF6EFFC", "#FC5D6096", "#9BC799B9", "#94CCF2AC")
    names(CPCOLS) <- lab
    backs <- data.frame(
      x1 = c(0, 0.62, 0, 0.62),
      x2 = c(0.62, 1, 0.62, 1),
      y1 = c(-Inf, 2.5, 2.5, -Inf),
      y2 = c(2.5, Inf, Inf, 2.5),
      lab = factor(lab, levels = lab)
    )
  }
  p <- ggplot() +
    geom_rect(data = backs, mapping = aes(xmin = x1, xmax = x2, ymin = y1, ymax = y2, fill = lab), alpha = 0.7) +
    guides(fill = guide_legend(title = "Topological roles")) +
    scale_fill_manual(values = CPCOLS) +
    geom_point(data = taxa.roles, aes(x = Pi, y = Zi, color = factor(v_class))) +
    scale_color_manual(values = setNames(unique(taxa.roles$color), unique(taxa.roles$v_class))) +
    MetaNet_theme +
    guides(colour = "none") +
    theme(strip.background = element_rect(fill = "white")) +
    xlab("Participation Coefficient (Pi)") +
    ylab("Within-module connectivity (Zi)")

  if (label) {
    label_dat <- taxa.roles[!taxa.roles$roles %in% c("Peripherals", "Ultra-peripherals"), ]
    label_dat <- label_dat[!is.na(label_dat$roles), ]
    p <- p + ggrepel::geom_text_repel(
      data = label_dat,
      aes(x = Pi, y = Zi, label = name), size = 3
    )
  }
  return(p)
}

#' @title MetaNet networks
#' @description MetaNet co_nets
#'
#' @docType data
#' @name co_net
NULL

#' @title MetaNet networks
#' @description MetaNet co_nets
#'
#' @docType data
#' @name co_net2
NULL

#' @title MetaNet networks
#' @description MetaNet co_nets
#'
#' @docType data
#' @name co_net_rmt
NULL

#' @title Edgelist
#' @description Edgelist for `c_net_from_edgelist()`
#'
#' @docType data
#' @name arc_count
#'
NULL

#' @title Edgelist
#' @description Edgelist for `c_net_from_edgelist()`
#'
#' @docType data
#' @name arc_taxonomy
#'
NULL

#' @title MetaNet networks metadata
#' @description MetaNet co_nets
#'
#' @docType data
#' @name metab_g
NULL

#' @title MetaNet networks metadata
#' @description MetaNet co_nets
#'
#' @docType data
#' @name transc_g
NULL

#' @title MetaNet networks metadata
#' @description MetaNet co_nets
#'
#' @docType data
#' @name micro_g
NULL

#' @title MetaNet networks abundance
#' @description MetaNet co_nets
#'
#' @docType data
#' @name metab
NULL

#' @title MetaNet networks abundance
#' @description MetaNet co_nets
#'
#' @docType data
#' @name micro
NULL

#' @title MetaNet networks abundance
#' @description MetaNet co_nets
#'
#' @docType data
#' @name transc
NULL

#' @title MetaNet networks
#' @description MetaNet co_nets
#'
#' @docType data
#' @name multi1
NULL

# ========5.topological=======

#' Extract each sample network from the whole network
#'
#' @param whole_net the whole network
#' @param otutab otutab, columns are samples, these columns will be extract
#' @param threads threads, default: 1
#' @param save_net should save these sub_nets? FALSE or a filename
#' @param fast less indexes for faster calculate ?
#' @param verbose verbose
#' @param remove_negative remove negative edge or not? default: FALSE
#'
#' @return a dataframe contains all sub_net parameters
#' @export
#' @family topological
#' @examples
#' data(otutab, package = "pcutils")
#' extract_sample_net(co_net, otutab) -> sub_net_pars
extract_sample_net <- function(whole_net, otutab, threads = 1, save_net = FALSE, fast = TRUE, remove_negative = FALSE, verbose = TRUE) {
  i <- NULL
  V(whole_net)$name -> v_name
  reps <- ncol(otutab)

  if (verbose) message("extracting")
  sub_nets <- lapply(1:reps, \(i){
    rownames(otutab)[otutab[, i] > 0] -> exist_sp
    subgraph(whole_net, which(v_name %in% exist_sp)) -> spe_sub
    class(spe_sub) <- c("metanet", "igraph")
    return(spe_sub)
  })
  names(sub_nets) <- colnames(otutab)
  if (verbose) message("calculating topological indexes")

  # parallel
  # main function
  loop <- function(i) {
    spe_sub <- sub_nets[[i]]
    indexs <- net_par(spe_sub, mode = "n", fast = fast, remove_negative = remove_negative)[["n_index"]]
    wc <- igraph::cluster_fast_greedy(spe_sub, weights = abs(igraph::E(spe_sub)$weight))
    indexs$Modularity <- igraph::modularity(wc)
    indexs
  }
  {
    if (threads > 1) {
      pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
      if (verbose) {
        pb <- utils::txtProgressBar(max = reps, style = 3)
        opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
      } else {
        opts <- NULL
      }
      cl <- snow::makeCluster(threads)
      doSNOW::registerDoSNOW(cl)
      res <- foreach::`%dopar%`(
        foreach::foreach(i = 1:reps, .options.snow = opts),
        loop(i)
      )
      snow::stopCluster(cl)
      gc()
    } else {
      res <- lapply(1:reps, loop)
    }
  }
  # simplify method
  sub_net_pars <- do.call(rbind, res)

  rownames(sub_net_pars) <- colnames(otutab)

  if (is.logical(save_net)) {
    if (save_net) save_net <- paste0("sub_net_", date())
  }
  if (is.character(save_net)) {
    saveRDS(sub_nets, file = paste0(save_net, ".RDS"))
  }
  sub_net_pars
}

#' Calculate natural_connectivity
#'
#' @param p an igraph or metanet object
#' @return natural_connectivity (numeric)
#' @export
#' @references \code{`nc` in `ggClusterNet`}
#' @family topological
#' @examples
#' igraph::make_ring(10) %>% nc()
nc <- function(p) {
  if (length(p) == 0) {
    return(NaN)
  }
  adj_matrix <- as.matrix(igraph::as_adj(p, sparse = FALSE))
  adj_matrix[abs(adj_matrix) != 0] <- 1

  lambda <- eigen(adj_matrix, only.values = TRUE)$values
  lambda <- sort(lambda, decreasing = TRUE)

  lambda_sum <- 0
  N <- length(lambda)
  for (i in 1:N) lambda_sum <- lambda_sum + exp(lambda[i])
  lambda_average <- log(lambda_sum / N, base = exp(1))
  lambda_average
}


#' Calculate all topological indexes of a network
#'
#' @param go an igraph or metanet object
#' @param fast less indexes for faster calculate ?
#' @param mode calculate what? c("v", "e", "n", "all")
#' @param remove_negative remove negative edge or not? default: FALSE
#' @param only_topological only return topological indexes
#'
#' @return a 3-elements list
#' \item{n_index}{indexs of the whole network}
#' \item{v_index}{indexs of each vertex}
#' \item{e_index}{indexs of each edge}
#' @export
#' @family topological
#' @examples
#' igraph::make_graph("Walther") %>% net_par()
#' c_net_index(co_net) -> co_net_with_par
net_par <- function(go, mode = c("v", "e", "n", "all"), fast = TRUE, remove_negative = FALSE, only_topological = FALSE) {
  from <- to <- NULL
  stopifnot(is_igraph(go))
  if ("all" %in% mode) mode <- c("v", "e", "n")

  n_index <- NULL
  v_index <- NULL
  e_index <- NULL

  Negative_percentage <- ifelse(!is.null(E(go)$cor), sum(igraph::E(go)$cor < 0) / length(igraph::E(go)), NA)
  # remove negative weight
  if (remove_negative) {
    if (!is.null(E(go)$cor)) {
      # message("Remove negative correlation edges")
      c_net_filter(go, cor > 0, mode = "e") -> go
    }
  }

  # non-weighted network
  up <- go
  if (!is.null(igraph::edge_attr(up)[["weight"]])) {
    up <- igraph::delete_edge_attr(up, "weight")
  }

  if ("n" %in% mode) {
    # Calculate Network Parameters
    n_index <- data.frame(
      check.names = F,
      `Node_number` = length(igraph::V(go)), # number of nodes
      `Edge_number` = length(igraph::E(go)), # number of edges
      `Edge_density` = igraph::edge_density(go), # density of network, connectance
      `Negative_percentage` = Negative_percentage, # negative edges percentage
      `Average_path_length` = igraph::average.path.length(up), # Average path length

      `Global_efficiency` = igraph::global_efficiency(up),
      `Average_degree` = mean(igraph::degree(go)), # Average degree
      `Average_weighted_degree` = ifelse(is.null(igraph::E(go)$weight), mean(igraph::degree(go)), sum(igraph::E(go)$weight) / length(igraph::V(go))), # weighted degree
      Diameter = igraph::diameter(up), # network diameter
      `Clustering_coefficient` = igraph::transitivity(go), # Clustering coefficient
      `Centralized_betweenness` = igraph::centralization.betweenness(go)$centralization, # Betweenness centralization
      `Natural_connectivity` = nc(go) # natural
    )

    if (!fast) {
      # mean_dist=mean_distance(go)#
      # w_mean_dist=ifelse(is.null(E(go)$weight),mean_dist,mean_distance(go))
      # v_conn= vertex.connectivity(go) #
      # e_conn= edge.connectivity(go) #
      # components= count_components(go) #
      modularity <- igraph::modularity(igraph::cluster_fast_greedy(go)) #
      rand.g <- igraph::erdos.renyi.game(length(V(go)), length(E(go)), type = "gnm")
      rand_m <- igraph::modularity(igraph::cluster_fast_greedy(rand.g))
      relative_modularity <- (modularity - rand_m) / rand_m #

      n_index <- data.frame(
        check.names = F,
        n_index,
        Modularity = modularity,
        `Relative_modularity` = relative_modularity,
        `Centralized_closeness` = igraph::centralization.closeness(go)$centralization, # Closeness centralization
        `Centralized_degree` = igraph::centralization.degree(go)$centralization, # Degree centralization
        `Centralized_eigenvector` = igraph::centralization.evcent(go)$centralization # eigenvector centralization
      )
    }
    n_index <- apply(n_index, 1, FUN = \(x)replace(x, is.nan(x), 0)) %>%
      t() %>%
      as.data.frame()
    if (!only_topological) n_index <- cbind_new(get_n(go, simple = TRUE), n_index)
  }
  if ("v" %in% mode) {
    negative_weight <- FALSE
    if (!is.null(igraph::edge_attr(go)[["weight"]])) {
      if (any(igraph::edge_attr(go)[["weight"]] < 0)) {
        negative_weight <- TRUE
        warning("Weight vector must be positive, drop the weight.")
      }
    }
    if (is.null(igraph::vertex_attr(go)[["name"]])) {
      igraph::V(go)$name <- as.character(igraph::V(go))
    }
    # Calculate Vertices Parameters
    v_index <- data.frame(
      check.names = F,
      Degree = igraph::degree(go),
      `Clustering_coefficient` = igraph::transitivity(go, type = "local"), # local clustering coefficient
      Betweenness = ifelse(negative_weight, igraph::betweenness(up), igraph::betweenness(go)), # betweenness
      Eccentricity = igraph::eccentricity(go),
      Closeness = ifelse(negative_weight, igraph::closeness(up), igraph::closeness(go)),
      `Hub_score` = igraph::hub_score(go)[["vector"]]
      # page_rank = page.rank(go)$vector
      # igraph::evcent(go)[["vector"]]
      # igraph::local_efficiency(go)
    )

    # weighted degree
    if (!is.null(E(go)$cor)) {
      get_e(go) -> edge_list
      edge_list %>%
        dplyr::select(from, cor) %>%
        rbind(., dplyr::select(edge_list, to, cor) %>% dplyr::rename(from = to)) %>%
        dplyr::group_by(from) %>%
        dplyr::summarise(w_degree = sum(cor)) -> w_degree
      v_index$`Average_weighted_degree` <- w_degree[match(rownames(v_index), w_degree$from), "w_degree"] %>% unlist()
    }

    v_index <- apply(v_index, 1, FUN = \(x)replace(x, is.nan(x), 0)) %>%
      t() %>%
      as.data.frame()
    v_index <- data.frame(
      check.names = F,
      name = igraph::vertex_attr(go, "name"),
      v_index
    )
    if (!only_topological) v_index <- cbind_new(get_v(go), v_index)
  }
  if ("e" %in% mode) {
    # Calculate Edges Parameters
    e_index <- get_e(go)
    # if(!(edge_attr(go)%>%unlist()%>%is.null()))e_index=data.frame(edge_attr(go),e_index)
  }
  return(list(n_index = n_index, v_index = v_index, e_index = e_index))
}


#' Add topological indexes for a network
#' @param go igraph or metanet
#' @param force replace existed net_par
#'
#' @export
#' @rdname net_par
c_net_index <- function(go, force = FALSE) {
  if (!force) {
    if (!is.null(graph_attr(go)[["net_par"]])) stop("Already calculated net_pars, set `force = TRUE to replace existed net_par")
  }
  net_par(go, fast = FALSE) -> res
  graph_attr(go) <- as.list(res$n_index)
  graph_attr(go)[["net_par"]] <- TRUE
  vertex_attr(go) <- as.list(res$v_index)
  edge_attr(go) <- as.list(res$e_index)
  go
}


#' Fit power-law distribution for an igraph
#'
#' @param go igraph
#' @param p.value calculate p.value
#'
#' @return ggplot
#' @export
#' @family topological
#' @examples
#' fit_power(co_net)
fit_power <- function(go, p.value = FALSE) {
  x <- y <- formula <- NULL
  # igraph::degree distribution
  degree_dist <- table(igraph::degree(go))
  dat <- data.frame(degree = as.numeric(names(degree_dist)), count = as.numeric(degree_dist))
  # fit, set the original a & b
  mod <- stats::nls(count ~ a * degree^b, data = dat, start = list(a = 2, b = 1.5))
  summary(mod)
  # extract the coefficient
  a <- round(coef(mod)[1], 3)
  b <- round(coef(mod)[2], 3)
  fit <- fitted(mod)
  SSre <- sum((dat$count - fit)^2)
  SStot <- sum((dat$count - mean(dat$count))^2)
  R2 <- round(1 - SSre / SStot, 3)

  # bootstrap t get p.value
  if (p.value) {
    dat_rand <- dat
    p_num <- lapply(seq_len(999), \(i){
      dat_rand$count <- sample(dat_rand$count)
      SSre_rand <- sum((dat_rand$count - fit)^2)
      SStot_rand <- sum((dat_rand$count - mean(dat_rand$count))^2)
      R2_rand <- 1 - SSre_rand / SStot_rand
      R2_rand > R2
    })
    p_value <- (sum(unlist(p_num)) + 1) / (999 + 1)
  }

  p <- ggplot(dat, aes(x = degree, y = count)) +
    geom_point() +
    theme_bw() +
    stat_smooth(method = "nls", formula = y ~ a * x^b, method.args = list(start = list(a = 2, b = 1.5)), se = FALSE) +
    labs(x = "Degree", y = "Count")

  if (p.value) {
    label <- data.frame(
      x = 0.8 * max(dat$degree),
      y = c(0.9, 0.8, 0.7) * max(dat$count),
      formula = c(
        sprintf("italic(Y) == %.3f*italic(X)^%.3f", a, b),
        sprintf("italic(R^2) == %.3f", R2),
        sprintf("italic(P) < %.3f", p_value)
      )
    )
  } else {
    label <- data.frame(
      x = 0.8 * max(dat$degree),
      y = c(0.9, 0.8) * max(dat$count),
      formula = c(
        sprintf("italic(Y) == %.3f*italic(X)^%.3f", a, b),
        sprintf("italic(R^2) == %.3f", R2)
      )
    )
  }
  p + geom_text(aes(x = x, y = y, label = formula), data = label, parse = TRUE)
}


#' Plot degree distribution of networks
#'
#' @param gols list of metanet
#' @param net_names names of networks
#'
#' @returns ggplot
#' @export
plot_net_degree <- function(gols, net_names = NULL) {
  freq <- net <- degree <- NULL
  if (is.null(names(gols))) {
    names(gols) <- paste0("Network", seq_along(gols))
  }
  if (!is.null(net_names)) {
    names(gols) <- net_names
  }
  lapply(seq_along(gols), \(i){
    data.frame(
      freq = igraph::degree_distribution(gols[[i]]), net = names(gols)[[i]],
      degree = 0:(length(degree_distribution(gols[[i]])) - 1)
    )
  }) -> data_list
  data <- do.call(rbind, data_list)

  # if data1[1,1]=0, it'is delete single vertex
  if (data[1, 1] == 0) data <- data[-1, ]

  p1 <- ggplot(data) +
    geom_point(aes(x = degree, y = freq, group = net, fill = net), pch = 21, size = 2) +
    geom_smooth(aes(x = degree, y = freq, group = net, color = net), se = FALSE, method = "loess", formula = "y ~ x") +
    labs(x = "Degree", y = "Proportion") +
    MetaNet_theme +
    theme(legend.position = c(0.8, 0.9), legend.title = element_blank())
  p1
}

#' Degree distribution comparison with random network
#'
#' @param go igraph object
#' @param plot plot or not
#'
#' @return ggplot
#' @export
#' @family topological
#' @examples
#' rand_net(co_net)
rand_net <- function(go = go, plot = TRUE) {
  freq <- net <- NULL
  # generate a random network
  rand.g <- igraph::erdos.renyi.game(length(V(go)), length(E(go)), type = "gnm")

  if (!plot) {
    return(rand.g)
  }
  plot_net_degree(list(go, rand.g), net_names = c("Network", "Random E-R")) -> p1
  print(p1 + scale_color_manual(values = c("#F58B8B", "#7AADF0")) +
    scale_fill_manual(values = c("#F58B8B", "#7AADF0")))
  return(rand.g)
}


#' Net_pars of many random network
#'
#' @param go igraph
#' @param reps simulation time
#' @param threads threads
#' @param verbose verbose
#'
#' @export
#' @rdname compare_rand
rand_net_par <- function(go, reps = 99, threads = 1, verbose = TRUE) {
  i <- NULL
  # parallel
  # main function
  loop <- function(i) {
    # generate a random network
    rand.g <- igraph::erdos.renyi.game(length(igraph::V(go)),
      length(igraph::E(go)),
      type = "gnm"
    )
    indexs <- net_par(rand.g, mode = "n")[["n_index"]]
    wc <- igraph::cluster_fast_greedy(rand.g)
    indexs$modularity <- igraph::modularity(wc)
    indexs
  }
  {
    if (threads > 1) {
      pcutils::lib_ps("foreach", "doSNOW", "snow", library = FALSE)
      if (verbose) {
        pb <- utils::txtProgressBar(max = reps, style = 3)
        opts <- list(progress = function(n) utils::setTxtProgressBar(pb, n))
      } else {
        opts <- NULL
      }
      cl <- snow::makeCluster(threads)
      doSNOW::registerDoSNOW(cl)
      res <- foreach::`%dopar%`(
        foreach::foreach(i = 1:reps, .options.snow = opts),
        loop(i)
      )
      snow::stopCluster(cl)
      gc()
    } else {
      res <- lapply(1:reps, loop)
    }
  }
  # simplify method
  rand_net_pars <- do.call(rbind, res)

  rand_net_pars
}

#' Compare some indexes between your net with random networks
#'
#' @param pars your net pars resulted by net_pars()
#' @param randp random networks pars resulted by rand_net_par()
#' @param index compared indexes: "Average_path_length","Clustering_coefficient" or else
#'
#' @return ggplot
#' @export
#' @family topological
#' @examples
#' data("c_net")
#' rand_net_par(co_net_rmt, reps = 30) -> randp
#' net_par(co_net_rmt, fast = FALSE) -> pars
#' compare_rand(pars, randp)
compare_rand <- function(pars, randp, index = c("Average_path_length", "Clustering_coefficient")) {
  V1 <- NULL
  labss <- t(pars$n_index[, index, drop = FALSE]) %>% as.data.frame()
  rownames(labss) -> labss$indexes

  p <- pcutils::group_box(randp[, index, drop = FALSE])

  p <- p +
    geom_hline(data = labss, aes(yintercept = V1), linetype = 2, color = "blue3") +
    geom_text(
      data = labss, aes(x = 1, y = V1 * 1.05, label = paste0("Network: ", round(V1, 3))),
      color = "blue3"
    ) +
    MetaNet_theme +
    theme(legend.position = "none", axis.text.x = element_blank())
  p
}


#' Calculate small-world coefficient
#'
#' @param go igraph or metanet
#' @param reps simulation time
#' @param threads threads
#' @param verbose verbose
#'
#' @return number
#' @export
#' @family topological
#' @examples
#' \donttest{
#' # set reps at least 99 when you run.
#' smallworldness(co_net, reps = 9)
#' }
smallworldness <- function(go, reps = 99, threads = 1, verbose = TRUE) {
  rand_net_par(go, reps = reps, threads = threads, verbose = verbose) -> rands
  small_world_coefficient <- (igraph::transitivity(go) / mean(rands$Clustering_coefficient)) /
    (igraph::average.path.length(go) / mean(rands$`Average_path_length`))
  small_world_coefficient
}

# ========4.1.other_plot========
#
#' Transfer an igraph object to a ggig
#'
#' @param go igraph or meatnet
#' @param coors coordinates for nodes,columns: name, X, Y
#'
#' @return ggig object
#' @export
#' @family plot
#' @examples
#' as.ggig(co_net, coors = c_net_layout(co_net)) -> ggig
#' plot(ggig)
#' as.ggig(multi1, coors = c_net_layout(multi1)) -> ggig
#' plot(ggig, labels_num = 0.3)
as.ggig <- function(go, coors = NULL) {
  list(n_index = get_n(go), v_index = get_v(go), e_index = get_e(go)) -> net_par_res

  if (is.null(coors)) coors <- c_net_layout(go)
  coors <- get_coors(coors, go)

  # add coors
  coors <- coors[, 1:3] %>% na.omit()
  net_par_res$v_index %<>% dplyr::left_join(., coors, by = "name", suffix = c("", ".1"))
  net_par_res$e_index %<>% dplyr::left_join(., coors, by = c("from" = "name")) %>%
    dplyr::rename(X1 = "X", Y1 = "Y") %>%
    dplyr::left_join(., coors, by = c("to" = "name")) %>%
    dplyr::rename(X2 = "X", Y2 = "Y")

  class(net_par_res) <- c("ggig", "list")
  return(net_par_res)
}

#' Plot a ggig
#'
#' @param x ggig object
#' @inheritParams c_net_plot
#'
#' @family plot
#' @return ggplot
#' @exportS3Method
#' @method plot ggig
plot.ggig <- function(x, coors = NULL, ..., labels_num = NULL,
                      vertex_size_range = NULL, edge_width_range = NULL,
                      plot_module = FALSE,
                      mark_module = FALSE, mark_color = NULL, mark_alpha = 0.3,
                      module_label = FALSE, module_label_cex = 2, module_label_color = "black",
                      module_label_just = c(0.5, 0.5),
                      legend_number = FALSE, legend = TRUE, legend_cex = 1,
                      legend_position = c(left_leg_x = -2, left_leg_y = 1, right_leg_x = 1.2, right_leg_y = 1),
                      group_legend_title = NULL, group_legend_order = NULL,
                      color_legend = TRUE, color_legend_order = NULL,
                      size_legend = FALSE, size_legend_title = "Node Size",
                      edge_legend = TRUE, edge_legend_title = "Edge type", edge_legend_order = NULL,
                      width_legend = FALSE, width_legend_title = "Edge width",
                      lty_legend = FALSE, lty_legend_title = "Edge class", lty_legend_order = NULL,
                      params_list = NULL,
                      seed = 1234) {
  if (!is.null(params_list)) {
    as.list(match.call()[-1]) -> set_params_list
    set_params_list[["params_list"]] <- NULL
    for (i in seq_along(params_list)) {
      if (names(params_list)[i] %in% names(set_params_list)) {
        message("The parameter `", names(params_list)[i], "` is duplicated, the format argument will be used.")
      }
    }
    pcutils::update_param(params_list, set_params_list) -> set_params_list
    do.call(c_net_plot, set_params_list)
    return(invisible())
  }

  rename <- size <- color <- e_type <- lty <- e_class <- v_class <- shape <- X1 <- Y1 <- X2 <- Y2 <- width <- X <- Y <- label <- NULL
  edge_width_text <- NULL

  ggig <- x
  ggig$v_index -> tmp_v
  ggig$e_index -> tmp_e

  set.seed(seed)
  # get coordinates
  if (!is.null(coors)) {
    tmp_v$X <- tmp_v$Y <- NULL
    tmp_e$X1 <- tmp_e$X2 <- tmp_e$Y1 <- tmp_e$Y2 <- NULL
    # add coors
    tmp_v %<>% dplyr::left_join(., coors, by = "name", suffix = c("", ".1"))
    tmp_e %<>% dplyr::left_join(., coors, by = c("from" = "name")) %>%
      rename(X1 = "X", Y1 = "Y") %>%
      dplyr::left_join(., coors, by = c("to" = "name")) %>%
      rename(X2 = "X", Y2 = "Y")
  }

  # get network type
  main <- get_net_main(ggig$n_index)

  # scale the size and width
  scale_size_width(tmp_v, tmp_e, vertex_size_range, edge_width_range)

  # new shapes
  tmp_v$shape <- tidai(tmp_v$v_group, 21:26)

  # some custom parameters
  some_custom_paras(tmp_v, tmp_e, ...)

  # show labels
  tmp_v <- get_show_labels(tmp_v, labels_num)

  if (TRUE) {
    tmp_e$e_type <- pcutils::change_fac_lev(tmp_e$e_type, edge_legend_order)
    edges <- levels(tmp_e$e_type)
    edge_cols <- dplyr::distinct(tmp_e, color, e_type)
    edge_cols <- setNames(edge_cols$color, edge_cols$e_type)
    if (legend_number) {
      eee <- table(tmp_e$e_type)
      edge_text <- paste(edges, eee[edges], sep = ": ")
    } else {
      edge_text <- edges
    }
  }

  if (TRUE) {
    edges1 <- levels(factor(tmp_e$e_class))
    edge_ltys <- dplyr::distinct(tmp_e, lty, e_class)
    edge_ltys <- setNames(edge_ltys$lty, edge_ltys$e_class)

    if (legend_number) {
      eee <- table(tmp_e$e_class)
      lty_text <- paste(edges1, eee[edges1], sep = ": ")
    } else {
      lty_text <- edges1
    }
  }

  if (TRUE) {
    vgroups <- pcutils::change_fac_lev(tmp_v$v_group, group_legend_order)

    node_size_text <- c(
      paste(lapply(node_size_text[levels(vgroups)], \(i)round(i[1], 3)), collapse = "/ "),
      paste(lapply(node_size_text[levels(vgroups)], \(i)round(i[2], 3)), collapse = "/ ")
    )

    new_f <- c()
    for (g_i in levels(vgroups)) {
      tmp_v1 <- tmp_v[tmp_v$v_group == g_i, c("v_class", "color", "shape")]
      tmp_f <- pcutils::change_fac_lev(tmp_v1$v_class, color_legend_order)
      new_f <- c(new_f, levels(tmp_f))
    }

    tmp_v$v_class <- pcutils::change_fac_lev(tmp_v$v_class, new_f)
    vclass <- levels(tmp_v$v_class)

    node_cols <- dplyr::distinct(tmp_v, color, v_class)
    node_cols <- setNames(node_cols$color, node_cols$v_class)
    node_shapes <- dplyr::distinct(tmp_v, shape, v_class)
    node_shapes <- setNames(node_shapes$shape, node_shapes$v_class)

    if (legend_number) {
      eee <- table(tmp_v$v_class)
      le_text <- paste(vclass, eee[vclass], sep = ": ")
    } else {
      le_text <- vclass
    }
  }

  p <- ggplot() +
    geom_segment(aes(
      x = X1, y = Y1, xend = X2, yend = Y2, color = e_type,
      linewidth = width, linetype = e_class
    ), data = tmp_e, alpha = 0.7) + # draw edges
    scale_color_manual(
      name = edge_legend_title, values = edge_cols,
      label = edge_text, guide = ifelse(edge_legend, "legend", "none")
    ) + # edge colors
    scale_linetype_manual(
      name = lty_legend_title, values = edge_ltys,
      label = lty_text, guide = ifelse(lty_legend, "legend", "none")
    ) + # edge linetype
    scale_linewidth(
      name = width_legend_title, breaks = c(min(tmp_e$width), max(tmp_e$width)), range = edge_width_range,
      labels = edge_width_text, guide = ifelse(width_legend, "legend", "none")
    )

  p1 <- p +
    geom_point(aes(X, Y, fill = v_class, size = size, shape = v_class), data = tmp_v) + # draw nodes
    # scale_shape_manual(values =setNames(default_v_shape[node_shapes],vclass))+#node shape
    scale_shape_manual(values = node_shapes) +
    scale_fill_manual(
      name = group_legend_title, values = node_cols[vclass],
      labels = le_text, guide = ifelse(color_legend, "legend", "none")
    ) + # node color
    scale_size(
      name = size_legend_title, breaks = c(min(tmp_v$size), max(tmp_v$size)),
      labels = node_size_text, guide = ifelse(size_legend, "legend", "none"),
      range = range(unlist(vertex_size_range))
    ) + # node size

    ggnewscale::new_scale("size") +
    geom_text(aes(X, Y, size = size, label = label), col = "black", data = tmp_v, show.legend = FALSE) +
    scale_size(range = range(unlist(vertex_size_range) / 5), guide = "none") +
    guides(
      fill = guide_legend(override.aes = list(shape = node_shapes[vclass])),
      shape = "none"
    )

  p2 <- p1 + labs(title = main) +
    scale_x_continuous(breaks = NULL) + scale_y_continuous(breaks = NULL) +
    coord_fixed(ratio = 1) +
    theme(panel.background = element_blank()) +
    theme(axis.title.x = element_blank(), axis.title.y = element_blank()) +
    theme(
      legend.background = element_rect(colour = NA),
      legend.box.background = element_rect(colour = NA),
      legend.key = element_rect(fill = NA)
    ) +
    theme(panel.background = element_rect(fill = "white", colour = NA)) +
    theme(panel.grid.minor = element_blank(), panel.grid.major = element_blank())

  if (!legend) {
    return(p2 + theme(legend.position = "none"))
  }

  p2
}


#' Input a graphml file exported by Gephi
#'
#' @param file graphml file exported by Gephi
#' @family plot
#' @return list contains the igraph object and coordinates
#'
#' @export
input_gephi <- function(file) {
  X <- Y <- code <- NULL
  igraph::read.graph(file, format = "graphml") -> gephi
  get_v(gephi) -> tmp_v
  # extract coors
  if (!all(c("x", "y", "r", "g", "b", "id") %in% colnames(tmp_v))) {
    stop("This file is not exported by Gephi, please use igraph::read.graph()")
  }
  coors <- tmp_v[, c("x", "y")]
  coors <- data.frame(name = tmp_v$label, X = coors[, 1], Y = coors[, 2])
  attributes(coors)$class <- c("coors", class(coors))
  coors <- rescale_coors(coors)

  # transform color
  pcutils::rgb2code(tmp_v[, c("r", "g", "b")]) %>% dplyr::pull(code) -> tmp_v$color
  if ("cor" %in% edge.attributes(gephi)) {
    E(gephi)$color <- ifelse(E(gephi)$cor > 0, "#48A4F0", "#E85D5D")
  } else {
    E(gephi)$color <- "#48A4F0"
  }
  # scale size
  tmp_v$size <- pcutils::mmscale(tmp_v$size, 1, 5)
  E(gephi)$width <- pcutils::mmscale(E(gephi)$width, 0.05, 0.2)
  # delete
  tmp_v %>%
    dplyr::select(-c("label", "x", "y", "r", "g", "b", "id")) %>%
    as.list() -> vertex.attributes(gephi)
  edge.attributes(gephi)["Edge Label"] <- edge.attributes(gephi)["id"] <- NULL

  gephi <- c_net_update(gephi, initialize = TRUE)
  igraph::graph_attr(gephi, "coors") <- coors
  return(list(go = gephi, coors = coors))
}


#' Input a cyjs file exported by Cytoscape
#'
#' @param file cyjs file exported by Cytoscape
#' @family plot
#' @return list contains the igraph object and coordinates
#'
#' @export
input_cytoscape <- function(file) {
  c_net_load(file, format = "cyjs") -> cyto

  get_v(cyto) -> tmp_v
  coors <- tmp_v[, c("x", "y")]
  coors <- data.frame(name = tmp_v$label, X = coors[, 1], Y = coors[, 2])

  attributes(coors)$class <- c("coors", class(coors))
  coors <- rescale_coors(coors)

  cyto <- c_net_update(cyto, initialize = TRUE)
  igraph::graph_attr(cyto, "coors") <- coors
  return(list(go = cyto, coors = coors))
}


#' plot use networkD3
#'
#' @param go metanet
#' @param v_class which attributes use to be v_class
#' @param ... see \code{\link[networkD3]{forceNetwork}}
#' @return D3 plot
#' @export
#' @family plot
#' @examples
#' data("c_net")
#' plot(co_net2)
#' if (requireNamespace("networkD3")) {
#'   netD3plot(co_net2)
#' }
netD3plot <- function(go, v_class = "v_class", ...) {
  flag <- "y"
  if (length(V(go)) > 200) {
    message("Too big network, recommend using Gephi to layout,still use networkD3?")
    flag <- readline("yes/no(y/n):")
  }
  if (tolower(flag) %in% c("yes", "y")) {
    lib_ps("networkD3", library = FALSE)
    go <- c_net_set(go, vertex_class = v_class)
    get_v(go) -> tmp_v
    nodes <- tmp_v[, c("name", "v_class", "size", "color")]
    colnames(nodes) <- c("name", "group", "size", "color")
    nodes$size <- pcutils::mmscale(nodes$size, 2, 40)

    colors <- unique(nodes$color)

    get_e(go) -> tmp_e
    links <- tmp_e[, c("from", "to", "width", "color")]
    links$width <- pcutils::mmscale(links$width, 0.5, 1.5)
    # give ids
    links$IDsource <- match(links$from, nodes$name) - 1
    links$IDtarget <- match(links$to, nodes$name) - 1
    # Create force directed network plot
    networkD3::forceNetwork(
      Links = links, Nodes = nodes,
      Source = "IDsource", Target = "IDtarget", linkColour = links$color, linkDistance = 20,
      linkWidth = networkD3::JS("function(d) { return (d.width); }"), charge = -5,
      NodeID = "name", Group = "group", Nodesize = "size",
      colourScale = networkD3::JS(paste0("d3.scaleOrdinal([`", paste(colors, collapse = "`,`"), "`])")), legend = TRUE, ...
    )
  }
}

MetaNet_theme <- {
  ggplot2::theme_classic(base_size = 13) + ggplot2::theme(
    axis.text = element_text(color = "black"),
    plot.margin = grid::unit(rep(0.5, 4), "lines"),
    strip.background = ggplot2::element_rect(fill = NA)
  )
}

#' Venn network
#'
#' @param tab data.frame (row is elements, column is group), or a list (names is group, value is elements)
#'
#' @return plot
#' @export
#' @family plot
#' @examples
#' data(otutab, package = "pcutils")
#' tab <- otutab[400:485, 1:3]
#' venn_net(tab) -> v_net
#' plot(v_net)
venn_net <- function(tab) {
  # pcutils:::venn_cal(tab)->vennlist
  tab[is.na(tab)] <- 0
  edgelist <- data.frame()
  if (is.data.frame(tab)) {
    groupss <- colnames(tab)
    for (i in groupss) {
      if (sum(tab[, i] > 0) > 0) edgelist <- rbind(edgelist, data.frame(Group = i, elements = rownames(tab)[tab[, i] > 0]))
    }
  } else if (all(class(tab) == "list")) {
    vennlist <- tab
    groupss <- names(vennlist)
    for (i in groupss) {
      if (length(vennlist[[i]] > 0)) edgelist <- rbind(edgelist, data.frame(Group = i, elements = vennlist[[i]]))
    }
  } else {
    stop("wrong input tab")
  }

  nodelist <- rbind(
    data.frame(name = groupss, v_group = "Group", v_class = paste0("Group: ", groupss)),
    data.frame(name = unique(edgelist$elements), v_group = "elements", v_class = "elements")
  )
  venn_net <- c_net_from_edgelist(edgelist, vertex_df = nodelist)
  graph.attributes(venn_net)$n_type <- "venn"
  all_group <- get_e(venn_net)[, c("from", "to")] %>%
    pcutils::squash("from") %>%
    dplyr::rename(name = "to", all_group = "from")
  venn_net <- c_net_set(venn_net, all_group, vertex_class = "all_group", edge_type = "from")
  venn_net
}


#' Quick build a metanet from two columns table
#'
#' @param edgelist two columns table (no elements exist in two columns at same time)
#'
#' @return metanet
#' @export
#' @family plot
#' @examples
#' twocol <- data.frame(
#'   "col1" = sample(letters, 30, replace = TRUE),
#'   "col2" = sample(c("A", "B"), 30, replace = TRUE)
#' )
#' twocol_net <- twocol_edgelist(twocol)
#' plot(twocol_net)
#' c_net_plot(twocol_net, g_layout_polygon(twocol_net))
twocol_edgelist <- function(edgelist) {
  if (any(edgelist[, 1] %in% edgelist[, 2])) stop("Must no elements exist in two columns at same time")
  nodelist <- rbind(
    data.frame(name = unique(edgelist[, 1]), v_group = names(edgelist)[1], v_class = names(edgelist)[1]),
    data.frame(name = unique(edgelist[, 2]), v_group = names(edgelist)[2], v_class = names(edgelist)[2])
  )
  venn_net <- c_net_from_edgelist(edgelist, vertex_df = nodelist)
  graph.attributes(venn_net)$n_type <- "twocol"
  # venn_net=c_net_set(venn_net,edge_type = "from")
  venn_net
}


#' Transform a dataframe to a network edgelist.
#'
#' @param test df
#' @param fun default: sum
#'
#' @return metanet
#' @export
#'
#' @examples
#' data("otutab", package = "pcutils")
#' cbind(taxonomy, num = rowSums(otutab))[1:20, ] -> test
#' df2net_tree(test) -> ttt
#' plot(ttt)
#' if (requireNamespace("ggraph")) plot(ttt, coors = as_circle_tree())
df2net_tree <- function(test, fun = sum) {
  flag <- FALSE
  if (!is.numeric(test[, ncol(test)])) {
    test$num <- 1
  } else {
    flag <- TRUE
    name <- colnames(test)[ncol(test)]
  }
  nc <- ncol(test)
  if (nc < 3) stop("as least 3-columns dataframe")

  link <- pcutils::df2link(test, fun = fun)

  nodes <- link$nodes
  links <- link$links
  if (flag) {
    colnames(links)[3] <- colnames(nodes)[3] <- name
  } else {
    name <- "weight"
  }

  # c_net_from_edgelist(as.data.frame(links),vertex = nodes)
  net <- igraph::graph_from_data_frame(as.data.frame(links), vertices = nodes)
  net <- c_net_set(net, vertex_class = "level", vertex_size = name, edge_width = name)
  net <- c_net_update(net, initialize = TRUE, verbose = FALSE)
  graph_attr(net, "coors") <- c_net_layout(net, as_tree())
  net
}

#' Plot olympic rings using network
#'
#' @return network plot
#' @export
#' @family plot
#' @examples
#' olympic_rings_net()
olympic_rings_net <- function() {
  r <- 1
  pensize <- r / 6
  rings_data <- data.frame(
    x = c(-2 * (r + pensize), -(r + pensize), 0, (r + pensize), 2 * (r + pensize)),
    y = c(r, 0, r, 0, r),
    color = c("#0081C8", "#FCB131", "#000000", "#00A651", "#EE334E")
  )
  g1 <- module_net(module_number = 5, n_node_in_module = 30)
  plot(g1,
    coors = g_layout(g1, layout1 = rings_data[, 1:2], zoom1 = 1.2, zoom2 = 0.5),
    rescale = FALSE, legend = FALSE, main = "Olympic Rings", vertex.frame.color = NA,
    edge.width = 0, vertex.color = setNames(rings_data$color, 1:5), vertex.size = 7
  )
}

# ========3.1.shapes========

add_metanet_shape_diamond <- function() {
  mydiamond <- function(coords, v = NULL, params) {
    vertex.color <- params("vertex", "color")
    if (length(vertex.color) != 1 && !is.null(v)) {
      vertex.color <- vertex.color[v]
    }
    vertex.frame.color <- params("vertex", "frame.color")
    if (length(vertex.frame.color) != 1 && !is.null(v)) {
      vertex.frame.color <- vertex.frame.color[v]
    }
    vertex.frame.width <- params("vertex", "frame.width")
    if (length(vertex.frame.width) != 1 && !is.null(v)) {
      vertex.frame.width <- vertex.frame.width[v]
    }
    vertex.size <- params("vertex", "size")
    if (length(vertex.size) != 1 && !is.null(v)) {
      vertex.size <- vertex.size[v]
    }
    vertex.size <- rep(vertex.size, length.out = nrow(coords))
    vertex.frame.color[vertex.frame.width <= 0] <- NA
    vertex.frame.width[vertex.frame.width <= 0] <- 1
    if (length(vertex.frame.width) == 1) {
      symbols(
        x = coords[, 1], y = coords[, 2], bg = vertex.color,
        fg = vertex.frame.color, stars = cbind(vertex.size, vertex.size, vertex.size, vertex.size),
        lwd = vertex.frame.width, add = TRUE, inches = FALSE
      )
    } else {
      mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
        vertex.size, vertex.frame.width,
        FUN = function(x, y, bg, fg, size, lwd) {
          symbols(
            x = x, y = y, bg = bg, fg = fg, lwd = lwd,
            stars = cbind(size, size, size, size), add = TRUE, inches = FALSE
          )
        }
      )
    }
  }
  igraph::add_shape("diamond", clip = shape_noclip, plot = mydiamond)
}

add_metanet_shape_triangle1 <- function() {
  mytriangle1 <- function(coords, v = NULL, params) {
    vertex.color <- params("vertex", "color")
    if (length(vertex.color) != 1 && !is.null(v)) {
      vertex.color <- vertex.color[v]
    }
    vertex.frame.color <- params("vertex", "frame.color")
    if (length(vertex.frame.color) != 1 && !is.null(v)) {
      vertex.frame.color <- vertex.frame.color[v]
    }
    vertex.frame.width <- params("vertex", "frame.width")
    if (length(vertex.frame.width) != 1 && !is.null(v)) {
      vertex.frame.width <- vertex.frame.width[v]
    }
    vertex.size <- params("vertex", "size")
    if (length(vertex.size) != 1 && !is.null(v)) {
      vertex.size <- vertex.size[v]
    }
    vertex.size <- rep(vertex.size, length.out = nrow(coords))
    vertex.frame.color[vertex.frame.width <= 0] <- NA
    vertex.frame.width[vertex.frame.width <= 0] <- 1
    if (length(vertex.frame.width) == 1) {
      symbols(
        x = coords[, 1], y = coords[, 2] - vertex.size / sqrt(3), bg = vertex.color,
        fg = vertex.frame.color, stars = cbind(vertex.size, vertex.size * sqrt(3), vertex.size, 0),
        lwd = vertex.frame.width, add = TRUE, inches = FALSE
      )
    } else {
      mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
        vertex.size, vertex.frame.width,
        FUN = function(x, y, bg, fg, size, lwd) {
          symbols(
            x = x, y = y - size / sqrt(3), bg = bg, fg = fg, lwd = lwd,
            stars = cbind(size, size * sqrt(3), size, 0), add = TRUE, inches = FALSE
          )
        }
      )
    }
  }
  igraph::add_shape("triangle1", clip = shape_noclip, plot = mytriangle1)
}

add_metanet_shape_triangle2 <- function() {
  mytriangle2 <- function(coords, v = NULL, params) {
    vertex.color <- params("vertex", "color")
    if (length(vertex.color) != 1 && !is.null(v)) {
      vertex.color <- vertex.color[v]
    }
    vertex.frame.color <- params("vertex", "frame.color")
    if (length(vertex.frame.color) != 1 && !is.null(v)) {
      vertex.frame.color <- vertex.frame.color[v]
    }
    vertex.frame.width <- params("vertex", "frame.width")
    if (length(vertex.frame.width) != 1 && !is.null(v)) {
      vertex.frame.width <- vertex.frame.width[v]
    }
    vertex.size <- params("vertex", "size")
    if (length(vertex.size) != 1 && !is.null(v)) {
      vertex.size <- vertex.size[v]
    }
    vertex.size <- rep(vertex.size, length.out = nrow(coords))
    vertex.frame.color[vertex.frame.width <= 0] <- NA
    vertex.frame.width[vertex.frame.width <= 0] <- 1
    if (length(vertex.frame.width) == 1) {
      symbols(
        x = coords[, 1], y = coords[, 2] + vertex.size / sqrt(3), bg = vertex.color,
        fg = vertex.frame.color, stars = cbind(vertex.size, 0, vertex.size, vertex.size * sqrt(3)),
        lwd = vertex.frame.width, add = TRUE, inches = FALSE
      )
    } else {
      mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
        vertex.size, vertex.frame.width,
        FUN = function(x, y, bg, fg, size, lwd) {
          symbols(
            x = x, y = y + size / sqrt(3), bg = bg, fg = fg, lwd = lwd,
            stars = cbind(size, 0, size, size * sqrt(3)), add = TRUE, inches = FALSE
          )
        }
      )
    }
  }
  igraph::add_shape("triangle2", clip = shape_noclip, plot = mytriangle2)
}

add_metanet_shape_star <- function() {
  mystar <- function(coords, v = NULL, params) {
    vertex.color <- params("vertex", "color")
    if (length(vertex.color) != 1 && !is.null(v)) {
      vertex.color <- vertex.color[v]
    }
    vertex.frame.color <- params("vertex", "frame.color")
    if (length(vertex.frame.color) != 1 && !is.null(v)) {
      vertex.frame.color <- vertex.frame.color[v]
    }
    vertex.frame.width <- params("vertex", "frame.width")
    if (length(vertex.frame.width) != 1 && !is.null(v)) {
      vertex.frame.width <- vertex.frame.width[v]
    }
    vertex.size <- params("vertex", "size")
    if (length(vertex.size) != 1 && !is.null(v)) {
      vertex.size <- vertex.size[v]
    }
    norays <- params("vertex", "norays")
    if (is.null(norays)) norays <- 5
    if (length(norays) != 1 && !is.null(v)) {
      norays <- norays[v]
    }

    # # 五角星坐标计算
    # star_coords <- function(x, y, size) {
    #   theta <- seq(0, 2 * pi, length.out = 6)[-6]
    #   outer_x <- x + size * cos(theta)
    #   outer_y <- y + size * sin(theta)
    #   inner_x <- x + size / 2 * cos(theta + pi / 5)
    #   inner_y <- y + size / 2 * sin(theta + pi / 5)
    #   list(x = c(rbind(outer_x, inner_x)), y = c(rbind(outer_y, inner_y)))
    # }

    # 处理边框宽度和颜色
    vertex.frame.color[vertex.frame.width <= 0] <- NA
    vertex.frame.width[vertex.frame.width <= 0] <- 1 # 避免宽度为0

    # 绘制五角星
    # mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
    #   vertex.size, vertex.frame.width,
    #   FUN = function(x, y, bg, fg, size, lwd) {
    #     coords <- star_coords(x, y, size)
    #     polygon(coords$x, coords$y, col = bg, border = fg, lwd = lwd)
    #   }
    # )
    mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
      vertex.size, vertex.frame.width, norays,
      FUN = function(x, y, bg, fg, size, lwd, nor) {
        symbols(
          x = x, y = y, bg = bg, fg = fg, lwd = lwd,
          stars = matrix(c(size, size / 2), nrow = 1, ncol = nor * 2), add = TRUE, inches = FALSE
        )
      }
    )
  }
  igraph::add_shape("star", clip = igraph::shape_noclip, plot = mystar)
}

add_metanet_shape_polygon <- function() {
  mypolygon <- function(coords, v = NULL, params) {
    vertex.color <- params("vertex", "color")
    if (length(vertex.color) != 1 && !is.null(v)) {
      vertex.color <- vertex.color[v]
    }
    vertex.frame.color <- params("vertex", "frame.color")
    if (length(vertex.frame.color) != 1 && !is.null(v)) {
      vertex.frame.color <- vertex.frame.color[v]
    }
    vertex.frame.width <- params("vertex", "frame.width")
    if (length(vertex.frame.width) != 1 && !is.null(v)) {
      vertex.frame.width <- vertex.frame.width[v]
    }
    vertex.size <- params("vertex", "size")
    if (length(vertex.size) != 1 && !is.null(v)) {
      vertex.size <- vertex.size[v]
    }
    norays <- params("vertex", "norays")
    if (is.null(norays)) norays <- 5
    if (length(norays) != 1 && !is.null(v)) {
      norays <- norays[v]
    }

    # hexagon_coords <- function(x, y, size) {
    #   theta <- seq(0, 2 * pi, length.out = 7)[-7] # 6个顶点
    #   x_coords <- x + size * cos(theta)
    #   y_coords <- y + size * sin(theta)
    #   list(x = x_coords, y = y_coords)
    # }

    # 处理边框宽度和颜色
    vertex.frame.color[vertex.frame.width <= 0] <- NA
    vertex.frame.width[vertex.frame.width <= 0] <- 1

    # 批量绘制六边形
    # mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
    #   vertex.size, vertex.frame.width,
    #   FUN = function(x, y, bg, fg, size, lwd) {
    #     coords <- hexagon_coords(x, y, size)
    #     polygon(coords$x, coords$y, col = bg, border = fg, lwd = lwd)
    #   }
    # )
    mapply(coords[, 1], coords[, 2], vertex.color, vertex.frame.color,
      vertex.size, vertex.frame.width, norays,
      FUN = function(x, y, bg, fg, size, lwd, nor) {
        symbols(
          x = x, y = y, bg = bg, fg = fg, lwd = lwd,
          stars = matrix(size, nrow = 1, ncol = nor), add = TRUE, inches = FALSE
        )
      }
    )
  }
  igraph::add_shape("polygon", clip = igraph::shape_noclip, plot = mypolygon)
}

# 因为igraph默认只有circle和square适合展示，所以这里添加了更多的形状

add_metanet_shapes <- function() {
  # ！！！考虑添加更多的形状，至少是21:25，形状为pie时添加额外legend
  for (i in c(names(default_v_shape)[3:5], "star", "polygon")) {
    paste0("add_metanet_shape_", i) -> fun_name
    do.call(fun_name, list())
  }
}

---
title: "MetaNet: Network Analysis for Omics Data"
output: 
  prettydoc::html_pretty:
    toc: true
    toc_depth: 3
    theme: tactile
author: "Chen Peng"
date: "`r Sys.Date()`"
vignette: >
  %\VignetteIndexEntry{MetaNet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Install

```{r eval=FALSE}
if (!requireNamespace("devtools")) install.packages("devtools")
devtools::install_github("Asa12138/pcutils")
devtools::install_github("Asa12138/MetaNet")
```

## Vignette

The HTML documentation of the latest version is available at [Github page](https://asa12138.github.io/MetaNet/).

Please go to <https://asa12138.github.io/MetaNet_tutorial/> for the full vignette.

## Citation

Please cite:

Chen Peng (2023). MetaNet: Network Analysis for Multi-Omics Data. _R package_, <https://github.com/Asa12138/MetaNet>.

## Need helps?

If you have questions/issues, please visit
[MetaNet homepage](https://github.com/Asa12138/MetaNet) first.
Your problems are mostly documented. 

If you think you found a bug, please post on
[github issue](https://github.com/Asa12138/MetaNet/issues).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7.stability.R
\name{Cohesion}
\alias{Cohesion}
\alias{plot.cohesion}
\title{Cohesion calculation}
\usage{
Cohesion(otutab, reps = 200, threads = 1, mycor = NULL, verbose = TRUE)

\method{plot}{cohesion}(x, group, metadata, mode = 1, ...)
}
\arguments{
\item{otutab}{otutab}

\item{reps}{iteration time}

\item{threads}{threads}

\item{mycor}{a correlation matrix you want to use, skip the null model build when mycor is not NULL, default: NULL}

\item{verbose}{verbose}

\item{x}{\code{Cohesion()} result (cohesion object)}

\item{group}{group name in colnames(metadata)}

\item{metadata}{metadata}

\item{mode}{plot mode, 1~2}

\item{...}{additional arguments for \code{\link[pcutils]{group_box}} (mode=1) or \code{\link[pcutils]{group_box}} (mode=2)}
}
\value{
Cohesion object: a list with two dataframe

a ggplot
}
\description{
Cohesion calculation

Plot cohesion
}
\examples{
\donttest{
data("otutab", package = "pcutils")
# set reps at least 99 when you run.
Cohesion(otutab[1:50, ], reps = 19) -> cohesion_res
if (requireNamespace("ggpubr")) {
  plot(cohesion_res, group = "Group", metadata = metadata, mode = 1)
  plot(cohesion_res, group = "Group", metadata = metadata, mode = 2)
}
}
}
\references{
Herren, C. M. & McMahon, K. (2017) Cohesion: a method for quantifying the connectivity of microbial communities. doi:10.1038/ismej.2017.91.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{as_coors}
\alias{as_coors}
\title{Transfer to a coors object}
\usage{
as_coors(coors, curved = NULL)
}
\arguments{
\item{coors}{data.frame}

\item{curved}{line curved}
}
\value{
coors object
}
\description{
Transfer to a coors object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{read_corr}
\alias{read_corr}
\title{Read a corr object}
\usage{
read_corr(filename)
}
\arguments{
\item{filename}{filename of .corr}
}
\value{
a corr object
}
\description{
Read a corr object
}
\seealso{
Other calculate: 
\code{\link{c_net_calculate}()},
\code{\link{cal_sim}()},
\code{\link{fast_cor}()},
\code{\link{p.adjust.table}()}
}
\concept{calculate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{g_layout}
\alias{g_layout}
\title{Layout with group}
\usage{
g_layout(
  go,
  group = "module",
  group_order = NULL,
  layout1 = in_circle(),
  zoom1 = 20,
  layout2 = in_circle(),
  zoom2 = 3,
  show_big_layout = FALSE,
  rescale = TRUE,
  ...
)
}
\arguments{
\item{go}{igraph or metanet object}

\item{group}{group name (default: module)}

\item{group_order}{group_order}

\item{layout1}{layout1 method, one of
(1) a dataframe or matrix: rowname is group, two columns are X and Y
(2) function: layout method for \code{\link{c_net_layout}} default: in_circle()}

\item{zoom1}{big network layout size}

\item{layout2}{one of functions: layout method for \code{\link{c_net_layout}}, or a list of functions.}

\item{zoom2}{average sub_network layout size, or numeric vector, or "auto"}

\item{show_big_layout}{show the big layout to help you adjust.}

\item{rescale}{logical, scale the X, Y to (-1,1)}

\item{...}{add}
}
\value{
coors
}
\description{
Layout with group
}
\examples{
\donttest{
data("c_net")
module_detect(co_net, method = "cluster_fast_greedy") -> co_net_modu
g_layout(co_net_modu, group = "module", zoom1 = 30, zoom2 = "auto", layout2 = as_line()) -> oridata
plot(co_net_modu, coors = oridata)
}
}
\seealso{
Other g_layout: 
\code{\link{g_layout_nice}()},
\code{\link{g_layout_poly_sector}()},
\code{\link{g_layout_polygon}()}
}
\concept{g_layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{cal_sim}
\alias{cal_sim}
\title{Calculate similarity for one t(otutab)}
\usage{
cal_sim(totu, totu2 = NULL, method = "bray")
}
\arguments{
\item{totu}{t(otutab), row are samples, column are features.}

\item{totu2}{t(otutab) or NULL, row are samples, column are features.}

\item{method}{Dissimilarity index, see \code{\link[vegan]{vegdist}}.}
}
\value{
similarity = 1-distance
}
\description{
Calculate similarity for one t(otutab)
}
\examples{
if (requireNamespace("vegan")) {
  data("otutab", package = "pcutils")
  t(otutab) -> totu
  cal_sim(totu) -> sim_corr
}
}
\seealso{
\code{\link[vegan]{vegdist}}

Other calculate: 
\code{\link{c_net_calculate}()},
\code{\link{fast_cor}()},
\code{\link{p.adjust.table}()},
\code{\link{read_corr}()}
}
\concept{calculate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{transform_coors}
\alias{transform_coors}
\title{Transform the layout of a 'coors' object}
\usage{
transform_coors(
  coors,
  scale = 1,
  aspect_ratio = 1,
  rotation = 0,
  mirror_x = FALSE,
  mirror_y = FALSE,
  shear_x = 0,
  shear_y = 0
)
}
\arguments{
\item{coors}{An object of class 'coors', containing node coordinates.}

\item{scale}{A numeric value to scale the layout (default = 1).}

\item{aspect_ratio}{A numeric value to adjust the Y-axis scaling (default = 1).}

\item{rotation}{A numeric value in degrees to rotate the layout (default = 0).}

\item{mirror_x}{A logical value indicating whether to mirror along the X-axis (default = FALSE).}

\item{mirror_y}{A logical value indicating whether to mirror along the Y-axis (default = FALSE).}

\item{shear_x}{A numeric value to apply a shear transformation in the X direction (default = 0).}

\item{shear_y}{A numeric value to apply a shear transformation in the Y direction (default = 0).}
}
\value{
A transformed 'coors' object with updated coordinates.
}
\description{
This function applies various transformations to a 'coors' object, including
scaling, aspect ratio adjustment, rotation, mirroring, and pseudo-3D perspective transformation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{venn_net}
\alias{venn_net}
\title{Venn network}
\usage{
venn_net(tab)
}
\arguments{
\item{tab}{data.frame (row is elements, column is group), or a list (names is group, value is elements)}
}
\value{
plot
}
\description{
Venn network
}
\examples{
data(otutab, package = "pcutils")
tab <- otutab[400:485, 1:3]
venn_net(tab) -> v_net
plot(v_net)
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{c_net_plot}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-3.RMT.R
\name{RMT_threshold}
\alias{RMT_threshold}
\alias{rmt}
\title{Get RMT threshold for a correlation matrix}
\usage{
RMT_threshold(
  occor.r,
  out_dir,
  min_threshold = 0.5,
  max_threshold = 0.8,
  step = 0.02,
  plot = FALSE,
  gif = FALSE,
  verbose = FALSE
)

rmt(occor.r, min_threshold = 0.5, max_threshold = 0.85, step = 0.01)
}
\arguments{
\item{occor.r}{a corr object or a correlation matrix}

\item{out_dir}{output dir}

\item{min_threshold}{min_threshold}

\item{max_threshold}{max_threshold}

\item{step}{step}

\item{plot}{plot pngs?}

\item{gif}{render a .gif file?}

\item{verbose}{verbose}
}
\value{
a r-threshold

recommend threshold
}
\description{
Get RMT threshold for a correlation matrix

Get RMT threshold for a correlation matrix roughly
}
\examples{
\donttest{
data(otutab, package = "pcutils")
t(otutab) -> totu
c_net_calculate(totu) -> corr
rmt(corr)
# recommend: 0.69
c_net_build(corr, r_threshold = 0.69) -> co_net_rmt
}
}
\references{
J. Zhou, Y. Deng, FALSE. Luo, Z. He, Q. Tu, X. Zhi, (2010) Functional Molecular Ecological Networks, doi:10.1128/mBio.00169-10.
\url{https://matstat.org/content_en/RMT/RMThreshold_Intro.pdf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{get_module_eigen}
\alias{get_module_eigen}
\title{Get module_eigen}
\usage{
get_module_eigen(go_m)
}
\arguments{
\item{go_m}{module metanet}
}
\value{
module_eigen
}
\description{
Get module_eigen
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{to_module_net}
\alias{to_module_net}
\title{Transformation a network to a module network}
\usage{
to_module_net(go, edge_type = c("module", "module_from", "module_to")[1])
}
\arguments{
\item{go}{metanet}

\item{edge_type}{"module", "module_from", "module_to"}
}
\value{
metanet with modules
}
\description{
Transformation a network to a module network
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{check_tabs}
\alias{check_tabs}
\title{Check tables and extract common samples}
\usage{
check_tabs(...)
}
\arguments{
\item{...}{tables}
}
\value{
formatted tables
}
\description{
Check tables and extract common samples
}
\examples{
data("otutab", package = "pcutils")
check_tabs(otutab)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-1.build.R
\name{multi_net_build}
\alias{multi_net_build}
\title{Multi-omics network build}
\usage{
multi_net_build(
  ...,
  mode = "full",
  method = "spearman",
  filename = FALSE,
  p.adjust.method = NULL,
  r_threshold = 0.6,
  p_threshold = 0.05,
  use_p_adj = TRUE,
  delete_single = TRUE
)
}
\arguments{
\item{...}{some omics abundance tables}

\item{mode}{"full"}

\item{method}{"spearman" or "pearson"}

\item{filename}{the prefix of saved .corr file or FALSE}

\item{p.adjust.method}{see \code{\link[stats]{p.adjust}}}

\item{r_threshold}{r_threshold (default: >0.6)}

\item{p_threshold}{p_threshold (default: <0.05)}

\item{use_p_adj}{use the p.adjust instead of p-value (default: TRUE)}

\item{delete_single}{should delete single vertexes?}
}
\value{
metanet
}
\description{
Multi-omics network build
}
\examples{
data("multi_test")
multi1 <- multi_net_build(list(Microbiome = micro, Metabolome = metab, Transcriptome = transc))
multi1 <- c_net_set(multi1, micro_g, metab_g, transc_g,
  vertex_class = c("Phylum", "kingdom", "type")
)
multi1 <- c_net_set(multi1, data.frame("Abundance1" = colSums(micro)),
  data.frame("Abundance2" = colSums(metab)), data.frame("Abundance3" = colSums(transc)),
  vertex_size = paste0("Abundance", 1:3)
)
c_net_plot(multi1)
}
\seealso{
Other build: 
\code{\link{c_net_build}()},
\code{\link{c_net_from_edgelist}()},
\code{\link{c_net_set}()},
\code{\link{c_net_update}()}
}
\concept{build}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{plot_e_type_bar}
\alias{plot_e_type_bar}
\title{Plot e_type bar}
\usage{
plot_e_type_bar(go, mode = c("left", "right")[1], degree_threshold = 0)
}
\arguments{
\item{go}{metanet object}

\item{mode}{"left" or "right"}

\item{degree_threshold}{degree threshold}
}
\value{
ggplot
}
\description{
Plot e_type bar
}
\examples{
data("c_net")
plot_e_type_bar(co_net, degree_threshold = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{save_corr}
\alias{save_corr}
\title{Save a corr object}
\usage{
save_corr(corr, filename = "corr")
}
\arguments{
\item{corr}{a corr object}

\item{filename}{filename without extension, default: "corr"}
}
\value{
a .corr file
}
\description{
Save a corr object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{twocol_edgelist}
\alias{twocol_edgelist}
\title{Quick build a metanet from two columns table}
\usage{
twocol_edgelist(edgelist)
}
\arguments{
\item{edgelist}{two columns table (no elements exist in two columns at same time)}
}
\value{
metanet
}
\description{
Quick build a metanet from two columns table
}
\examples{
twocol <- data.frame(
  "col1" = sample(letters, 30, replace = TRUE),
  "col2" = sample(c("A", "B"), 30, replace = TRUE)
)
twocol_net <- twocol_edgelist(twocol)
plot(twocol_net)
c_net_plot(twocol_net, g_layout_polygon(twocol_net))
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{c_net_plot}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{input_cytoscape}
\alias{input_cytoscape}
\title{Input a cyjs file exported by Cytoscape}
\usage{
input_cytoscape(file)
}
\arguments{
\item{file}{cyjs file exported by Cytoscape}
}
\value{
list contains the igraph object and coordinates
}
\description{
Input a cyjs file exported by Cytoscape
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{c_net_plot}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7.stability.R
\name{c_net_stability}
\alias{c_net_stability}
\alias{robust_test}
\alias{vulnerability}
\alias{robustness}
\title{Evaluate the stability of a network}
\usage{
c_net_stability(
  go_ls,
  mode = "robust_test",
  partial = 0.5,
  step = 10,
  reps = 9,
  threads = 1,
  verbose = TRUE,
  keystone = FALSE
)

robust_test(
  go_ls,
  partial = 0.5,
  step = 10,
  reps = 9,
  threads = 1,
  verbose = TRUE
)

vulnerability(go_ls, threads = 1, verbose = TRUE)

robustness(go_ls, keystone = FALSE, reps = 9, threads = 1, verbose = TRUE)
}
\arguments{
\item{go_ls}{an igraph object or igraph list.}

\item{mode}{"robust_test", "vulnerability", "robustness"}

\item{partial}{how much percent vertexes be removed in total (default: 0.5, only for robust_test)}

\item{step}{how many nodes be removed each time? (default: 10, only for robust_test)}

\item{reps}{simulation number (default: 9)}

\item{threads}{threads}

\item{verbose}{verbose}

\item{keystone}{remove 70\%\% keystones instead of remove 50\%\% nodes (default: False, only for robustness)}
}
\value{
a data.frame

data.frame (robustness class)

a vector
}
\description{
\deqn{Vi=\frac{E-Ei}{E}}
E is the global efficiency and Ei is the global efficiency after the removal of the node i and its entire links.
}
\examples{
\donttest{
data("c_net")
if (requireNamespace("ggpmisc")) {
  c_net_stability(co_net, mode = "robust_test", step = 20, reps = 9) -> robust_res
  plot(robust_res, index = "Average_degree", mode = 2)
}

c_net_stability(co_net, mode = "vulnerability") -> vulnerability_res
plot(vulnerability_res)

robustness(co_net) -> robustness_res
plot(robustness_res)

module_detect(co_net) -> co_net_modu
zp_analyse(co_net_modu, mode = 2) -> co_net_modu

c_net_stability(co_net_modu, mode = "robustness", keystone = TRUE) -> robustness_res
plot(robustness_res)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{summ_2col}
\alias{summ_2col}
\title{Summaries two columns information}
\usage{
summ_2col(df, from = 1, to = 2, count = 3, direct = FALSE)
}
\arguments{
\item{df}{data.frame}

\item{from}{first column name or index}

\item{to}{second column name or index}

\item{count}{(optional) weight column, if no, each equal to 1}

\item{direct}{consider direct? default: FALSE}
}
\value{
data.frame
}
\description{
Summaries two columns information
}
\examples{
test <- data.frame(
  a = sample(letters[1:4], 10, replace = TRUE),
  b = sample(letters[1:4], 10, replace = TRUE)
)
summ_2col(test, direct = TRUE)
summ_2col(test, direct = FALSE)
if (requireNamespace("circlize")) {
  summ_2col(test, direct = TRUE) \%>\% pcutils::my_circo()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{zp_analyse}
\alias{zp_analyse}
\alias{zp_plot}
\title{Zi-Pi calculate}
\usage{
zp_analyse(go_m, mode = 2, use_origin = TRUE)

zp_plot(go, label = TRUE, mode = 1)
}
\arguments{
\item{go_m}{igraph object after \code{module_detect()}}

\item{mode}{plot style, 1~3}

\item{use_origin}{use original_module, default:TRUE, if FALSE, use module}

\item{go}{igraph object after zp_analyse()}

\item{label}{show label or not}
}
\value{
igraph

a ggplot object
}
\description{
Zi-Pi calculate

Zi-Pi plot of vertexes
}
\examples{
data("c_net")
module_detect(co_net) -> co_net_modu
zp_analyse(co_net_modu) -> co_net_modu
zp_plot(co_net_modu)
zp_plot(co_net_modu, mode = 3)

}
\references{
\enumerate{
\item Guimerà, R. & Amaral, L. Functional cartography of complex metabolic networks. (2005) doi:10.1038/nature03288.
}
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_filter}
\alias{c_net_filter}
\title{Filter a network according to some attributes}
\usage{
c_net_filter(go, ..., mode = "v")
}
\arguments{
\item{go}{metanet object}

\item{...}{some attributes of vertex and edge}

\item{mode}{"v" or "e"}
}
\value{
metanet
}
\description{
Filter a network according to some attributes
}
\examples{
data("multi_net")
c_net_filter(multi1, v_group \%in\% c("omic1", "omic2"))
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_annotate}
\alias{c_net_annotate}
\title{Annotate a metanet}
\usage{
c_net_annotate(go, anno_tab, mode = "v", verbose = TRUE)
}
\arguments{
\item{go}{metanet object}

\item{anno_tab}{a dataframe using to annotate (mode v, e), or a list (mode n)}

\item{mode}{"v" for vertex, "e" for edge, "n" for network}

\item{verbose}{logical}
}
\value{
a annotated metanet object
}
\description{
Annotate a metanet
}
\examples{
data("c_net")
anno <- data.frame("name" = "s__Pelomonas_puraquae", new_atr = "new")
co_net_new <- c_net_annotate(co_net, anno, mode = "v")
get_v(co_net_new, c("name", "new_atr"))

anno <- data.frame("from" = "s__Pelomonas_puraquae", "to" = "s__un_g__Rhizobium", new_atr = "new")
co_net_new <- c_net_annotate(co_net, anno, mode = "e")
get_e(co_net_new, c("from", "to", "new_atr"))

co_net_new <- c_net_annotate(co_net, list(new_atr = "new"), mode = "n")
get_n(co_net_new)
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{module_detect}
\alias{module_detect}
\alias{c_net_module}
\title{Detect the modules}
\usage{
module_detect(
  go,
  method = "cluster_fast_greedy",
  n_node_in_module = 0,
  delete = FALSE
)
}
\arguments{
\item{go}{an igraph object}

\item{method}{cluster_method: "cluster_walktrap", "cluster_edge_betweenness", "cluster_fast_greedy", "cluster_spinglass"}

\item{n_node_in_module}{transfer the modules less than n_node_in_module to "others"}

\item{delete}{logical, delete others modules? default:FALSE, the others module will be "others".}
}
\value{
an igraph object
}
\description{
Detect the modules
}
\examples{
data("c_net")
module_detect(co_net) -> co_net_modu
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{spatstat_layout}
\alias{spatstat_layout}
\title{Generate spatial layout using \code{spatstat}}
\usage{
spatstat_layout(
  go,
  win,
  type = c("random", "regular"),
  mode = c("surface", "boundary"),
  jitter = 0,
  curved = NULL,
  order_by = NULL,
  order_ls = NULL,
  order_circle = FALSE,
  seed = 1234,
  rescale = TRUE
)
}
\arguments{
\item{go}{igraph or metanet object}

\item{win}{A spatstat window object (owin), e.g. disc(), owin(poly=...); Or sf object.}

\item{type}{Type of distribution: "random", "regular"}

\item{mode}{"surface", "boundary"}

\item{jitter}{for surface-regular, defalut 0}

\item{curved}{Optional curved attribute for coors}

\item{order_by}{order nodes according to a node attribute}

\item{order_ls}{manual the discrete variable with a vector, or continuous variable with "desc" to decreasing}

\item{order_circle}{order nodes from the center of a circle}

\item{seed}{random seed}

\item{rescale}{rescale the coordinates to (0,1)}
}
\value{
A coors object (data.frame with class "coors" and attribute "curved")
}
\description{
Generate spatial layout using \code{spatstat}
}
\examples{
\donttest{
if (requireNamespace("spatstat.geom") && requireNamespace("spatstat.random")) {
  poly_x <- c(0, 2, 2, 0)
  poly_y <- c(0, 0, 1, 1)
  win_poly <- spatstat.geom::owin(poly = list(x = poly_x, y = poly_y))
  plot(win_poly)
  coors1 <- spatstat_layout(co_net, win_poly, type = "regular", mode = "surface")
  plot(co_net, coors = coors1)
  coors2 <- spatstat_layout(co_net2, win_poly, type = "random", mode = "boundary")
  plot(co_net2, coors = coors2)

  if (requireNamespace("sf")) {
    nc <- sf::st_read(system.file("shape/nc.shp", package = "sf"), quiet = TRUE)
    poly <- nc[1, ]
    coors <- spatstat_layout(go = multi1, win = poly, type = "regular", mode = "surface")
    plot(multi1, coors = coors)
  }
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{rand_net_par}
\alias{rand_net_par}
\alias{compare_rand}
\title{Net_pars of many random network}
\usage{
rand_net_par(go, reps = 99, threads = 1, verbose = TRUE)

compare_rand(
  pars,
  randp,
  index = c("Average_path_length", "Clustering_coefficient")
)
}
\arguments{
\item{go}{igraph}

\item{reps}{simulation time}

\item{threads}{threads}

\item{verbose}{verbose}

\item{pars}{your net pars resulted by net_pars()}

\item{randp}{random networks pars resulted by rand_net_par()}

\item{index}{compared indexes: "Average_path_length","Clustering_coefficient" or else}
}
\value{
ggplot
}
\description{
Net_pars of many random network

Compare some indexes between your net with random networks
}
\examples{
data("c_net")
rand_net_par(co_net_rmt, reps = 30) -> randp
net_par(co_net_rmt, fast = FALSE) -> pars
compare_rand(pars, randp)
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{c_net_compare}
\alias{c_net_compare}
\title{Compare Two Networks}
\usage{
c_net_compare(g1, g2)
}
\arguments{
\item{g1}{network1}

\item{g2}{network2}
}
\value{
A list containing the following elements:
\itemize{
\item \code{g1}: The first network.
\item \code{g2}: The second network.
\item \code{g_union}: The union of the two networks.
\item \code{g_inter}: The intersection of the two networks.
\item \code{net_par_df}: A data frame containing the network parameters.
\item \code{net_similarity}: A list containing the similarity metrics.
}
}
\description{
Compare Two Networks
}
\examples{
data("c_net")
set.seed(12)
co_net_p1 <- c_net_filter(co_net, name \%in\% sample(V(co_net)$name, 300))
co_net_p2 <- c_net_filter(co_net, name \%in\% sample(V(co_net)$name, 300))
c_net_compare(co_net_p1, co_net_p2) -> c_net_comp
plot(c_net_comp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-1.build.R
\name{clean_igraph}
\alias{clean_igraph}
\title{Clean a igraph object}
\usage{
clean_igraph(go, direct = NULL)
}
\arguments{
\item{go}{igraph, metanet objects}

\item{direct}{direct?}
}
\value{
a igraph object
}
\description{
Clean a igraph object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{g_layout_poly_sector}
\alias{g_layout_poly_sector}
\title{Layout with group}
\usage{
g_layout_poly_sector(
  go,
  group = "v_group",
  group_order = NULL,
  group2 = NULL,
  group2_order = NULL,
  space = pi/4,
  jitter = 0,
  scale_node_num = TRUE,
  type = c("regular", "random"),
  mode = c("surface", "boundary"),
  curved = NULL
)
}
\arguments{
\item{go}{igraph}

\item{group}{group name (default:v_group)}

\item{group_order}{group_order}

\item{group2}{group2 name, will order nodes in each group according to group2_order}

\item{group2_order}{group2_order}

\item{space}{the space between each arc, default: pi/4}

\item{jitter}{for surface-regular, defalut 0}

\item{scale_node_num}{scale with the node number in each group}

\item{type}{Type of distribution: "random", "regular"}

\item{mode}{"surface", "boundary"}

\item{curved}{Optional curved attribute for coors}
}
\value{
coors
}
\description{
Layout with group
}
\seealso{
Other g_layout: 
\code{\link{g_layout}()},
\code{\link{g_layout_nice}()},
\code{\link{g_layout_polygon}()}
}
\concept{g_layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7.stability.R
\name{plot.robust}
\alias{plot.robust}
\title{Plot robust}
\usage{
\method{plot}{robust}(
  x,
  indexes = c("Natural_connectivity", "Average_degree"),
  use_ratio = FALSE,
  mode = 1,
  ...
)
}
\arguments{
\item{x}{\code{robust_test()} result (robust object)}

\item{indexes}{indexes selected to show}

\item{use_ratio}{use the delete nodes ratio rather than nodes number}

\item{mode}{plot mode, 1~3}

\item{...}{additional arguments for \code{\link[pcutils]{group_box}}}
}
\value{
a ggplot
}
\description{
Plot robust
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{plot.metanet_compare}
\alias{plot.metanet_compare}
\title{Plot a metanet_compare}
\usage{
\method{plot}{metanet_compare}(x, coors_com = NULL, mains = NULL, ...)
}
\arguments{
\item{x}{metanet_compare object}

\item{coors_com}{coors object}

\item{mains}{a vector of two strings for the main titles of the two networks}

\item{...}{add}
}
\value{
plot
}
\description{
Plot a metanet_compare
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{get_group_skeleton}
\alias{get_group_skeleton}
\alias{c_net_skeleton}
\alias{skeleton_plot}
\title{Get skeleton network according to a group}
\usage{
get_group_skeleton(go, Group = "v_class", count = NULL, top_N = 8)

skeleton_plot(ske_net, split_e_type = TRUE, ...)
}
\arguments{
\item{go}{network}

\item{Group}{vertex column name}

\item{count}{take which column count, default: NULL}

\item{top_N}{top_N}

\item{ske_net}{skeleton}

\item{split_e_type}{split by e_type? default: TRUE}

\item{...}{additional parameters for \code{\link[igraph]{igraph.plotting}}}
}
\value{
skeleton network
}
\description{
Get skeleton network according to a group

Skeleton plot
}
\examples{
get_group_skeleton(co_net) -> ske_net
skeleton_plot(ske_net)
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-3.plot.R
\name{c_net_plot}
\alias{c_net_plot}
\title{Plot a metanet}
\usage{
c_net_plot(
  go,
  coors = NULL,
  ...,
  labels_num = NULL,
  vertex_size_range = NULL,
  edge_width_range = NULL,
  plot_module = FALSE,
  mark_module = FALSE,
  mark_color = NULL,
  mark_alpha = 0.3,
  module_label = FALSE,
  module_label_cex = 2,
  module_label_color = "black",
  module_label_just = c(0.5, 0.5),
  pie_value = NULL,
  pie_color = NULL,
  legend = TRUE,
  legend_number = FALSE,
  legend_cex = 1,
  legend_position = c(left_leg_x = -2, left_leg_y = 1, right_leg_x = 1.2, right_leg_y =
    1),
  group_legend_title = NULL,
  group_legend_order = NULL,
  color_legend = TRUE,
  color_legend_order = NULL,
  size_legend = FALSE,
  size_legend_title = "Node Size",
  edge_legend = TRUE,
  edge_legend_title = "Edge type",
  edge_legend_order = NULL,
  width_legend = FALSE,
  width_legend_title = "Edge width",
  lty_legend = FALSE,
  lty_legend_title = "Edge class",
  lty_legend_order = NULL,
  module_legend = FALSE,
  module_legend_title = "Module",
  module_legend_order = NULL,
  pie_legend = FALSE,
  pie_legend_title = "Pie part",
  pie_legend_order = NULL,
  label_cex = 1,
  arrow_size_cex = 1,
  arrow_width_cex = 1,
  params_list = NULL,
  rescale = FALSE,
  seed = 1234
)
}
\arguments{
\item{go}{an igraph or metanet object}

\item{coors}{the coordinates you saved}

\item{...}{additional parameters for \code{\link[igraph]{igraph.plotting}}}

\item{labels_num}{show how many labels, >1 indicates number, <1 indicates fraction, "all" indicates all.}

\item{vertex_size_range}{the vertex size range, e.g. c(1,10)}

\item{edge_width_range}{the edge width range, e.g. c(1,10)}

\item{plot_module}{logical, plot module?}

\item{mark_module}{logical, mark the modules?}

\item{mark_color}{mark color}

\item{mark_alpha}{mark fill alpha, default 0.3}

\item{module_label}{show module label?}

\item{module_label_cex}{module label cex}

\item{module_label_color}{module label color}

\item{module_label_just}{module label just, default c(0.5,0.5)}

\item{pie_value}{a dataframe using to plot pie (with rowname or a "name" column)}

\item{pie_color}{color vector}

\item{legend}{all legends}

\item{legend_number}{legend with numbers}

\item{legend_cex}{character expansion factor relative to current par("cex"), default: 1}

\item{legend_position}{legend_position, default: c(left_leg_x=-1.9,left_leg_y=1,right_leg_x=1.2,right_leg_y=1)}

\item{group_legend_title}{group_legend_title, length must same to the numbers of v_group}

\item{group_legend_order}{group_legend_order vector}

\item{color_legend}{logical}

\item{color_legend_order}{color_legend_order vector}

\item{size_legend}{logical}

\item{size_legend_title}{size_legend_title}

\item{edge_legend}{logical}

\item{edge_legend_title}{edge_legend_title}

\item{edge_legend_order}{edge_legend_order vector, e.g. c("positive","negative")}

\item{width_legend}{logical}

\item{width_legend_title}{width_legend_title}

\item{lty_legend}{logical}

\item{lty_legend_title}{lty_legend_title}

\item{lty_legend_order}{lty_legend_order}

\item{module_legend}{logical}

\item{module_legend_title}{module_legend_title}

\item{module_legend_order}{module_legend_order}

\item{pie_legend}{logical}

\item{pie_legend_title}{pie_legend_title}

\item{pie_legend_order}{pie_legend_order}

\item{label_cex}{label cex, default 1, relative to the vertex size}

\item{arrow_size_cex}{arrow size cex, default 1, relative to the vertex size}

\item{arrow_width_cex}{arrow width cex, default 1, relative to the vertex size}

\item{params_list}{a list of parameters, e.g. list(edge_legend = TRUE, lty_legend = FALSE), when the parameter is duplicated, the format argument will be used rather than the argument in params_list.}

\item{rescale}{Logical constant, whether to rescale the coordinates to the (-1,1).}

\item{seed}{random seed, default:1234, make sure each plot is the same.}
}
\value{
a network plot
}
\description{
Plot a metanet
}
\examples{
data("c_net")
c_net_plot(co_net)
c_net_plot(co_net2)
c_net_plot(multi1)
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{is_metanet}
\alias{is_metanet}
\alias{is.metanet}
\title{Is this object a metanet object?}
\usage{
is_metanet(go)
}
\arguments{
\item{go}{a test object}
}
\value{
logical
}
\description{
Is this object a metanet object?
}
\examples{
data(c_net)
is_metanet(co_net)
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_union}
\alias{c_net_union}
\title{Union two networks}
\usage{
c_net_union(go1, go2, ...)
}
\arguments{
\item{go1}{metanet object}

\item{go2}{metanet object}

\item{...}{add}
}
\value{
metanet
}
\description{
Union two networks
}
\examples{
g1 <- make_graph(edges = c("1", 2, 2, 3, 3, 4, 4, 5, 5, 1), directed = FALSE) \%>\% as.metanet()
g2 <- make_graph(edges = c("4", 5, 5, 6, 6, 7, 7, 8, 8, 4), directed = FALSE) \%>\% as.metanet()
par(mfrow = c(1, 3))
plot(c_net_union(g1, g2))
plot(c_net_intersect(g1, g2))
plot(c_net_difference(g1, g2))
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-3.plot.R
\name{plot.metanet}
\alias{plot.metanet}
\title{Plot a metanet}
\usage{
\method{plot}{metanet}(x, ...)
}
\arguments{
\item{x}{metanet object}

\item{...}{add}
}
\value{
plot
}
\description{
Plot a metanet
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{c_net_highlight}
\alias{c_net_highlight}
\title{Highlight specific nodes in a network}
\usage{
c_net_highlight(graph, nodes = NULL, edges = NULL, gray_color = "gray80")
}
\arguments{
\item{graph}{An igraph/metatnet object}

\item{nodes}{Vector of node names to highlight}

\item{edges}{a data.frame of edges to highlight, colnames must be "from" and "to"}

\item{gray_color}{Color for non-highlighted nodes (default: "gray80")}
}
\value{
metanet
}
\description{
Adds highlight markers to specified nodes and grays out non-highlighted nodes.
Preserves all existing vertex/edge attributes and class structure.
}
\examples{
par(mfrow = c(1, 3))
nodes <- c("s__Kribbella_catacumbae", "s__Verrucosispora_andamanensis")
nodes <- V(c_net_ego(co_net, nodes))$name
g_hl <- c_net_highlight(co_net, nodes = nodes)
plot(g_hl) # Highlighted nodes keep colors, others turn gray
get_e(co_net) \%>\% head(20) -> hl_edges
g_hl2 <- c_net_highlight(co_net, edges = hl_edges[, 2:3])
c_net_plot(g_hl2)
g_hl3 <- c_net_highlight(co_net, nodes = nodes, edges = hl_edges[, 2:3])
c_net_plot(g_hl3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{links_stat}
\alias{links_stat}
\title{Link summary of the network}
\usage{
links_stat(
  go,
  group = "v_class",
  e_type = "all",
  topN = 10,
  colors = NULL,
  mode = 1,
  plot_param = list()
)
}
\arguments{
\item{go}{igraph or metanet}

\item{group}{summary which group of vertex attribution in names(vertex_attr(go))}

\item{e_type}{"positive", "negative", "all"}

\item{topN}{topN of group, default: 10}

\item{colors}{colors}

\item{mode}{1~2}

\item{plot_param}{plot parameters}
}
\value{
plot
}
\description{
Link summary of the network
}
\examples{
if (requireNamespace("circlize")) {
  links_stat(co_net, topN = 10)
  module_detect(co_net) -> co_net_modu
  links_stat(co_net_modu, group = "module")
}
if (requireNamespace("corrplot")) {
  links_stat(co_net, topN = 10, mode = 2)
}
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{c_net_calculate}
\alias{c_net_calculate}
\alias{c_net_cal}
\title{Calculate correlation for one or two t(otutab), or distance for one t(otutab).}
\usage{
c_net_calculate(
  totu,
  totu2 = NULL,
  method = "spearman",
  filename = FALSE,
  p.adjust.method = NULL,
  p.adjust.mode = "all",
  threads = 1,
  verbose = TRUE
)
}
\arguments{
\item{totu}{t(otutab), row are samples, column are features.}

\item{totu2}{t(otutab2) or NULL, row are samples, column are features.}

\item{method}{"spearman" (default), "pearson", "sparcc", or distance index from \code{\link[vegan]{vegdist}}.}

\item{filename}{the prefix of saved .corr file or FALSE.}

\item{p.adjust.method}{see \code{\link[stats]{p.adjust}}}

\item{p.adjust.mode}{see \code{\link{p.adjust.table}}}

\item{threads}{threads, default: 1.}

\item{verbose}{verbose, default: TRUE.}
}
\value{
a corr object with 3 elements:
\item{r}{default: spearman correlation}
\item{p.value}{default: p-value of spearman correlation}
\item{p.adjust}{default p.adjust.method = NULL}
}
\description{
Calculate correlation for one or two t(otutab), or distance for one t(otutab).
}
\examples{
data("otutab", package = "pcutils")
t(otutab) -> totu
c_net_calculate(totu) -> corr
metadata[, 3:10] -> env
c_net_calculate(totu, env) -> corr2
}
\seealso{
Other calculate: 
\code{\link{cal_sim}()},
\code{\link{fast_cor}()},
\code{\link{p.adjust.table}()},
\code{\link{read_corr}()}
}
\concept{calculate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{clean_multi_edge_metanet}
\alias{clean_multi_edge_metanet}
\title{Clean multi edge metanet to plot}
\usage{
clean_multi_edge_metanet(go)
}
\arguments{
\item{go}{metanet object}
}
\value{
metanet object
}
\description{
Clean multi edge metanet to plot
}
\examples{
g <- igraph::make_ring(2)
g <- igraph::add_edges(g, c(1, 1, 1, 1, 2, 1))
plot(g)
plot(clean_multi_edge_metanet(g))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_load}
\alias{c_net_load}
\title{Load network file}
\usage{
c_net_load(filename, format = "data.frame")
}
\arguments{
\item{filename}{filename}

\item{format}{"data.frame","graphml"}
}
\value{
metanet
}
\description{
Load network file
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{fast_cor}
\alias{fast_cor}
\title{Fast correlation calculation}
\usage{
fast_cor(totu, totu2 = NULL, method = c("pearson", "spearman"))
}
\arguments{
\item{totu}{t(otutab), row are samples, column are features.}

\item{totu2}{t(otutab) or NULL, row are samples, column are features.}

\item{method}{"spearman" or "pearson"}
}
\value{
a list with 2 elements:
\item{r}{default: spearman correlation}
\item{p.value}{default: p-value of spearman correlation}
}
\description{
Fast correlation calculation
}
\examples{
data("otutab", package = "pcutils")
t(otutab[1:100, ]) -> totu
fast_cor(totu, method = "spearman") -> corr
}
\seealso{
Other calculate: 
\code{\link{c_net_calculate}()},
\code{\link{cal_sim}()},
\code{\link{p.adjust.table}()},
\code{\link{read_corr}()}
}
\concept{calculate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{as_polycircle}
\alias{as_polycircle}
\title{Layout as a polycircle}
\usage{
as_polycircle(n = 5)
}
\arguments{
\item{n}{how many circles of this polycircle}
}
\value{
A two-column matrix, each row giving the coordinates of a vertex, according to the ids of the vertex ids.
}
\description{
Layout as a polycircle
}
\examples{
as_polycircle()(co_net)
}
\seealso{
Other layout: 
\code{\link{as_arc}()},
\code{\link{as_circle_tree}()},
\code{\link{as_line}()},
\code{\link{as_multi_layer}()},
\code{\link{as_poly_sector}()},
\code{\link{as_polyarc}()},
\code{\link{as_polygon}()},
\code{\link{c_net_layout}()}
}
\concept{layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{c_net_layout}
\alias{c_net_layout}
\alias{c_net_lay}
\title{Layout coordinates}
\usage{
c_net_layout(
  go,
  method = igraph::nicely(),
  order_by = NULL,
  order_ls = NULL,
  seed = 1234,
  line_curved = 0.5,
  rescale = TRUE,
  ...
)
}
\arguments{
\item{go}{igraph or metanet}

\item{method}{(1) as_line(), as_arc(), as_polygon(), as_polyarc(), as_polycircle(), as_circle_tree();
(2) as_star(), as_tree(), in_circle(), nicely(), on_grid(), on_sphere(),randomly(), with_dh(), with_fr(), with_gem(), with_graphopt(), with_kk(),with_lgl(), with_mds(),. see \code{\link[igraph]{layout_}};
(3) a character, "auto","backbone","centrality","circlepack","dendrogram",
"eigen","focus","hive","igraph","linear","manual","matrix",
"partition","pmds","stress","treemap","unrooted". see \code{\link[ggraph]{create_layout}}}

\item{order_by}{order nodes according to a node attribute}

\item{order_ls}{manual the discrete variable with a vector, or continuous variable with "desc" to decreasing}

\item{seed}{random seed}

\item{line_curved}{consider line curved, only for some layout methods like as_line(), as_polygon().default:0}

\item{rescale}{logical, scale the X, Y to (-1,1)}

\item{...}{add}
}
\value{
coors object: coordinates for nodes, columns: name, X, Y; curved for edges, columns: from, to, curved;
}
\description{
Layout coordinates
}
\examples{
library(igraph)
c_net_layout(co_net) -> coors
c_net_plot(co_net, coors)
c_net_plot(co_net, c_net_layout(co_net, in_circle()), vertex.size = 2)
c_net_plot(co_net, c_net_layout(co_net, in_circle(), order_by = "v_class"), vertex.size = 2)
c_net_plot(co_net, c_net_layout(co_net, in_circle(), order_by = "size", order_ls = "desc"))
c_net_plot(co_net, c_net_layout(co_net, as_polygon(3)))
}
\seealso{
Other layout: 
\code{\link{as_arc}()},
\code{\link{as_circle_tree}()},
\code{\link{as_line}()},
\code{\link{as_multi_layer}()},
\code{\link{as_poly_sector}()},
\code{\link{as_polyarc}()},
\code{\link{as_polycircle}()},
\code{\link{as_polygon}()}
}
\concept{layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{filter_n_module}
\alias{filter_n_module}
\alias{combine_n_module}
\alias{plot_module_tree}
\title{Filter some modules as others}
\usage{
filter_n_module(go_m, n_node_in_module = 0, keep_id = NULL, delete = FALSE)

combine_n_module(go_m, module_number = 5)

plot_module_tree(go_m, module = "module", community = NULL, label.size = 2)
}
\arguments{
\item{go_m}{module metanet}

\item{n_node_in_module}{transfer the modules less than n_node_in_module to "others"}

\item{keep_id}{keep modules ids, will not be "others"}

\item{delete}{logical, delete others modules? default:FALSE, the others module will be "others".}

\item{module_number}{number of modules}

\item{module}{which column name is module. default: "module"}

\item{community}{community object, default: NULL, use the community of go_m}

\item{label.size}{label.size}
}
\value{
metanet with modules

ggplot
}
\description{
Filter some modules as others

Combine or cut modules to module_number

Plot module tree
}
\examples{
data("c_net")
module_detect(co_net) -> co_net_modu
filter_n_module(co_net_modu, n_node_in_module = 30) -> co_net_modu
if (requireNamespace("ggtree") && requireNamespace("treeio")) plot_module_tree(co_net_modu)
combine_n_module(co_net_modu, 20) -> co_net_modu1
if (requireNamespace("ggtree") && requireNamespace("treeio")) plot_module_tree(co_net_modu1)
}
\seealso{
Other module: 
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.ggig}
\alias{print.ggig}
\title{Print method for 'ggig' objects}
\usage{
\method{print}{ggig}(x, ...)
}
\arguments{
\item{x}{'ggig' object}

\item{...}{Additional arguments}
}
\value{
No value
}
\description{
Print method for 'ggig' objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{summary.corr}
\alias{summary.corr}
\title{Summary method for 'corr' objects}
\usage{
\method{summary}{corr}(object, ...)
}
\arguments{
\item{object}{'corr' object}

\item{...}{Additional arguments}
}
\value{
No value
}
\description{
Summary method for 'corr' objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{net_par}
\alias{net_par}
\alias{c_net_index}
\title{Calculate all topological indexes of a network}
\usage{
net_par(
  go,
  mode = c("v", "e", "n", "all"),
  fast = TRUE,
  remove_negative = FALSE,
  only_topological = FALSE
)

c_net_index(go, force = FALSE)
}
\arguments{
\item{go}{igraph or metanet}

\item{mode}{calculate what? c("v", "e", "n", "all")}

\item{fast}{less indexes for faster calculate ?}

\item{remove_negative}{remove negative edge or not? default: FALSE}

\item{only_topological}{only return topological indexes}

\item{force}{replace existed net_par}
}
\value{
a 3-elements list
\item{n_index}{indexs of the whole network}
\item{v_index}{indexs of each vertex}
\item{e_index}{indexs of each edge}
}
\description{
Calculate all topological indexes of a network

Add topological indexes for a network
}
\examples{
igraph::make_graph("Walther") \%>\% net_par()
c_net_index(co_net) -> co_net_with_par
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{plot_net_degree}
\alias{plot_net_degree}
\title{Plot degree distribution of networks}
\usage{
plot_net_degree(gols, net_names = NULL)
}
\arguments{
\item{gols}{list of metanet}

\item{net_names}{names of networks}
}
\value{
ggplot
}
\description{
Plot degree distribution of networks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{arc_count}
\alias{arc_count}
\title{Edgelist}
\description{
Edgelist for \code{c_net_from_edgelist()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.metanet}
\alias{print.metanet}
\title{Print method for 'metanet' objects}
\usage{
\method{print}{metanet}(x, ...)
}
\arguments{
\item{x}{'metanet' object}

\item{...}{Additional arguments}
}
\value{
No value
}
\description{
Print method for 'metanet' objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{c_net_ego}
\alias{c_net_ego}
\alias{c_net_neighbors}
\title{Extract ego-centric subnetwork with preserved class attributes}
\usage{
c_net_ego(graph, nodes, order = 1, mode = "all")
}
\arguments{
\item{graph}{An igraph object with potential "metanet" class}

\item{nodes}{Center node(s) for subnetwork extraction (vertex IDs or names)}

\item{order}{Integer specifying the order of neighbors to include}

\item{mode}{Character scalar, either "in", "out" or "all" for directed networks}
}
\value{
metanet
}
\description{
Wrapper around \code{igraph::make_ego_graph()} that ensures output retains
"metanet" and "igraph" class structure. Supports single or multiple center nodes.
}
\examples{
library(igraph)
c_net_plot(co_net)
c_net_plot(c_net_ego(co_net, "s__Kribbella_catacumbae"))
nodes <- c("s__Kribbella_catacumbae", "s__Verrucosispora_andamanensis")
c_net_plot(c_net_ego(co_net, nodes))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{summary_module}
\alias{summary_module}
\title{Summary module index}
\usage{
summary_module(go_m, var = "v_class", module = "module", ...)
}
\arguments{
\item{go_m}{module metanet}

\item{var}{variable name}

\item{module}{which column name is module. default: "module"}

\item{...}{add}
}
\value{
ggplot
}
\description{
Summary module index
}
\examples{
data("c_net")
module_detect(co_net, n_node_in_module = 30) -> co_net_modu
summary_module(co_net_modu, var = "v_class", module = "module")
summary_module(co_net_modu, var = "Abundance", module = "module")
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{module_net}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{as.ggig}
\alias{as.ggig}
\title{Transfer an igraph object to a ggig}
\usage{
as.ggig(go, coors = NULL)
}
\arguments{
\item{go}{igraph or meatnet}

\item{coors}{coordinates for nodes,columns: name, X, Y}
}
\value{
ggig object
}
\description{
Transfer an igraph object to a ggig
}
\examples{
as.ggig(co_net, coors = c_net_layout(co_net)) -> ggig
plot(ggig)
as.ggig(multi1, coors = c_net_layout(multi1)) -> ggig
plot(ggig, labels_num = 0.3)
}
\seealso{
Other plot: 
\code{\link{c_net_plot}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{extract_sample_net}
\alias{extract_sample_net}
\title{Extract each sample network from the whole network}
\usage{
extract_sample_net(
  whole_net,
  otutab,
  threads = 1,
  save_net = FALSE,
  fast = TRUE,
  remove_negative = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{whole_net}{the whole network}

\item{otutab}{otutab, columns are samples, these columns will be extract}

\item{threads}{threads, default: 1}

\item{save_net}{should save these sub_nets? FALSE or a filename}

\item{fast}{less indexes for faster calculate ?}

\item{remove_negative}{remove negative edge or not? default: FALSE}

\item{verbose}{verbose}
}
\value{
a dataframe contains all sub_net parameters
}
\description{
Extract each sample network from the whole network
}
\examples{
data(otutab, package = "pcutils")
extract_sample_net(co_net, otutab) -> sub_net_pars
}
\seealso{
Other topological: 
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7.stability.R
\name{plot.robustness}
\alias{plot.robustness}
\title{Plot robustness}
\usage{
\method{plot}{robustness}(x, indexes = "Node_number", ...)
}
\arguments{
\item{x}{\code{robustness()} result (robustness object)}

\item{indexes}{indexes selected to show}

\item{...}{additional arguments for \code{\link[pcutils]{group_box}}}
}
\value{
a ggplot
}
\description{
Plot robustness
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MetaNet-package.R
\docType{package}
\name{MetaNet-package}
\alias{MetaNet}
\alias{MetaNet-package}
\title{MetaNet: Network Analysis for Omics Data}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Comprehensive network analysis package. Calculate correlation network fastly, accelerate lots of analysis by parallel computing. Support for multi-omics data, search sub-nets fluently. Handle bigger data, more than 10,000 nodes in each omics. Offer various layout method for multi-omics network and some interfaces to other software ('Gephi', 'Cytoscape', 'ggplot2'), easy to visualize. Provide comprehensive topology indexes calculation, including ecological network stability.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Asa12138/MetaNet}
  \item Report bugs at \url{https://github.com/Asa12138/MetaNet/issues}
}

}
\author{
\strong{Maintainer}: Chen Peng \email{bfzede@gmail.com} (\href{https://orcid.org/0000-0002-9449-7606}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{anno_edge}
\alias{anno_edge}
\title{Use dataframe to annotate edges of an igraph}
\usage{
anno_edge(go, anno_tab, verbose = TRUE)
}
\arguments{
\item{go}{metanet an igraph object}

\item{anno_tab}{a dataframe using to annotate (with rowname or a name column)}

\item{verbose}{logical}
}
\value{
a annotated igraph object
}
\description{
Use dataframe to annotate edges of an igraph
}
\examples{
data("c_net")
anno <- data.frame("from" = "s__Pelomonas_puraquae", "to" = "s__un_g__Rhizobium", new_atr = "new")
anno_edge(co_net, anno) -> anno_net
}
\seealso{
Other manipulate: 
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{g_layout_nice}
\alias{g_layout_nice}
\alias{get_big_lay_nice}
\alias{g_layout_circlepack}
\alias{g_layout_treemap}
\alias{g_layout_backbone}
\alias{g_layout_stress}
\title{Layout with group nicely}
\usage{
g_layout_nice(go, group = "module", mode = "circlepack", group_zoom = 1, ...)

get_big_lay_nice(go, group = "module", mode = "circlepack", ...)

g_layout_circlepack(go, group = "module", ...)

g_layout_treemap(go, group = "module", ...)

g_layout_backbone(go, group = "module", ...)

g_layout_stress(go, group = "module", ...)
}
\arguments{
\item{go}{igraph or metanet}

\item{group}{group name (default: module)}

\item{mode}{circlepack, treemap, backbone, stress}

\item{group_zoom}{zoom for each group}

\item{...}{add}
}
\value{
coors
}
\description{
Layout with group nicely
}
\examples{
\donttest{
data("c_net")
module_detect(co_net, method = "cluster_fast_greedy") -> co_net_modu
if (requireNamespace("ggraph")) {
  plot(co_net_modu, coors = g_layout_nice(co_net_modu, group = "module"))
  plot(co_net_modu, coors = g_layout_nice(co_net_modu, group = "module", mode = "treemap"))
}
}
}
\seealso{
Other g_layout: 
\code{\link{g_layout}()},
\code{\link{g_layout_poly_sector}()},
\code{\link{g_layout_polygon}()}
}
\concept{g_layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-1.layout.R
\name{g_layout_polygon}
\alias{g_layout_polygon}
\alias{g_layout_polyarc}
\alias{g_layout_polycircle}
\alias{g_layout_multi_layer}
\title{Layout with group as a polygon}
\usage{
g_layout_polygon(
  go,
  group = "v_group",
  group_order = NULL,
  group2 = NULL,
  group2_order = NULL,
  line_curved = 0.5
)

g_layout_polyarc(
  go,
  group = "v_group",
  group_order = NULL,
  group2 = NULL,
  group2_order = NULL,
  space = pi/4,
  scale_node_num = TRUE
)

g_layout_polycircle(
  go,
  group = "v_group",
  group_order = NULL,
  group2 = NULL,
  group2_order = NULL
)

g_layout_multi_layer(
  go,
  layout = igraph::in_circle(),
  group = "v_group",
  group_order = NULL,
  group2 = NULL,
  group2_order = NULL,
  scale_node_num = TRUE
)
}
\arguments{
\item{go}{igraph}

\item{group}{group name (default:v_group)}

\item{group_order}{group_order}

\item{group2}{group2 name, will order nodes in each group according to group2_order}

\item{group2_order}{group2_order}

\item{line_curved}{line_curved 0~1}

\item{space}{the space between each arc, default: pi/4}

\item{scale_node_num}{scale with the node number in each group}

\item{layout}{see method in \code{\link[MetaNet]{c_net_layout}}}
}
\value{
coors
}
\description{
Layout with group as a polygon

Layout with group as a polyarc

Layout with group as a polycircle

Layout with group as a multi_layer
}
\examples{
g_layout_polygon(multi1) -> oridata
c_net_plot(multi1, oridata)
g_layout_polyarc(multi1, group2 = "v_class", group2_order = c(LETTERS[4:1])) -> oridata
c_net_plot(multi1, oridata)
g_layout_polycircle(co_net2, group2 = "v_class") -> oridata
c_net_plot(co_net2, oridata)
g_layout_multi_layer(co_net2, group2 = "v_class") -> oridata
c_net_plot(co_net2, oridata)
}
\seealso{
Other g_layout: 
\code{\link{g_layout}()},
\code{\link{g_layout_nice}()},
\code{\link{g_layout_poly_sector}()}
}
\concept{g_layout}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{smallworldness}
\alias{smallworldness}
\title{Calculate small-world coefficient}
\usage{
smallworldness(go, reps = 99, threads = 1, verbose = TRUE)
}
\arguments{
\item{go}{igraph or metanet}

\item{reps}{simulation time}

\item{threads}{threads}

\item{verbose}{verbose}
}
\value{
number
}
\description{
Calculate small-world coefficient
}
\examples{
\donttest{
# set reps at least 99 when you run.
smallworldness(co_net, reps = 9)
}
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7.stability.R
\name{plot.vulnerability}
\alias{plot.vulnerability}
\title{Plot vulnerability}
\usage{
\method{plot}{vulnerability}(x, ...)
}
\arguments{
\item{x}{\code{vulnerability()} result (vulnerability object)}

\item{...}{add}
}
\value{
a ggplot
}
\description{
Plot vulnerability
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-1.build.R
\name{c_net_from_edgelist}
\alias{c_net_from_edgelist}
\title{Construct a network from edge_list dataframe}
\usage{
c_net_from_edgelist(
  edgelist,
  vertex_df = NULL,
  direct = FALSE,
  e_type = NULL,
  e_class = NULL
)
}
\arguments{
\item{edgelist}{first is source, second is target, others are annotation}

\item{vertex_df}{vertex metadata data.frame}

\item{direct}{logical}

\item{e_type}{set e_type}

\item{e_class}{set e_class}
}
\value{
metanet
}
\description{
Construct a network from edge_list dataframe
}
\examples{
data(edgelist)
edge_net <- c_net_from_edgelist(arc_count, vertex_df = arc_taxonomy)
edge_net <- c_net_set(edge_net, vertex_class = "Phylum", edge_width = "n")
c_net_plot(edge_net)
}
\seealso{
Other build: 
\code{\link{c_net_build}()},
\code{\link{c_net_set}()},
\code{\link{c_net_update}()},
\code{\link{multi_net_build}()}
}
\concept{build}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-3.RMT.R
\name{plot.rmt_res}
\alias{plot.rmt_res}
\title{Plot a rmt_res}
\usage{
\method{plot}{rmt_res}(x, ...)
}
\arguments{
\item{x}{rmt_res}

\item{...}{Additional arguments}
}
\value{
ggplot
}
\description{
Plot a rmt_res
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-1.build.R
\name{c_net_update}
\alias{c_net_update}
\alias{as.metanet}
\title{Update a metanet object or transform igraph object to metanet object}
\usage{
c_net_update(
  go,
  node_break = 5,
  edge_break = 5,
  initialize = FALSE,
  verbose = TRUE,
  uniq_v_class = FALSE
)
}
\arguments{
\item{go}{a metanet object or igraph object}

\item{node_break}{node_break if v_class is numeric, default: 5}

\item{edge_break}{edge_break if e_type is numeric, default: 5}

\item{initialize}{initialize?}

\item{verbose}{verbose?}

\item{uniq_v_class}{if TRUE, add prefix to v_class if multiple v_class belong to same v_group.}
}
\value{
metanet
}
\description{
Update a metanet object or transform igraph object to metanet object
}
\seealso{
Other build: 
\code{\link{c_net_build}()},
\code{\link{c_net_from_edgelist}()},
\code{\link{c_net_set}()},
\code{\link{multi_net_build}()}
}
\concept{build}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MetaNet-package.R
\name{show_MetaNet_logo}
\alias{show_MetaNet_logo}
\title{Show MetaNet logo}
\usage{
show_MetaNet_logo()
}
\value{
picture
}
\description{
Show MetaNet logo
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{nc}
\alias{nc}
\title{Calculate natural_connectivity}
\usage{
nc(p)
}
\arguments{
\item{p}{an igraph or metanet object}
}
\value{
natural_connectivity (numeric)
}
\description{
Calculate natural_connectivity
}
\examples{
igraph::make_ring(10) \%>\% nc()
}
\references{
\code{`nc` in `ggClusterNet`}
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{module_net}
\alias{module_net}
\title{Generate a n-modules network}
\usage{
module_net(
  module_number = 3,
  n_node_in_module = 30,
  intra_module_density = 0.3,
  inter_module_density = 0.01
)
}
\arguments{
\item{module_number}{number of modules}

\item{n_node_in_module}{number of nodes in each modules}

\item{intra_module_density}{intra_module_density, recommend bigger than 20*inter_module_density, default:0.3}

\item{inter_module_density}{inter_module_density, default:0.01}
}
\value{
n-modules metanet
}
\description{
this is just a random generation method, the module number of result is not exactly the module_number, you can change the inter_module_density and intra_module_density to get the proper result.
}
\examples{
g1 <- module_net()
get_n(g1)
plot(g1, mark_module = TRUE)
plot(g1, coors = g_layout(g1, zoom2 = 15))
plot(g1, coors = g_layout_polyarc(g1, group = "module"))
plot(g1, coors = g_layout_polygon(g1, group = "module"))
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_eigen}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-1.build.R
\name{c_net_build}
\alias{c_net_build}
\title{Construct a metanet from a corr object}
\usage{
c_net_build(
  corr,
  r_threshold = 0.6,
  p_threshold = 0.05,
  use_p_adj = TRUE,
  delete_single = TRUE
)
}
\arguments{
\item{corr}{corr object from \code{c_net_calculate()} or \code{read_corr()}.}

\item{r_threshold}{r_threshold (default: >0.6).}

\item{p_threshold}{p_threshold (default: <0.05).}

\item{use_p_adj}{use the p.adjust instead of p.value (default: TRUE), if p.adjust not in the corr object, use p.value.}

\item{delete_single}{should delete single vertexes?}
}
\value{
an metanet object
}
\description{
Construct a metanet from a corr object
}
\examples{
data("otutab", package = "pcutils")
t(otutab) -> totu
metadata[, 3:10] -> env
c_net_calculate(totu) -> corr
c_net_build(corr, r_threshold = 0.65) -> co_net

c_net_calculate(totu, env) -> corr2
c_net_build(corr2) -> co_net2
}
\seealso{
Other build: 
\code{\link{c_net_from_edgelist}()},
\code{\link{c_net_set}()},
\code{\link{c_net_update}()},
\code{\link{multi_net_build}()}
}
\concept{build}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{rand_net}
\alias{rand_net}
\title{Degree distribution comparison with random network}
\usage{
rand_net(go = go, plot = TRUE)
}
\arguments{
\item{go}{igraph object}

\item{plot}{plot or not}
}
\value{
ggplot
}
\description{
Degree distribution comparison with random network
}
\examples{
rand_net(co_net)
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{fit_power}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{anno_vertex}
\alias{anno_vertex}
\alias{anno_node}
\title{Use data.frame to annotate vertexes of metanet}
\usage{
anno_vertex(go, anno_tab, verbose = TRUE)
}
\arguments{
\item{go}{metanet object}

\item{anno_tab}{a dataframe using to annotate (with rowname or a "name" column)}

\item{verbose}{logical}
}
\value{
a annotated metanet object
}
\description{
Use data.frame to annotate vertexes of metanet
}
\examples{
data("c_net")
data("otutab", package = "pcutils")
anno_vertex(co_net, taxonomy)
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_e}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{plot_multi_nets}
\alias{plot_multi_nets}
\title{Batch drawing multiple network diagrams}
\usage{
plot_multi_nets(graph_ls, nrow = NULL, ncol = NULL, multi_params_list = NULL)
}
\arguments{
\item{graph_ls}{a list containing igraph objects}

\item{nrow}{nrow}

\item{ncol}{ncol}

\item{multi_params_list}{a list of parameters for each network}
}
\value{
No value
}
\description{
Batch drawing multiple network diagrams
}
\examples{
plot_multi_nets(list(co_net, co_net2),
  multi_params_list = list(
    list(vertex.color = "skyblue"),
    list(vertex.color = "green3")
  )
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{netD3plot}
\alias{netD3plot}
\title{plot use networkD3}
\usage{
netD3plot(go, v_class = "v_class", ...)
}
\arguments{
\item{go}{metanet}

\item{v_class}{which attributes use to be v_class}

\item{...}{see \code{\link[networkD3]{forceNetwork}}}
}
\value{
D3 plot
}
\description{
plot use networkD3
}
\examples{
data("c_net")
plot(co_net2)
if (requireNamespace("networkD3")) {
  netD3plot(co_net2)
}
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{c_net_plot}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{olympic_rings_net}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8.net_compare.R
\name{adjacency_similarity}
\alias{adjacency_similarity}
\title{Calculate Similarity Between Two Graphs via Adjacency Matrices}
\usage{
adjacency_similarity(g1, g2, method = "frobenius")
}
\arguments{
\item{g1}{An igraph object representing the first graph.}

\item{g2}{An igraph object representing the second graph.}

\item{method}{A character string specifying the similarity method:
"frobenius" (default) or "cosine".}
}
\value{
A numeric value between 0 (no similarity) and 1 (identical graphs).
}
\description{
Computes the similarity between two igraph objects using their adjacency matrices.
Supports Frobenius norm-based similarity and cosine similarity.
}
\examples{
library(igraph)
g1 <- graph_from_edgelist(matrix(c(1, 2, 2, 3), ncol = 2, byrow = TRUE), directed = FALSE)
g2 <- graph_from_edgelist(matrix(c(1, 2, 2, 4), ncol = 2, byrow = TRUE), directed = FALSE)
adjacency_similarity(g1, g2, method = "frobenius") # Output: 0.5
adjacency_similarity(g1, g2, method = "cosine") # Output: 0.5
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{df2net_tree}
\alias{df2net_tree}
\title{Transform a dataframe to a network edgelist.}
\usage{
df2net_tree(test, fun = sum)
}
\arguments{
\item{test}{df}

\item{fun}{default: sum}
}
\value{
metanet
}
\description{
Transform a dataframe to a network edgelist.
}
\examples{
data("otutab", package = "pcutils")
cbind(taxonomy, num = rowSums(otutab))[1:20, ] -> test
df2net_tree(test) -> ttt
plot(ttt)
if (requireNamespace("ggraph")) plot(ttt, coors = as_circle_tree())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{get_e}
\alias{get_e}
\title{Get edge information}
\usage{
get_e(go, index = NULL)
}
\arguments{
\item{go}{metanet object}

\item{index}{attribute name, default: NULL}
}
\value{
data.frame
}
\description{
Get edge information
}
\seealso{
Other manipulate: 
\code{\link{anno_edge}()},
\code{\link{anno_vertex}()},
\code{\link{c_net_annotate}()},
\code{\link{c_net_filter}()},
\code{\link{c_net_load}()},
\code{\link{c_net_save}()},
\code{\link{c_net_union}()},
\code{\link{get_n}()},
\code{\link{get_v}()},
\code{\link{is_metanet}()}
}
\concept{manipulate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_set}
\alias{c_net_set}
\title{Set basic attributes from totu table}
\usage{
c_net_set(
  go,
  ...,
  vertex_group = "v_group",
  vertex_class = "v_class",
  vertex_size = "size",
  edge_type = "e_type",
  edge_class = "e_class",
  edge_width = "width",
  node_break = 5,
  edge_break = 5,
  initialize = TRUE
)
}
\arguments{
\item{go}{metanet an igraph object}

\item{...}{some data.frames to annotate go}

\item{vertex_group}{choose which column to be vertex_group (map to vertex_shape)}

\item{vertex_class}{choose which column to be vertex_class (map to vertex_color)}

\item{vertex_size}{choose which column to be vertex_size (map to vertex_size)}

\item{edge_type}{choose which column to be edge_type (map to edge_color)}

\item{edge_class}{choose which column to be edge_class (map to edge_linetype)}

\item{edge_width}{choose which column to be edge_width (map to edge_width)}

\item{node_break}{node_break if v_class is numeric, default: 5}

\item{edge_break}{edge_break if e_type is numeric, default: 5}

\item{initialize}{initialize, default: TRUE}
}
\value{
a metanet object
}
\description{
Set basic attributes from totu table
}
\examples{
data("otutab", package = "pcutils")
t(otutab) -> totu
metadata[, 3:10] -> env

data("c_net")
co_net <- c_net_set(co_net, taxonomy, data.frame("Abundance" = colSums(totu)),
  vertex_class = "Phylum", vertex_size = "Abundance"
)
co_net2 <- c_net_set(co_net2, taxonomy, data.frame(name = colnames(env), env = colnames(env)),
  vertex_class = c("Phylum", "env")
)
co_net2 <- c_net_set(co_net2, data.frame("Abundance" = colSums(totu)), vertex_size = "Abundance")
}
\seealso{
Other build: 
\code{\link{c_net_build}()},
\code{\link{c_net_from_edgelist}()},
\code{\link{c_net_update}()},
\code{\link{multi_net_build}()}
}
\concept{build}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5.topological.R
\name{fit_power}
\alias{fit_power}
\title{Fit power-law distribution for an igraph}
\usage{
fit_power(go, p.value = FALSE)
}
\arguments{
\item{go}{igraph}

\item{p.value}{calculate p.value}
}
\value{
ggplot
}
\description{
Fit power-law distribution for an igraph
}
\examples{
fit_power(co_net)
}
\seealso{
Other topological: 
\code{\link{extract_sample_net}()},
\code{\link{get_group_skeleton}()},
\code{\link{links_stat}()},
\code{\link{nc}()},
\code{\link{net_par}()},
\code{\link{rand_net}()},
\code{\link{rand_net_par}()},
\code{\link{smallworldness}()}
}
\concept{topological}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{plot_corr_heatmap}
\alias{plot_corr_heatmap}
\title{Plot a correlation heatmap}
\usage{
plot_corr_heatmap(
  corr,
  show.p = FALSE,
  cluster_rows = TRUE,
  cluster_cols = TRUE,
  ...
)
}
\arguments{
\item{corr}{a corr object, must contain 'r' and 'p.value' matrices.}

\item{show.p}{whether to show p-values as significance stars on the heatmap.}

\item{cluster_rows}{whether to cluster rows.}

\item{cluster_cols}{whether to cluster columns.}

\item{...}{additional arguments passed to \code{pheatmap::pheatmap}.}
}
\value{
plot of the correlation heatmap.
}
\description{
Plot a correlation heatmap
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1.calculate.R
\name{p.adjust.table}
\alias{p.adjust.table}
\title{p.adjust apply on a correlation table (matrix or data.frame)}
\usage{
p.adjust.table(pp, method = "BH", mode = "all")
}
\arguments{
\item{pp}{table of p-values}

\item{method}{see \code{\link[stats]{p.adjust}}, default: "BH".}

\item{mode}{"all" for all values; "rows" adjust each row one by one; "columns" adjust each column one by one. Default: "all".}
}
\value{
a table of adjusted p-values
}
\description{
p.adjust apply on a correlation table (matrix or data.frame)
}
\examples{
matrix(abs(rnorm(100, 0.01, 0.1)), 10, 10) -> pp
p.adjust.table(pp, method = "BH", mode = "all") -> pp_adj
}
\seealso{
Other calculate: 
\code{\link{c_net_calculate}()},
\code{\link{cal_sim}()},
\code{\link{fast_cor}()},
\code{\link{read_corr}()}
}
\concept{calculate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3-4.other_plot.R
\name{olympic_rings_net}
\alias{olympic_rings_net}
\title{Plot olympic rings using network}
\usage{
olympic_rings_net()
}
\value{
network plot
}
\description{
Plot olympic rings using network
}
\examples{
olympic_rings_net()
}
\seealso{
Other plot: 
\code{\link{as.ggig}()},
\code{\link{c_net_plot}()},
\code{\link{input_cytoscape}()},
\code{\link{input_gephi}()},
\code{\link{netD3plot}()},
\code{\link{plot.ggig}()},
\code{\link{twocol_edgelist}()},
\code{\link{venn_net}()}
}
\concept{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2-2.manipulate.R
\name{c_net_difference}
\alias{c_net_difference}
\title{Difference two networks}
\usage{
c_net_difference(go1, go2, ...)
}
\arguments{
\item{go1}{metanet object}

\item{go2}{metanet object}

\item{...}{add}
}
\value{
metanet
}
\description{
Difference two networks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6.modules.R
\name{module_eigen}
\alias{module_eigen}
\alias{module_expression}
\title{Calculate the eigenvalue of each module and correlation of nodes and eigenvalue (node_eigen_cor).}
\usage{
module_eigen(go_m, totu, cor_method = "spearman")

module_expression(
  go_m,
  totu,
  group = NULL,
  r_threshold = 0.6,
  x_order = NULL,
  facet_param = NULL,
  plot_eigen = FALSE
)
}
\arguments{
\item{go_m}{module metanet}

\item{totu}{original abundance table used for module_eigen().}

\item{cor_method}{"pearson", "kendall", "spearman"}

\item{group}{group variable for totu}

\item{r_threshold}{the threshold for node_eigen_cor, default: 0.6.}

\item{x_order}{order the x axis.}

\item{facet_param}{parameters parse to \code{\link[ggplot2]{facet_wrap}}, e.g. nrow=2.}

\item{plot_eigen}{plot the eigen value line.}
}
\value{
module metanet with module_eigen
}
\description{
Calculate the eigenvalue of each module and correlation of nodes and eigenvalue (node_eigen_cor).

Plot the expression of each modules
}
\examples{
data("otutab", package = "pcutils")
t(otutab) -> totu
data("c_net")
module_detect(co_net, n_node_in_module = 30) -> co_net_modu
module_eigen(co_net_modu, totu) -> co_net_modu
module_expression(co_net_modu, totu)
}
\seealso{
Other module: 
\code{\link{filter_n_module}()},
\code{\link{get_community}()},
\code{\link{get_module}()},
\code{\link{get_module_eigen}()},
\code{\link{module_detect}()},
\code{\link{module_net}()},
\code{\link{summary_module}()},
\code{\link{to_module_net}()},
\code{\link{zp_analyse}()}
}
\concept{module}
